USE ManageSoft
SET QUOTED_IDENTIFIER ON
/*
 * $Id: DBMaintenance.sql 101389 2011-11-25 01:51:25Z rlowery $
 *
 * Procedure to delete old ManageSoft application usage
 * data from the ManageSoft database.
 *
 * Protected by US patent 6,006,034; patents pending.
 * Copyright 2011 Flexera Software LLC
 * Reproduction, adaptation, or translation without prior permission
 * is prohibited.
 */

IF EXISTS (
	SELECT * FROM dbo.sysobjects
	WHERE [name]= 'SoftwareUsageDeleteConditional' AND xtype = 'P'
)
	DROP PROCEDURE SoftwareUsageDeleteConditional
GO
CREATE PROCEDURE SoftwareUsageDeleteConditional
	@UsageDataObsoletePeriod int = NULL,		-- Number of weeks
	@UsageDataObsoleteScope varchar(4000) = NULL,
	@DNListingDelimiter varchar(1) = ';',
	@DNExactMatch int = 0
AS
	-- PRINT '@UsageDataObsoletePeriod='+STR(@UsageDataObsoletePeriod)
	-- PRINT '@UsageDataObsoleteScope='+@UsageDataObsoleteScope
	-- PRINT '@DNListingDelimiter='+@DNListingDelimiter
	-- PRINT '@DNExactMatch='+STR(@DNExactMatch)

	DECLARE @end AS INTEGER			-- index of next delimiter
	DECLARE @start AS INTEGER		-- index of current delimiter
	DECLARE @l_token AS VARCHAR(500)	-- token found
	DECLARE @tempIdx AS INTEGER		-- index to end of current token
	DECLARE @l_StartOfThisWeek AS DATETIME	-- Monday of the current week
	DECLARE @l_numWeeks AS INTEGER

	-- List of DNs this deletion should affect
	CREATE TABLE #TDNs(
		DN VARCHAR(500) COLLATE database_default DEFAULT(NULL), -- Date of the start of that week
	);

	-- Process @UsageDataObsoleteScope, which is a delimited string and
	-- populate the temporary table with the resulting DNs
	SET @end = CHARINDEX(@DNListingDelimiter, @UsageDataObsoleteScope)

	-- @end = NULL if there was a problem in delimiter or Scope
	IF (@end IS NOT NULL)
	BEGIN
		-- if 'abc' or 'abc;', ie the semi-colon is not at the
		-- start of the string.
		IF (@end = 0) AND LEN(@UsageDataObsoleteScope) > 0
		BEGIN
			-- Simple case, one string
			SET @UsageDataObsoleteScope = LTRIM(@UsageDataObsoleteScope)
			SET @UsageDataObsoleteScope = RTRIM(@UsageDataObsoleteScope)
			INSERT INTO #TDNs VALUES(@UsageDataObsoleteScope)
		END
		ELSE
			-- if ';;' or ';' or ';abc'
			IF (@end = 1)
				SET @start = @end + 1
		ELSE
			-- IF 'abc;'
			IF(@end > 1)
				SET @start = 1

		WHILE ((@end <> 0) AND (@end IS NOT NULL))
		BEGIN
			SET @end = CHARINDEX(
					@DNListingDelimiter,
					@UsageDataObsoleteScope,
					@start
				)

			IF (@end = 0 AND @start < LEN(@UsageDataObsoleteScope))
			OR @end > @start+1
			BEGIN
				IF (@end = 0)
					SET @tempIdx = LEN(@UsageDataObsoleteScope) + 1
				ELSE
					SET @tempIdx = @end

				SET @l_token = SUBSTRING(@UsageDataObsoleteScope, @start, @tempIdx - @start)
				SET @l_token = RTRIM(@l_token)
				SET @l_token = LTRIM(@l_token)
				INSERT INTO #TDNs VALUES(@l_token)
				SET @start = @end + 1
			END
			ELSE
				IF (@end <= (@start + 1)) -- ';;'
					SET @start = @end + 1
			ELSE
				BREAK
		END
	END

	SET @l_StartOfThisWeek = '1/1/1990 12:00:00 AM'
	SET @l_numWeeks = DATEDIFF(Week, @l_StartOfThisWeek, GETDATE())
	SET @l_StartOfThisWeek = DATEADD(Week, @l_numWeeks, @l_StartOfThisWeek)

	-- Delete anything older than the specified date
	DELETE SoftwareUsagePerWeek
	FROM (
		-- List of relevant ComputerIDs
		SELECT c.ComputerID
		FROM (
			-- List of relevant OUIDs
			SELECT s.OUID
			FROM	(
				SELECT	so.OrganizationID as OUID,
					CASE
					WHEN so.RDN IS NULL THEN	-- Unknown OU in this domain
						'<NULL>,'+sd.DN		-- or the universal domain
					WHEN so.RDN = '' THEN
						sd.DN			-- DomainDN for the domain
					ELSE
						so.RDN+','+sd.DN		-- Ordinary OU or container
					END AS OU
				FROM Organization AS so, Domain AS sd
				WHERE so.DomainID = sd.OrganizationID) as s, #TDNs AS d
			WHERE (s.OU LIKE d.DN AND @DNExactMatch = 0)
				OR (s.OU = d.DN AND @DNExactMatch = 1)

		) AS o, Computer AS c
		WHERE o.OUID = c.ComputerOUID
	) AS ci, SoftwareUsagePerWeek as su
	WHERE	su.ComputerID = ci.ComputerID
	  AND	StartOfWeek <= DATEADD(
					Week,
					-@UsageDataObsoletePeriod,
					@l_StartOfThisWeek
				)
GO

IF EXISTS (
	SELECT * FROM dbo.sysobjects
	WHERE [name]= 'SoftwareUsageDeleteOld' AND xtype = 'P'
)
	DROP PROCEDURE SoftwareUsageDeleteOld
GO
CREATE PROCEDURE SoftwareUsageDeleteOld
AS
	DECLARE @UsageDataObsoletePeriod AS int		-- Number of weeks
	DECLARE @UsageDataObsoleteScope AS varchar(4000)
	DECLARE @DNListingDelimiter AS varchar(1)
	DECLARE @DNExactMatch AS int

	SELECT @UsageDataObsoletePeriod = Value
		FROM DatabaseConfiguration
		WHERE Property = 'UsageDataObsoletePeriod'
	SELECT @UsageDataObsoleteScope = Value
		FROM DatabaseConfiguration
		WHERE Property = 'UsageDataObsoleteScope'
	SELECT @DNListingDelimiter = Value
		FROM DatabaseConfiguration
		WHERE Property = 'DNListingDelimiter'
	SELECT @DNExactMatch = Value
		FROM DatabaseConfiguration
		WHERE Property = 'DNExactMatch'

	IF @UsageDataObsoletePeriod IS NULL
		SELECT @UsageDataObsoletePeriod = 26
	IF @UsageDataObsoleteScope IS NULL
	OR @UsageDataObsoleteScope = ''
		SELECT @UsageDataObsoleteScope = '%'
	IF @DNListingDelimiter IS NULL
		SELECT @DNListingDelimiter = ';'
	IF @DNExactMatch IS NULL
		SELECT @DNExactMatch = 0

	EXEC SoftwareUsageDeleteConditional
		@UsageDataObsoletePeriod,
		@UsageDataObsoleteScope,
		@DNListingDelimiter,
		@DNExactMatch
GO

IF EXISTS (
	SELECT * FROM dbo.sysobjects
	WHERE [name]= 'HardwarePropertyDeleteUnused' AND xtype = 'P'
)
	DROP PROCEDURE HardwarePropertyDeleteUnused
GO
CREATE PROCEDURE HardwarePropertyDeleteUnused
AS
	DELETE unused
		FROM HardwareProperty AS unused
		WHERE NOT EXISTS(
			SELECT * FROM HardwareValue AS ref
			WHERE unused.HardwarePropertyID = ref.HardwarePropertyID
		)
GO

IF EXISTS (
	SELECT * FROM dbo.sysobjects
	WHERE [name]= 'InstallationHistoryDeleteOld' AND xtype = 'P'
)
	DROP PROCEDURE InstallationHistoryDeleteOld
GO
CREATE PROCEDURE InstallationHistoryDeleteOld
AS
	-- Keep InstallationHistory data for 13 weeks (3 months):
	DELETE InstallationHistory
	WHERE DATEDIFF(week, Reported, GETDATE()) > 13
GO

IF EXISTS (
	SELECT * FROM dbo.sysobjects
	WHERE [name]= 'PackagePathDeleteUnused' AND xtype = 'P'
)
	DROP PROCEDURE PackagePathDeleteUnused
GO
CREATE PROCEDURE PackagePathDeleteUnused
AS
	DELETE unused
		FROM PackagePath AS unused
		WHERE NOT EXISTS(
			SELECT * FROM PackageVersion AS ref
			WHERE unused.PackagePathID = ref.PackagePathID
		) AND NOT EXISTS (
			SELECT * FROM SoftwareOccurrence AS ref
			WHERE unused.PackagePathID = ref.PackagePathID
		) AND NOT EXISTS (
			SELECT * FROM PackageAllocation AS ref
			WHERE unused.PackagePathID = ref.PackagePathID
		) AND NOT EXISTS (
			SELECT * FROM PackageApplies AS ref
			WHERE unused.PackagePathID = ref.PackagePathID
		) AND NOT EXISTS (
			SELECT * FROM Task AS ref
			WHERE unused.PackagePathID = ref.PackagePathID
		)
		-- More clauses might be needed here for sub-product schemata
GO

IF EXISTS (
	SELECT * FROM dbo.sysobjects
	WHERE [name]= 'MediaDeleteUnused' AND xtype = 'P'
)
	DROP PROCEDURE MediaDeleteUnused
GO
CREATE PROCEDURE MediaDeleteUnused
AS
	DELETE unused
		FROM Media AS unused
		WHERE NOT EXISTS(
			SELECT * FROM MediaContainsPackageVersion AS ref
			WHERE unused.MediaID = ref.MediaID
		)
GO

IF EXISTS (
	SELECT * FROM dbo.sysobjects
	WHERE [name]= 'SoftwareVersionDeleteUnused' AND xtype = 'P'
)
	DROP PROCEDURE SoftwareVersionDeleteUnused
GO
CREATE PROCEDURE SoftwareVersionDeleteUnused
AS
	DECLARE @SoftwareToDelete table(SoftwareID int)

	INSERT INTO @SoftwareToDelete
	SELECT 	sv.SoftwareID
	FROM	SoftwareVersion as sv
	WHERE NOT EXISTS(
		SELECT * from SoftwareOccurrence AS so
		WHERE so.SoftwareID = sv.SoftwareID
	)
	AND NOT EXISTS(
		SELECT * from ProductContainsSoftware AS pc
		WHERE pc.SoftwareVersionID = sv.SoftwareID
	)
	AND NOT EXISTS(
		SELECT * from SoftwareUsagePerWeek AS su
		WHERE su.SoftwareID = sv.SoftwareID
	)
	AND NOT EXISTS(
		SELECT * from SoftwareFile AS sf
		WHERE sf.SoftwareID = sv.SoftwareID
	)

	DELETE sv
	FROM dbo.SoftwareVersion as sv
		join @SoftwareToDelete as d on d.SoftwareID = sv.SoftwareID
GO

IF EXISTS (
	SELECT * FROM dbo.sysobjects
	WHERE name= 'ReasonDeleteUnused' AND xtype = 'P'
)
	DROP PROCEDURE ReasonDeleteUnused
GO
CREATE PROCEDURE ReasonDeleteUnused
AS
	DELETE Reason
	WHERE NOT EXISTS(
		SELECT * FROM Installation AS i
		WHERE Reason.ReasonID = i.FailReasonID
	) AND NOT EXISTS(
		SELECT * FROM InstallationHistory AS ih
		WHERE Reason.ReasonID = ih.FailReasonID
	)
GO

IF EXISTS (
	SELECT * FROM dbo.sysobjects
	WHERE name= 'InstallationPostponementDeleteStale' AND xtype = 'P'
)
	DROP PROCEDURE InstallationPostponementDeleteStale
GO
CREATE PROCEDURE InstallationPostponementDeleteStale
AS
	-- Remove postponed installations where no corresponding policy exists
	DELETE	i
	FROM	InstallationPostponement AS i		-- An installation was postponed
	WHERE NOT EXISTS(				-- but isn't in policy any more
		SELECT * FROM PackageApplies pa, Member m
		WHERE	((
				i.TargetTypeID = pa.TargetTypeID
				AND i.TargetID = pa.TargetID
			) OR (
				pa.TargetTypeID = 3 -- RSOP Groups
				AND m.GroupID = pa.TargetID
				AND m.TargetTypeID = i.TargetTypeID
				AND m.TargetID = i.TargetID
			))
		  AND	i.PackagePathID = pa.PackagePathID
		  AND	i.PolicyGUID = pa.PolicyGUID
		  AND	i.AllocationDetailsID = pa.AllocationDetailsID
	)
GO

IF EXISTS (
	SELECT * FROM dbo.sysobjects
	WHERE [name]= 'SecurityEventDeleteConditional' AND xtype = 'P'
)
	DROP PROCEDURE SecurityEventDeleteConditional
GO
CREATE PROCEDURE SecurityEventDeleteConditional
	@SecurityEventObsoletePeriod int = NULL,		-- Number of weeks
	@SecurityEventObsoleteScope varchar(4000) = NULL,
	@DNListingDelimiter varchar(1) = ';',
	@DNExactMatch int = 0
AS
	-- PRINT '@UsageDataObsoletePeriod='+STR(@UsageDataObsoletePeriod)
	-- PRINT '@UsageDataObsoleteScope='+@UsageDataObsoleteScope
	-- PRINT '@DNListingDelimiter='+@DNListingDelimiter
	-- PRINT '@DNExactMatch='+STR(@DNExactMatch)

	DECLARE @end AS INTEGER			-- index of next delimiter
	DECLARE @start AS INTEGER		-- index of current delimiter
	DECLARE @l_token AS VARCHAR(500)	-- token found
	DECLARE @tempIdx AS INTEGER		-- index to end of current token
	DECLARE @l_numWeeks AS INTEGER

	-- List of DNs this deletion should affect
	CREATE TABLE #TDNs(
		DN VARCHAR(500) COLLATE database_default DEFAULT(NULL), -- Date of the start of that week
	);

	-- Process @UsageDataObsoleteScope, which is a delimited string and
	-- populate the temporary table with the resulting DNs
	SET @end = CHARINDEX(@DNListingDelimiter, @SecurityEventObsoleteScope)

	-- @end = NULL if there was a problem in delimiter or Scope
	IF (@end IS NOT NULL)
	BEGIN
		-- if 'abc' or 'abc;', ie the semi-colon is not at the
		-- start of the string.
		IF (@end = 0) AND LEN(@SecurityEventObsoleteScope) > 0
		BEGIN
			-- Simple case, one string
			SET @SecurityEventObsoleteScope = LTRIM(@SecurityEventObsoleteScope)
			SET @SecurityEventObsoleteScope = RTRIM(@SecurityEventObsoleteScope)
			INSERT INTO #TDNs VALUES(@SecurityEventObsoleteScope)
		END
		ELSE
			-- if ';;' or ';' or ';abc'
			IF (@end = 1)
				SET @start = @end + 1
		ELSE
			-- IF 'abc;'
			IF(@end > 1)
				SET @start = 1

		WHILE ((@end <> 0) AND (@end IS NOT NULL))
		BEGIN
			SET @end = CHARINDEX(
					@DNListingDelimiter,
					@SecurityEventObsoleteScope,
					@start
				)

			IF (@end = 0 AND @start < LEN(@SecurityEventObsoleteScope))
			OR @end > @start+1
			BEGIN
				IF (@end = 0)
					SET @tempIdx = LEN(@SecurityEventObsoleteScope) + 1
				ELSE
					SET @tempIdx = @end

				SET @l_token = SUBSTRING(@SecurityEventObsoleteScope, @start, @tempIdx - @start)
				SET @l_token = RTRIM(@l_token)
				SET @l_token = LTRIM(@l_token)
				INSERT INTO #TDNs VALUES(@l_token)
				SET @start = @end + 1
			END
			ELSE
				IF (@end <= (@start + 1)) -- ';;'
					SET @start = @end + 1
			ELSE
				BREAK
		END
	END

	-- Delete anything older than the specified date
	DELETE DeviceEvent
	FROM (
		-- List of relevant ComputerIDs
		SELECT c.ComputerID
		FROM (
			-- List of relevant OUIDs
			SELECT s.OUID
			FROM	(
				SELECT	so.OrganizationID as OUID,
					CASE
					WHEN so.RDN IS NULL THEN	-- Unknown OU in this domain
						'<NULL>,'+sd.DN		-- or the universal domain
					WHEN so.RDN = '' THEN
						sd.DN			-- DomainDN for the domain
					ELSE
						so.RDN+','+sd.DN		-- Ordinary OU or container
					END AS OU
				FROM Organization AS so, Domain AS sd
				WHERE so.DomainID = sd.OrganizationID) as s, #TDNs AS d
			WHERE (s.OU LIKE d.DN AND @DNExactMatch = 0)
				OR (s.OU = d.DN AND @DNExactMatch = 1)

		) AS o, Computer AS c
		WHERE o.OUID = c.ComputerOUID
	) AS ci, DeviceEvent as de
	WHERE	de.ComputerID = ci.ComputerID
	  AND	[DateTime] <= DATEADD(
					Week,
					-@SecurityEventObsoletePeriod,
					GETDATE()
				)

	DELETE SessionStateEvent
	FROM (
		-- List of relevant ComputerIDs
		SELECT c.ComputerID
		FROM (
			-- List of relevant OUIDs
			SELECT s.OUID
			FROM	(
				SELECT	so.OrganizationID as OUID,
					CASE
					WHEN so.RDN IS NULL THEN	-- Unknown OU in this domain
						'<NULL>,'+sd.DN		-- or the universal domain
					WHEN so.RDN = '' THEN
						sd.DN			-- DomainDN for the domain
					ELSE
						so.RDN+','+sd.DN		-- Ordinary OU or container
					END AS OU
				FROM Organization AS so, Domain AS sd
				WHERE so.DomainID = sd.OrganizationID) as s, #TDNs AS d
			WHERE (s.OU LIKE d.DN AND @DNExactMatch = 0)
				OR (s.OU = d.DN AND @DNExactMatch = 1)

		) AS o, Computer AS c
		WHERE o.OUID = c.ComputerOUID
	) AS ci, SessionStateEvent as sse
	WHERE	sse.ComputerID = ci.ComputerID
	  AND	[DateTime] <= DATEADD(
					Week,
					-@SecurityEventObsoletePeriod,
					GETDATE()
				)


	DELETE GeneralSecurityEvent
	FROM (
		-- List of relevant ComputerIDs
		SELECT c.ComputerID
		FROM (
			-- List of relevant OUIDs
			SELECT s.OUID
			FROM	(
				SELECT	so.OrganizationID as OUID,
					CASE
					WHEN so.RDN IS NULL THEN	-- Unknown OU in this domain
						'<NULL>,'+sd.DN		-- or the universal domain
					WHEN so.RDN = '' THEN
						sd.DN			-- DomainDN for the domain
					ELSE
						so.RDN+','+sd.DN		-- Ordinary OU or container
					END AS OU
				FROM Organization AS so, Domain AS sd
				WHERE so.DomainID = sd.OrganizationID) as s, #TDNs AS d
			WHERE (s.OU LIKE d.DN AND @DNExactMatch = 0)
				OR (s.OU = d.DN AND @DNExactMatch = 1)

		) AS o, Computer AS c
		WHERE o.OUID = c.ComputerOUID
	) AS ci, GeneralSecurityEvent as gse
	WHERE	gse.ComputerID = ci.ComputerID
	  AND	[DateTime] <= DATEADD(
					Week,
					-@SecurityEventObsoletePeriod,
					GETDATE()
				)
GO

IF EXISTS (
	SELECT * FROM dbo.sysobjects
	WHERE [name]= 'SecurityEventDeleteOld' AND xtype = 'P'
)
	DROP PROCEDURE SecurityEventDeleteOld
GO
CREATE PROCEDURE SecurityEventDeleteOld
AS
	DECLARE @SecurityEventObsoletePeriod AS int		-- Number of weeks
	DECLARE @SecurityEventObsoleteScope AS varchar(4000)
	DECLARE @DNListingDelimiter AS varchar(1)
	DECLARE @DNExactMatch AS int

	SELECT @SecurityEventObsoletePeriod = Value
		FROM DatabaseConfiguration
		WHERE Property = 'SecurityEventObsoletePeriod'
	SELECT @SecurityEventObsoleteScope = Value
		FROM DatabaseConfiguration
		WHERE Property = 'SecurityEventObsoleteScope'
	SELECT @DNListingDelimiter = Value
		FROM DatabaseConfiguration
		WHERE Property = 'DNListingDelimiter'
	SELECT @DNExactMatch = Value
		FROM DatabaseConfiguration
		WHERE Property = 'DNExactMatch'

	IF @SecurityEventObsoletePeriod IS NULL
		SELECT @SecurityEventObsoletePeriod = 4
	IF @SecurityEventObsoleteScope IS NULL
	OR @SecurityEventObsoleteScope = ''
		SELECT @SecurityEventObsoleteScope = '%'
	IF @DNListingDelimiter IS NULL
		SELECT @DNListingDelimiter = ';'
	IF @DNExactMatch IS NULL
		SELECT @DNExactMatch = 0

	EXEC SecurityEventDeleteConditional
		@SecurityEventObsoletePeriod,
		@SecurityEventObsoleteScope,
		@DNListingDelimiter,
		@DNExactMatch
GO

IF EXISTS (
	SELECT * FROM dbo.sysobjects
	WHERE [name]= 'ServiceProviderDeleteConditional' AND xtype = 'P'
)
	DROP PROCEDURE ServiceProviderDeleteConditional
GO
CREATE PROCEDURE ServiceProviderDeleteConditional
	@ServiceProviderObsoletePeriod int = NULL		-- Number of weeks
AS

	DELETE ServiceProvider
	FROM ServiceProvider AS sp
		INNER JOIN InventoryReport AS ir
			ON sp.ComputerID = ir.ComputerID
	WHERE DATEADD(week, @ServiceProviderObsoletePeriod, LastInventoryDate) < ir.ServicesDate

GO

IF EXISTS (
	SELECT * FROM dbo.sysobjects
	WHERE [name]= 'ServiceProviderDeleteOld' AND xtype = 'P'
)
	DROP PROCEDURE ServiceProviderDeleteOld
GO
CREATE PROCEDURE ServiceProviderDeleteOld
AS

	-- For more complex conditional deletes, add extra parameters here

	DECLARE @ServiceProviderObsoletePeriod AS int		-- Number of weeks

	SELECT @ServiceProviderObsoletePeriod = Value
		FROM DatabaseConfiguration
		WHERE Property = 'ServiceProviderObsoletePeriod'

	IF @ServiceProviderObsoletePeriod IS NULL
		SELECT @ServiceProviderObsoletePeriod = 4
	ELSE
		PRINT 'ServiceProviderObsoletePeriod is not null'
	

	EXEC ServiceProviderDeleteConditional
		@ServiceProviderObsoletePeriod
GO

IF EXISTS (
	SELECT * FROM dbo.sysobjects
	WHERE name= 'SoftwareFilePathDeleteUnused' AND xtype = 'P'
)
	DROP PROCEDURE SoftwareFilePathDeleteUnused
GO
CREATE PROCEDURE SoftwareFilePathDeleteUnused
AS
	DELETE SoftwareFilePath
	WHERE NOT EXISTS(
		SELECT * FROM SoftwareFile
		WHERE SoftwareFilePath.SoftwareFilePathID = SoftwareFile.SoftwareFilePathID
	)
GO

IF EXISTS (
	SELECT * FROM dbo.sysobjects
	WHERE name= 'SoftwareFileNameDeleteUnused' AND xtype = 'P'
)
	DROP PROCEDURE SoftwareFileNameDeleteUnused
GO
CREATE PROCEDURE SoftwareFileNameDeleteUnused
AS
	DELETE SoftwareFileName
	WHERE NOT EXISTS (
		SELECT * FROM SoftwareFile
		WHERE SoftwareFileName.SoftwareFileNameID = SoftwareFile.SoftwareFileNameID
	) AND NOT EXISTS (
		SELECT * FROM SoftwareFileUsage
		WHERE SoftwareFileName.SoftwareFileNameID = SoftwareFileUsage.SoftwareFileNameID
	)
GO

IF EXISTS (
	SELECT * FROM dbo.sysobjects
	WHERE [name]= 'DBMaintenance' AND xtype = 'P'
)
	DROP PROCEDURE DBMaintenance
GO
CREATE PROCEDURE DBMaintenance
AS
	-- Delete old SoftwareUsage data according to the conditions defined
	-- in variables in the DatabaseConfiguration table
	EXEC SoftwareUsageDeleteOld

	-- Delete any HardwareProperty entries for which no HardwareValues exist
	EXEC HardwarePropertyDeleteUnused

	-- Delete InstallationHistory data which is too old:
	EXEC InstallationHistoryDeleteOld

	-- Delete reason rows that aren't used any more:
	EXEC ReasonDeleteUnused

	-- Delete unused PackagePath values:
	-- Not safe to do this because of (unconstrained) references
	-- from sub-product schemata:
	-- EXEC PackagePathDeleteUnused

	-- Delete unused SoftwareVersion entries:
	EXEC SoftwareVersionDeleteUnused

	-- Delete unused Media values:
	EXEC MediaDeleteUnused

	-- If we have postponement records no longer in policy, delete them
	EXEC InstallationPostponementDeleteStale

	-- Delete old security events from DeviceEvent, SessionStateEvent and GeneralSecurityEvent tables
	EXEC SecurityEventDeleteOld

	-- Delete old ServiceProvider data according to the conditions defined
	-- in variables in the DatabaseConfiguration table
	EXEC ServiceProviderDeleteOld
	
	EXEC SoftwareFilePathDeleteUnused

	EXEC SoftwareFileNameDeleteUnused	
	
	-- Modification to extend Maintenance task about neccessary steps (SQL Server Agent performing these too and can be deactivated)
	EXEC UpdateNetworkDevices

	EXEC PopulateNetworkDevices

	EXEC PolicyInstallationUpdate

	EXEC RefreshBundleAllocation

	IF EXISTS(Select * From DatabaseConfiguration Where Property like 'WiDepSchemaVersion')
	BEGIN
		DELETE FROM HardwareAssessmentCache

		EXEC WiMiValidateNUpdateAssessmentCache '%', 1

		EXEC WiMiValidateNUpdateAssessmentCache NULL, 0
	END
GO
/*
 * $Id: DiscoveryJobs.sql 101389 2011-11-25 01:51:25Z rlowery $
 *
 * Description:
 *
 * SQL Script to add ManageSoft discovery scheduled
 * tasks to Microsoft SQL Server.
 *
 * The steps performed by this script are:
 * 1. Add a SQL job to call UpdateNetworkDevices
 *
 * Protected by US patent 6,006,034; patents pending.
 * Copyright 2011 Flexera Software LLC
 * Reproduction, adaptation, or translation without prior permission
 * is prohibited.
 *
 */

-- Use MSDB database
USE MSDB
GO

DECLARE @mgsDatabaseCreateName nvarchar(256)
DECLARE @jobName nvarchar(256)
DECLARE @JobID BINARY(16)  

-- This line gets replaced in mgsDatabaseCreate with the appropriate database name.
SET @mgsDatabaseCreateName = 'ManageSoft'

IF EXISTS(SELECT * FROM sysjobs
	WHERE name = 'Update ManageSoft discovery information'
)
	EXEC sp_delete_job @job_name = 'Update ManageSoft discovery information'

IF EXISTS(SELECT * FROM sysjobs
	WHERE name = 'Populate ManageSoft discovery information'
)
	EXEC sp_delete_job @job_name = 'Populate ManageSoft discovery information'

SET @jobName = @mgsDatabaseCreateName + ' daily database maintenance'
IF EXISTS(SELECT * FROM sysjobs	WHERE name = @JobName)
	EXEC sp_delete_job @job_name = @jobName

-- Create daily maintenance scheduled job
IF NOT EXISTS(SELECT * FROM sysjobs	WHERE name = @JobName)
BEGIN
	-- Create SQL server job
	PRINT 'Creating a daily job to maintain the ' + @mgsDatabaseCreateName + ' database'
	EXEC sp_add_job
		@job_id = @JobID OUTPUT,
		@job_name = @jobName,
		@enabled = 1,
		@description = 'Updates Deployment Manager discovery and installation information',
		@category_name = 'Database Maintenance'

	-- Attach local server to job
	EXEC sp_add_jobserver @job_id = @JobID

	EXEC sp_add_jobstep @job_id = @JobID,
		@step_name = 'Update discovery from inventory',
		@subsystem = 'TSQL',
		@database_name = @mgsDatabaseCreateName,
		@command = 'EXEC UpdateNetworkDevices',
		@on_success_action = 3	-- goto next step

	EXEC sp_add_jobstep @job_id = @JobID,
		@step_name = 'Populate discovery from inventory',
		@subsystem = 'TSQL',
		@database_name = @mgsDatabaseCreateName,
		@command = 'EXEC PopulateNetworkDevices',
		@on_success_action = 3	-- goto next step

	EXEC sp_add_jobstep @job_id = @JobID,
		@step_name = 'Identify redundant and residual installation records',
		@subsystem = 'TSQL',
		@database_name = @mgsDatabaseCreateName,
		@command = 'EXEC PolicyInstallationUpdate',
		@on_success_action = 1	-- quit job with success

	-- Attach schedule to job
	EXEC sp_add_jobschedule @job_id = @JobID,
		@name = 'Daily database maintenance schedule',
		@enabled = 1,
		@freq_type = 4,			-- Daily
		@freq_interval = 1,		-- Every day
		@active_start_time = '043000'	-- 04:30 AM
END
ELSE
	PRINT 'Using existing job to maintain the ' + @mgsDatabaseCreateName + ' database daily'
GO

DECLARE @mgsDatabaseCreateName nvarchar(256)
DECLARE @jobName nvarchar(256)
DECLARE @JobID BINARY(16)  

-- This line gets replaced in mgsDatabaseCreate with the appropriate database name.
SET @mgsDatabaseCreateName = 'ManageSoft'

SET @jobName = 'Refresh BundleAllocation'

IF EXISTS(SELECT * FROM sysjobs
	WHERE name = @jobName
)
	EXEC sp_delete_job @job_name = @jobName


-- Create daily job to refresh BundleAllocation table
IF NOT EXISTS(SELECT * FROM sysjobs	WHERE name = @JobName)
BEGIN
	-- Create SQL server job
	PRINT 'Creating a daily job to refresh BundleAllocation table'
	EXEC sp_add_job
		@job_id = @JobID OUTPUT,
		@job_name = @jobName,
		@enabled = 1,
		@description = 'Refreshs BundleAllocation table',
		@category_name = 'Database Maintenance'

		-- Attach local server to job
	EXEC sp_add_jobserver @job_id = @JobID
	
	EXEC sp_add_jobstep @job_id = @JobID,
		@step_name = 'Identify redundant and residual installation records',
		@subsystem = 'TSQL',
		@database_name = @mgsDatabaseCreateName,
		@command = 'EXEC RefreshBundleAllocation',
		@on_success_action = 1	-- quit job with success
	
	EXEC sp_add_jobstep @job_id = @JobID,
			 @step_name=N'Refresh Bundle Filterung groups', 
             @step_id=1, 
             @cmdexec_success_code=0, 
             @on_success_action=3, 
             @on_fail_action=3, 
             @retry_attempts=0, 
             @retry_interval=0, 
             @os_run_priority=0, @subsystem=N'TSQL', 
             @command=N'EXEC RefreshBundleAllocationGroups 0', 
             @database_name=@mgsDatabaseCreateName, 
             @flags=0

		
		
	-- Attach schedule to job
	EXEC sp_add_jobschedule @job_id = @JobID,
		@name = 'Daily database maintenance schedule',
		@enabled = 1,
		@freq_type = 4,			-- Daily
		@freq_interval = 1,		-- Every day
		@active_start_time = '043000'	-- 04:30 AM
END
ELSE
	PRINT 'Using existing job to refresh BundleAllocation table'
GO

DECLARE @mgsDatabaseCreateName nvarchar(256)
DECLARE @jobId BINARY(16) 
 
SET @mgsDatabaseCreateName = 'ManageSoft'

IF EXISTS(SELECT * FROM sysjobs	WHERE name = N'Execute_FillReportTable_SoftwarePackageInstallationSummary')
	
	EXEC sp_delete_job @job_name = N'Execute_FillReportTable_SoftwarePackageInstallationSummary'



EXEC  msdb.dbo.sp_add_job @job_name=N'Execute_FillReportTable_SoftwarePackageInstallationSummary', 
		@enabled=1, 
		@notify_level_eventlog=0, 
		@notify_level_email=0, 
		@notify_level_netsend=0, 
		@notify_level_page=0, 
		@delete_level=0, 
		@description=N'generate data for FillReportTable_SoftwarePackageInstallationSummary', 
		@category_name=N'Data Collector', 
		@job_id = @jobId OUTPUT

EXEC msdb.dbo.sp_add_jobstep @job_id=@jobId, @step_name=N'GenerateData', 
		@step_id=1, 
		@cmdexec_success_code=0, 
		@on_success_action=1, 
		@on_success_step_id=0, 
		@on_fail_action=2, 
		@on_fail_step_id=0, 
		@retry_attempts=0, 
		@retry_interval=0, 
		@os_run_priority=0, @subsystem=N'TSQL', 
		@command=N'EXEC FillReportTable_SoftwarePackageInstallationSummary ''Computer'', ''%''', 
		@database_name=@mgsDatabaseCreateName, 
		@flags=0

EXEC msdb.dbo.sp_add_jobschedule @job_id=@jobId, @name=N'Start', 
		@enabled=1, 
		@freq_type=4, 
		@freq_interval=1, 
		@freq_subday_type=8, 
		@freq_subday_interval=3, 
		@freq_relative_interval=0, 
		@freq_recurrence_factor=0, 
		@active_start_date=20160114, 
		@active_end_date=99991231, 
		@active_start_time=60000, 
		@active_end_time=190000, 
		@schedule_uid=N'd3b2691d-fe72-4a55-82ad-5f6ae08824dd'

EXEC msdb.dbo.sp_add_jobserver @job_id = @jobId, @server_name = N'(local)'

GO	