/*
 * $Header$
 *
 * DESCRIPTION:
 *
 *	Schema for Group and related tables.
 *	-Creates required TABLES
 *	
 *
 * Protected by US patent 6,006,034; patents pending.
 * Copyright 2011 Flexera Software LLC
 * Reproduction, adaptation, or translation without prior permission
 * is prohibited.
 */


USE ManageSoft
GO

/* ----------------------------------------------------------------- */
PRINT '---------------------------------------------------------';
PRINT '--- Creating Windows Deployment: Rollout Group tables ---';
PRINT '---------------------------------------------------------';
/* ----------------------------------------------------------------- */

IF NOT EXISTS (
	SELECT *
	FROM INFORMATION_SCHEMA.TABLES
	WHERE TABLE_NAME = 'RolloutGroup'
)
BEGIN
	PRINT 'RolloutGroup Table:'
	CREATE TABLE RolloutGroup(

		-- Auto-generated Identity of RolloutGroup
		GroupID int NOT NULL IDENTITY
			CONSTRAINT PK_RolloutGroup PRIMARY KEY,			

		-- Rollout Group Name
		-- NOTE: Can have duplicate names though not under same parent
		-- This will have to be managed at resolving time
		GroupName nvarchar(256) NOT NULL, 			

		-- 0 = represents the Topmost RolloutGroup. Can have multiple groups
		-- at this level.	
		-- Ex: When source is Ghost; then Topmost Parent represents
		-- the GhostServer; under which the Root Groups are kept 		
		ParentID int NOT NULL DEFAULT(0),			

		--REVISIT:
		-- Can be NULL when it represents Topmost RolloutGroup
		-- (can have NULL if not a reference)
		--ParentID int NULL REFERENCES RolloutGroup(GroupID),		
				
		-- Represents the Source of RolloutGroup
		-- Ex: Ghost 7.5		
		Source nvarchar(64) NULL	
	);
	
END /* RolloutGroup */
ELSE
	PRINT 'Using existing RolloutGroup table'

GO

/* ----------------------------------------------------------------- */

IF NOT EXISTS (
	SELECT *
	FROM INFORMATION_SCHEMA.TABLES
	WHERE TABLE_NAME = 'RolloutGroupMember'
)
BEGIN
	PRINT 'RolloutGroupMember Table:'
	CREATE TABLE RolloutGroupMember(
			
		-- This RolloutGroup ....
		GroupID int NOT NULL
			CONSTRAINT FK_RolloutGroupMember_GroupID
				REFERENCES RolloutGroup(GroupID),
				
		-- ...contains Computer (ManageSoft ComputerID)
		-- And other types like User but *NOT* another 
		-- RolloutGroup (info in Group table)		
		MemberID int NOT NULL,
		
		-- type of the member - comes from TargetType table and
		-- must be a Computer (1) or in future User(2)
		MemberType int NOT NULL,
		
		--Unique as a Group can have a member listed only once
		CONSTRAINT UN_RolloutGroupMember UNIQUE(GroupID, MemberID, MemberType)
	);
	
END /* RolloutGroupMember */
ELSE
	PRINT 'Using existing RolloutGroupMember table'

GO
	
PRINT ' RolloutGroupMemberByType index'
IF NOT EXISTS (
    SELECT *
    FROM sysindexes
    WHERE name = 'RolloutGroupMemberByType'
)
	CREATE CLUSTERED INDEX RolloutGroupMemberByType
		ON RolloutGroupMember(MemberType, MemberID);

GO

/* ----------------------------------------------------------------- */

IF NOT EXISTS (
	SELECT *
	FROM INFORMATION_SCHEMA.TABLES
	WHERE TABLE_NAME = 'GhostServer'
)
BEGIN
	PRINT 'GhostServer Table:'
	CREATE TABLE GhostServer(

		GhostServerID int NOT NULL IDENTITY
			CONSTRAINT PK_GhostServer PRIMARY KEY,
		
		-- GhostServer is the *Parent* of root Rollout groups for a given 
		-- Ghost Console Server
		GroupID int NOT NULL 
			CONSTRAINT FK_GhostServer_GroupID
				REFERENCES RolloutGroup(GroupID),
		
		-- Name of Ghost Console Server
		-- This can be duplicates but unique in a domain
		ServerName nvarchar(256) NOT NULL, 
		
		--Name of the Domain to which Ghost Server belongs to		
		DomainName nvarchar(256) DEFAULT(NULL), 
		
		-- Network adapter physical address of the node
		-- Can be MAC address or token ring address
		PhysicalAddress varchar(64) NULL DEFAULT(NULL),
		
		-- ManageSoft Computer ID
		ComputerID int NULL DEFAULT(NULL),
		
		-- DateTime when Group XML was generated
		LastUpdateDateTime DATETIME NULL
	);
	
END /* GhostServer */
ELSE
	PRINT 'Using existing GhostServer table'

GO

/* ----------------------------------------------------------------- */
IF NOT EXISTS (
	SELECT *
	FROM INFORMATION_SCHEMA.TABLES
	WHERE TABLE_NAME = 'RolloutGroupToGhostComputer'
)
BEGIN
	PRINT 'RolloutGroupToGhostComputer Table:'
	CREATE TABLE RolloutGroupToGhostComputer(

		-- This Rollout Group ....
		GroupID int NOT NULL
			CONSTRAINT FK_RolloutGroupToGhostComputer_GroupID
				REFERENCES RolloutGroup(GroupID),				
		
		-- ...contains Computer
		--NOTE: Can be NULL if PhysicalAddress is not.				
		ComputerName nvarchar(64) NULL DEFAULT(NULL),
				
		-- Can be NULL if ComputerName is not.
		PhysicalAddress varchar(64) NULL DEFAULT(NULL),
		
		-- Is resolved into RolloutGroupMember table 
		-- with ManageSoft ComputerIDs 
		IsResolved int NOT NULL DEFAULT(0)
	);
	
END /* RolloutGroupToGhostComputer */
ELSE
	PRINT 'Using existing RolloutGroupToGhostComputer table'

GO

/* ----------------------------------------------------------------- */

IF NOT EXISTS (
   SELECT *
   FROM INFORMATION_SCHEMA.TABLES
   WHERE TABLE_NAME = 'HardwareMakeAndModel'
)
   BEGIN
   
   CREATE TABLE HardwareMakeAndModel
   (
		HWMAMID int IDENTITY NOT NULL
		CONSTRAINT PK_HardwareMakeAndModel PRIMARY KEY,
		HWMAMName	nvarchar(256) NULL,
		HWMAMMakeName nvarchar(64) NULL,
		HWMAMModelName nvarchar(64) NULL,
		HWMAMGUID binary(16) NOT NULL,
		HWMAMDescription nvarchar(1000) NULL,
		HWMAMPath nvarchar(256) NULL,
		HWMAMVersion nvarchar(64) NULL
	);
		
	END
	
ELSE
	PRINT 'Using existing HardwareMakeAndModel table'
	IF NOT EXISTS (SELECT * FROM INFORMATION_SCHEMA.COLUMNS WHERE COLUMN_NAME='HWMAMName' AND TABLE_NAME='HardwareMakeAndModel')
	BEGIN
		ALTER TABLE HardwareMakeAndModel
			ADD HWMAMName NVARCHAR(256) NULL DEFAULT(NULL)
	END	
GO

/* ----------------------------------------------------------------- */

IF NOT EXISTS (
   SELECT *
   FROM INFORMATION_SCHEMA.TABLES
   WHERE TABLE_NAME = 'HardwareProfile'
)
   BEGIN

	CREATE TABLE HardwareProfile
	(
		HWProfileID int IDENTITY NOT NULL
		CONSTRAINT PK_HardwareProfile PRIMARY KEY,
		HWProfileName nvarchar(64) NULL,
		HWProfileGUID binary(16) NOT NULL,
		HWProfileDescription nvarchar(1000) NULL,
		HWProfilePath nvarchar(256) NULL,
		HWProfileVersion nvarchar(64) NULL,
   
	);
   
	END
               
ELSE
	PRINT 'Using existing HardwareProfile table'
	
	IF EXISTS (SELECT * FROM INFORMATION_SCHEMA.COLUMNS WHERE COLUMN_NAME='HWProfileDescription' AND TABLE_NAME='HardwareProfile')
	BEGIN
		ALTER TABLE HardwareProfile
		ALTER COLUMN HWProfileDescription NVARCHAR(1000)
	END
GO

/* ----------------------------------------------------------------- */

IF NOT EXISTS (
   SELECT *
   FROM INFORMATION_SCHEMA.TABLES
   WHERE TABLE_NAME = 'HardwareDrivers'
)
   BEGIN

   CREATE TABLE HardwareDrivers
   (
		HWDriverID int IDENTITY NOT NULL
		CONSTRAINT PK_HardwareDrivers PRIMARY KEY,
		HWDriverName nvarchar(256) NULL,
		HWDriverGUID binary(16) NOT NULL,
		HWDriverPath nvarchar(250) NULL,
		HWDriverDescription nvarchar(1000) NULL,
		HWDriverArhitecture nvarchar(50) NULL,
		HWDriverSourcePath nvarchar(1000) NULL,
		HWDriverVersion nvarchar(64) NULL,
		HWDriverSize int NULL,
		HWDriverCreateDate smalldatetime NULL,
		HWDriverModifyDate smalldatetime NULL,
		HWDriverOSTypes nvarchar(200) NULL,
	
		HWDriverManufacturer nvarchar(200) NULL,
		HWDriverDate nvarchar(20) NULL,
		HWDriverClass nvarchar(200) NULL,
		HWDriverOSVersions nvarchar(200) NULL,
		HWDriverWHQLsigned bit NULL,
		HWDriverOSDType nvarchar(20) NULL,
		HWDriverDevices nvarchar(MAX) NULL,
		HWDriverActive bit NULL,
		HWDriverFile nvarchar(200) NULL,
		HWDriverMD5Sum nvarchar(32) NULL,
		
   );
   
   END
               
ELSE
   PRINT 'Using existing HardwareDrivers table'
	IF NOT EXISTS (SELECT * FROM INFORMATION_SCHEMA.COLUMNS WHERE COLUMN_NAME='HWDriverManufacturer' AND TABLE_NAME='HardwareDrivers')
	BEGIN
		ALTER TABLE HardwareDrivers
			ADD HWDriverManufacturer NVARCHAR(200) NULL DEFAULT(NULL)
	END	
	
	IF NOT EXISTS (SELECT * FROM INFORMATION_SCHEMA.COLUMNS WHERE COLUMN_NAME='HWDriverDate' AND TABLE_NAME='HardwareDrivers')
	BEGIN
		ALTER TABLE HardwareDrivers
			ADD HWDriverDate NVARCHAR(20) NULL DEFAULT(NULL)
	END	
	
	IF NOT EXISTS (SELECT * FROM INFORMATION_SCHEMA.COLUMNS WHERE COLUMN_NAME='HWDriverClass' AND TABLE_NAME='HardwareDrivers')
	BEGIN
		ALTER TABLE HardwareDrivers
			ADD HWDriverClass NVARCHAR(200) NULL DEFAULT(NULL)
	END
	
	IF NOT EXISTS (SELECT * FROM INFORMATION_SCHEMA.COLUMNS WHERE COLUMN_NAME='HWDriverOSVersions' AND TABLE_NAME='HardwareDrivers')
	BEGIN
		ALTER TABLE HardwareDrivers
			ADD HWDriverOSVersions NVARCHAR(200) NULL DEFAULT(NULL)
	END	
	
	IF NOT EXISTS (SELECT * FROM INFORMATION_SCHEMA.COLUMNS WHERE COLUMN_NAME='HWDriverWHQLsigned' AND TABLE_NAME='HardwareDrivers')
	BEGIN
		ALTER TABLE HardwareDrivers
			ADD HWDriverWHQLsigned BIT NULL DEFAULT(NULL)
	END	
	
	IF NOT EXISTS (SELECT * FROM INFORMATION_SCHEMA.COLUMNS WHERE COLUMN_NAME='HWDriverOSDType' AND TABLE_NAME='HardwareDrivers')
	BEGIN
		ALTER TABLE HardwareDrivers
			ADD HWDriverOSDType NVARCHAR(20) NULL DEFAULT(NULL)
	END	
	
	IF NOT EXISTS (SELECT * FROM INFORMATION_SCHEMA.COLUMNS WHERE COLUMN_NAME='HWDriverDevices' AND TABLE_NAME='HardwareDrivers')
	BEGIN
		ALTER TABLE HardwareDrivers
			ADD HWDriverDevices NVARCHAR(MAX) NULL DEFAULT(NULL)
	END
	
	IF NOT EXISTS (SELECT * FROM INFORMATION_SCHEMA.COLUMNS WHERE COLUMN_NAME='HWDriverActive' AND TABLE_NAME='HardwareDrivers')
	BEGIN
		ALTER TABLE HardwareDrivers
			ADD HWDriverActive bit NULL DEFAULT(NULL)
	END
	
	IF NOT EXISTS (SELECT * FROM INFORMATION_SCHEMA.COLUMNS WHERE COLUMN_NAME='HWDriverFile' AND TABLE_NAME='HardwareDrivers')
	BEGIN
		ALTER TABLE HardwareDrivers
			ADD HWDriverFile NVARCHAR(200) NULL DEFAULT(NULL)
	END
	
	IF EXISTS (SELECT * FROM INFORMATION_SCHEMA.COLUMNS WHERE COLUMN_NAME='HWDriverDescription' AND TABLE_NAME='HardwareDrivers')
	BEGIN
		ALTER TABLE HardwareDrivers
		ALTER COLUMN HWDriverDescription NVARCHAR(1000)
	END
	
	IF EXISTS (SELECT * FROM INFORMATION_SCHEMA.COLUMNS WHERE COLUMN_NAME='HWDriverArhitecture' AND TABLE_NAME='HardwareDrivers')
	BEGIN
		ALTER TABLE HardwareDrivers
		ALTER COLUMN HWDriverArhitecture NVARCHAR(50)
	END
	
	IF EXISTS (SELECT * FROM INFORMATION_SCHEMA.COLUMNS WHERE COLUMN_NAME='HWDriverSourcePath' AND TABLE_NAME='HardwareDrivers')
	BEGIN
		ALTER TABLE HardwareDrivers
		ALTER COLUMN HWDriverSourcePath NVARCHAR(1000)
	END
	
	IF EXISTS (SELECT * FROM INFORMATION_SCHEMA.COLUMNS WHERE COLUMN_NAME='HWDriverPath' AND TABLE_NAME='HardwareDrivers')
	BEGIN
		ALTER TABLE HardwareDrivers
		ALTER COLUMN HWDriverPath NVARCHAR(250)
	END
	
	IF NOT EXISTS (SELECT * FROM INFORMATION_SCHEMA.COLUMNS WHERE COLUMN_NAME='HWDriverMD5Sum' AND TABLE_NAME='HardwareDrivers')
	BEGIN
		ALTER TABLE HardwareDrivers
		ADD HWDriverMD5Sum NVARCHAR(32) NULL DEFAULT(NULL)
	END
GO
/* ----------------------------------------------------------------- */

IF NOT EXISTS (
   SELECT *
   FROM INFORMATION_SCHEMA.TABLES
   WHERE TABLE_NAME = 'HWDriversToHWProfile'
)
   BEGIN

   CREATE TABLE HWDriversToHWProfile(
	HWDriversToHWProfileID int IDENTITY NOT NULL
		CONSTRAINT PK_HWDriversToHWProfile PRIMARY KEY,
	HWProfileID int NULL
			CONSTRAINT FK_HWDriversToHWProfileToHardwareProfile_HWProfileID
				REFERENCES HardwareProfile(HWProfileID),
				
	HWDriverID int NULL
			CONSTRAINT FK_HWDriversToHWProfileToHardwareDriver_HWDriverID
				REFERENCES HardwareDrivers(HWDriverID),
 
	);
	
	END

               
ELSE
	PRINT 'Using existing HWDriversToHWProfile table'

GO
	PRINT ' HWDriversToHWProfile_HWProfileID index'
	
IF NOT EXISTS (
    SELECT *
    FROM sysindexes
    WHERE name = 'HWDriversToHWProfile_HWProfileID'
)
	CREATE INDEX HWDriversToHWProfile_HWProfileID
		ON HWDriversToHWProfile(HWProfileID);

GO

PRINT ' HWDriversToHWProfile_HWDriverID index'
IF NOT EXISTS (
    SELECT *
    FROM sysindexes
    WHERE name = 'HWDriversToHWProfile_HWDriverID'
)
	CREATE INDEX HWDriversToHWProfile_HWDriverID
		ON HWDriversToHWProfile(HWDriverID);

GO

/* ----------------------------------------------------------------- */
IF NOT EXISTS 
	(
		SELECT *
		FROM INFORMATION_SCHEMA.TABLES
		WHERE TABLE_NAME = 'HWProfileToHWMakeAndModel'
	)
	BEGIN
	CREATE TABLE HWProfileToHWMakeAndModel
	(
		HWProfileToHWMAMID int IDENTITY NOT NULL
		CONSTRAINT PK_HWProfilesToHWMakeAndModel PRIMARY KEY,
		HWMAMID int NULL
			CONSTRAINT FK_HWProfilesToHWMakeAndModelToHardwareMakeAndModel_HWMAMID
				REFERENCES HardwareMakeAndModel (HWMAMID),
		HWProfileID int NULL
			CONSTRAINT FK_HWProfilesToHWMakeAndModelToHardwareProfile_HWProfileID
				REFERENCES HardwareProfile (HWProfileID)
	  
	);
			
	END
	
ELSE
	PRINT 'Using existing HWProfileToHWMakeAndModel table'
	GO
	
/* ----------------------------------------------------------------- */

IF NOT EXISTS (
   SELECT *
   FROM INFORMATION_SCHEMA.TABLES
   WHERE TABLE_NAME = 'HWDriversToHWMakeAndModel'
)
	BEGIN
   
	CREATE TABLE HWDriversToHWMakeAndModel
	(
	HWDriversToHWMAMID int IDENTITY NOT NULL
	CONSTRAINT PK_HWDriversToHWMakeAndModel PRIMARY KEY,
	HWMAMID int NULL
		CONSTRAINT FK_HWDriversToHWMakeAndModelToHardwareMakeAndModel_HWMAMID
			REFERENCES HardwareMakeAndModel (HWMAMID),
	HWDriverID int NULL
		CONSTRAINT FK_HWDriversToHWMakeAndModelToHardwareDriver_HWDriverID
			REFERENCES HardwareDrivers (HWDriverID)
 	)
	
	END
ELSE
	PRINT 'Using existing HWDriversToHWMakeAndModel table'
	GO

/* ----------------------------------------------------------------- */

IF NOT EXISTS (
	SELECT *
	FROM INFORMATION_SCHEMA.TABLES
	WHERE TABLE_NAME = 'NameMap'
)
BEGIN
	PRINT 'NameMap Table:'
	CREATE TABLE NameMap(

		-- This name mapping ....
		ID int NOT NULL IDENTITY,
		
		-- old computer name
		Name nvarchar(64) NULL DEFAULT(NULL),

		-- old computer domain
		Domain int DEFAULT(NULL),

		-- new computer name
		NewName nvarchar(64) NULL DEFAULT(NULL),

		-- new computer domain
		NewDomain int DEFAULT(NULL),

		-- the name the comuter currently has
		CurrentID int NULL DEFAULT(NULL),
		
		-- the OU new computers are assigned into
		-- this is a foreign key onto the Organization table
		NewOU int NULL DEFAULT(NULL),
		
		-- the MACAddress of new computers. Existing ones will
		-- get their address from inventory.
		MACAddress nvarchar(32) NULL DEFAULT(NULL),
		
		-- the IP address of new computers
		IPAddress nvarchar(32) NULL DEFAULT(NULL),
				
		-- the ID from OSImage
		[OSImageID] [int] NULL DEFAULT(NULL)
		CONSTRAINT FK_NameMapToOSImage_OSImageID
				REFERENCES OSImage(OSImageID)
				ON DELETE SET NULL,
		-- the ID from Hardware Profile
		HWProfileID [int] NULL DEFAULT(NULL)
		CONSTRAINT FK_NameMapToHWProfile_HWProfileID
				REFERENCES HardwareProfile(HWProfileID)
				ON DELETE SET NULL,
		
		-- the BootMode
		BootMode nchar(4) NULL DEFAULT(NULL)
	);
			
END /* NameMap */
ELSE
	PRINT 'Using existing NameMap table'
	IF NOT EXISTS (SELECT * FROM INFORMATION_SCHEMA.COLUMNS WHERE COLUMN_NAME='NewOU' AND TABLE_NAME='NameMap')
	BEGIN
		ALTER TABLE NameMap
			ADD NewOU int NULL DEFAULT(NULL)
	END
	IF NOT EXISTS (SELECT * FROM INFORMATION_SCHEMA.COLUMNS WHERE COLUMN_NAME='MACAddress' AND TABLE_NAME='NameMap')
	BEGIN
        ALTER TABLE NameMap
                ADD MACAddress nvarchar(32) NULL DEFAULT(NULL)
	END
	IF NOT EXISTS (SELECT * FROM INFORMATION_SCHEMA.COLUMNS WHERE COLUMN_NAME='IPAddress' AND TABLE_NAME='NameMap')
	BEGIN
        ALTER TABLE NameMap
                ADD IPAddress nvarchar(32) NULL DEFAULT(NULL)
				
	END
	
	IF NOT EXISTS (SELECT * FROM INFORMATION_SCHEMA.COLUMNS WHERE COLUMN_NAME='OSImageID' AND TABLE_NAME='NameMap')
	BEGIN
        ALTER TABLE NameMap
                ADD OSImageID [int] NULL DEFAULT(NULL)
				CONSTRAINT FK_NameMapToOSImage_OSImageID
				REFERENCES OSImage(OSImageID)
				ON DELETE SET NULL
	END
	
	IF NOT EXISTS (SELECT * FROM INFORMATION_SCHEMA.COLUMNS WHERE COLUMN_NAME='HWProfileID' AND TABLE_NAME='NameMap')
	BEGIN
        ALTER TABLE NameMap
                ADD HWProfileID [int] NULL DEFAULT(NULL)
				CONSTRAINT FK_NameMapToHWProfile_HWProfileID
				REFERENCES HardwareProfile(HWProfileID)
				ON DELETE SET NULL
	END
	
	IF NOT EXISTS (SELECT * FROM INFORMATION_SCHEMA.COLUMNS WHERE COLUMN_NAME='BootMode' AND TABLE_NAME='NameMap')
	BEGIN
        ALTER TABLE NameMap
                ADD BootMode nchar(4) NULL DEFAULT(NULL)
				
	END
GO

PRINT ' NameMapNameDomain index'
IF NOT EXISTS (
    SELECT *
    FROM sysindexes
    WHERE name = 'NameMapNameDomain'
)
	CREATE INDEX NameMapNameDomain
		ON NameMap(name, domain);

GO

PRINT ' NameMapNewNameDomain index'
IF NOT EXISTS (
    SELECT *
    FROM sysindexes
    WHERE name = 'NameMapNewNameDomain'
)
	CREATE INDEX NameMapNewNameDomain
		ON NameMap(newName, newDomain);

GO

PRINT ' NameMapID index'
IF NOT EXISTS (
    SELECT *
    FROM sysindexes
    WHERE name = 'NameMapID'
)
	CREATE CLUSTERED INDEX NameMapID
		ON NameMap(CurrentId);

GO

PRINT ' NameMap_OSImageID index'
IF NOT EXISTS (
    SELECT *
    FROM sysindexes
    WHERE name = 'NameMap_OSImageID'
)
	CREATE INDEX NameMap_OSImageID
		ON NameMap(OSImageID);

GO

PRINT ' NameMap_HWProfileID index'
IF NOT EXISTS (
    SELECT *
    FROM sysindexes
    WHERE name = 'NameMap_HWProfileID'
)
	CREATE INDEX NameMap_HWProfileID
		ON NameMap(HWProfileID);

GO

IF NOT EXISTS (SELECT * FROM DatabaseConfiguration WHERE Property = 'WiDepSchemaVersion')
BEGIN
	INSERT INTO DatabaseConfiguration (Property, Value) VALUES ('WiDepSchemaVersion','11.0')
END
GO

/* ----------------------------------------------------------------- */