USE ManageSoft
GO

-- The SecPatch_AffectedProduct table contains the products and service packs
-- affected by a security patch.
IF NOT EXISTS (
	SELECT * FROM INFORMATION_SCHEMA.TABLES
	WHERE TABLE_NAME = N'SecPatch_AffectedProduct'
	AND TABLE_SCHEMA = 'dbo'
)
BEGIN
	CREATE TABLE [dbo].[SecPatch_AffectedProduct] (
		-- Primary key.
		AffectedProductID	int IDENTITY NOT NULL,
		-- Foreign key to the SecPatch_Product table.
		ProductID		int NOT NULL,
		-- Foreign key to the SecPatch_ServicePack table. Maps to service pack id.
		FixedInSP		int NOT NULL,
		-- Foreign key to the SecPatch_Patch table.
		PatchID			int NOT NULL,
		CONSTRAINT PK_AffectedProduct
			PRIMARY KEY CLUSTERED(AffectedProductID),
		CONSTRAINT UQ_AffectedProduct
			UNIQUE NONCLUSTERED(ProductID, PatchID)
	);

	CREATE NONCLUSTERED INDEX AffectedProductByPatchID
		ON dbo.SecPatch_AffectedProduct(PatchID)

END
ELSE BEGIN
	PRINT 'Using existing SecPatch_AffectedProduct table'
	IF EXISTS (
		SELECT * FROM sysindexes
		WHERE name = N'PK_AffectedProduct'
		AND ((status & 6162) <> 2066)
	)
	BEGIN
		PRINT 'Recreating existing PK_AffectedProduct index'
		ALTER TABLE [dbo].[SecPatch_AffectedProduct]
		DROP CONSTRAINT PK_AffectedProduct
	END

	IF EXISTS (
		SELECT * FROM sysindexes
		WHERE name = N'AffectedProductByPatchID'
		AND ((status & 6162) <> 0)
	)
	BEGIN
		PRINT 'Recreating existing AffectedProductByPatchID index'
		DROP INDEX AffectedProductByPatchID
		ON [dbo].[SecPatch_AffectedProduct]
	END

	IF EXISTS (
		SELECT * FROM sysindexes
		WHERE name = N'UQ_AffectedProduct'
		AND ((status & 6162) <> 4098)
	)
	BEGIN
		PRINT 'Recreating existing UQ_AffectedProduct index'
		ALTER TABLE [dbo].[SecPatch_AffectedProduct]
		DROP CONSTRAINT UQ_AffectedProduct
	END

	IF NOT EXISTS (
		SELECT * FROM sysindexes
		WHERE name = N'PK_AffectedProduct'
	)
		ALTER TABLE [dbo].[SecPatch_AffectedProduct]
		ADD CONSTRAINT PK_AffectedProduct
			PRIMARY KEY CLUSTERED(AffectedProductID)

	IF NOT EXISTS (
		SELECT * FROM sysindexes
		WHERE name = N'AffectedProductByPatchID'
	)
		CREATE NONCLUSTERED INDEX AffectedProductByPatchID
			ON dbo.SecPatch_AffectedProduct(PatchID)

	IF NOT EXISTS (
		SELECT * FROM sysindexes
		WHERE name = N'UQ_AffectedProduct'
	)
		ALTER TABLE [dbo].[SecPatch_AffectedProduct]
		ADD CONSTRAINT UQ_AffectedProduct
			UNIQUE NONCLUSTERED(ProductID, PatchID)

	END
GO

-- The SecPatch_AffectedServicePack table contains the service packs that a
-- product applies to for a security patch.
IF NOT EXISTS (
	SELECT * FROM INFORMATION_SCHEMA.TABLES
	WHERE TABLE_NAME = N'SecPatch_AffectedServicePack'
	AND TABLE_SCHEMA = 'dbo'
)
BEGIN
	CREATE TABLE [dbo].[SecPatch_AffectedServicePack] (
		-- Primary key.
		AffectedServicePackID	int IDENTITY NOT NULL,
		-- Foreign key to the SecPatch_AffectedProduct table.
		AffectedProductID	int NOT NULL,
		-- Foreign key to the SecPatch_ServicePack table.
		ServicePackID		int NOT NULL,
		CONSTRAINT FK_AffectedServicePack_AffectedProduct
			FOREIGN KEY(AffectedProductID)
			REFERENCES SecPatch_AffectedProduct(AffectedProductID) ON DELETE CASCADE,
		CONSTRAINT PK_AffectedServicePack
			PRIMARY KEY NONCLUSTERED(AffectedServicePackID),
		CONSTRAINT UQ_AffectedServicePack
			UNIQUE CLUSTERED(AffectedProductID, ServicePackID)
	);

	CREATE NONCLUSTERED INDEX SecPatch_AffectedServicePackByServicePackID
		ON dbo.SecPatch_AffectedServicePack(ServicePackID)

END
ELSE BEGIN
	PRINT 'Using existing SecPatch_AffectedServicePack table'
	IF EXISTS (
		SELECT * FROM sysindexes
		WHERE name = N'UQ_AffectedServicePack'
		AND ((status & 6162) <> 4114)
	)
	BEGIN
		PRINT 'Recreating existing UQ_AffectedServicePack index'
		ALTER TABLE [dbo].[SecPatch_AffectedServicePack]
		DROP CONSTRAINT UQ_AffectedServicePack
	END

	IF EXISTS (
		SELECT * FROM sysindexes
		WHERE name = N'PK_AffectedServicePack'
		AND ((status & 6162) <> 2050)
	)
	BEGIN
		PRINT 'Recreating existing PK_AffectedServicePack index'
		ALTER TABLE [dbo].[SecPatch_AffectedServicePack]
		DROP CONSTRAINT PK_AffectedServicePack
	END

	IF EXISTS (
		SELECT * FROM sysindexes
		WHERE name = N'SecPatch_AffectedServicePackByServicePackID'
		AND ((status & 6162) <> 0)
	)
	BEGIN
		PRINT 'Recreating existing SecPatch_AffectedServicePackByServicePackID index'
		DROP INDEX SecPatch_AffectedServicePackByServicePackID
		ON [dbo].[SecPatch_AffectedServicePack]
	END

	IF NOT EXISTS (
		SELECT * FROM sysindexes
		WHERE name = N'UQ_AffectedServicePack'
	)
		ALTER TABLE [dbo].[SecPatch_AffectedServicePack]
		ADD CONSTRAINT UQ_AffectedServicePack
			UNIQUE CLUSTERED(AffectedProductID, ServicePackID)

	IF NOT EXISTS (
		SELECT * FROM sysindexes
		WHERE name = N'PK_AffectedServicePack'
	)
		ALTER TABLE [dbo].[SecPatch_AffectedServicePack]
		ADD CONSTRAINT PK_AffectedServicePack
			PRIMARY KEY NONCLUSTERED(AffectedServicePackID)

	IF NOT EXISTS (
		SELECT * FROM sysindexes
		WHERE name = N'SecPatch_AffectedServicePackByServicePackID'
	)
		CREATE NONCLUSTERED INDEX SecPatch_AffectedServicePackByServicePackID
			ON dbo.SecPatch_AffectedServicePack(ServicePackID)

	IF EXISTS(
		SELECT *
		FROM INFORMATION_SCHEMA.TABLES
		WHERE TABLE_NAME = 'SecPatch_AffectedServicePack'
		AND TABLE_SCHEMA = 'dbo'
	)
	AND EXISTS(
		SELECT *
		FROM INFORMATION_SCHEMA.TABLES
		WHERE TABLE_NAME = 'SecPatch_AffectedProduct'
		AND TABLE_SCHEMA = 'dbo'
	)	
	AND NOT EXISTS (
		SELECT *
		FROM INFORMATION_SCHEMA.CONSTRAINT_COLUMN_USAGE
		WHERE CONSTRAINT_NAME = 'FK_AffectedServicePack_AffectedProduct'
	)
	BEGIN
		PRINT 'Create FK_AffectedServicePack_AffectedProduct constraint'
	
		ALTER TABLE [dbo].[SecPatch_AffectedServicePack]
			ADD CONSTRAINT FK_AffectedServicePack_AffectedProduct
			FOREIGN KEY(AffectedProductID)
			REFERENCES SecPatch_AffectedProduct(AffectedProductID) ON DELETE CASCADE
	END
END
GO

-- The SecPatch_Comment table contains descriptive text about security patches and bulletins.
IF NOT EXISTS (
	SELECT * FROM INFORMATION_SCHEMA.TABLES
	WHERE TABLE_NAME = N'SecPatch_Comment'
	AND TABLE_SCHEMA = 'dbo'
)
BEGIN
	CREATE TABLE [dbo].[SecPatch_Comment] (
		-- Primary key.
		CommentID		int NOT NULL,
		-- Text describing a security patch or bulletin.
		Text			ntext NOT NULL,
		CONSTRAINT PK_Comment
			PRIMARY KEY CLUSTERED(CommentID)
	);

END
ELSE BEGIN
	PRINT 'Using existing SecPatch_Comment table'
	IF EXISTS (
		SELECT * FROM sysindexes
		WHERE name = N'PK_Comment'
		AND ((status & 6162) <> 2066)
	)
	BEGIN
		PRINT 'Recreating existing PK_Comment index'
		ALTER TABLE [dbo].[SecPatch_Comment]
		DROP CONSTRAINT PK_Comment
	END

	IF NOT EXISTS (
		SELECT * FROM sysindexes
		WHERE name = N'PK_Comment'
	)
		ALTER TABLE [dbo].[SecPatch_Comment]
		ADD CONSTRAINT PK_Comment
			PRIMARY KEY CLUSTERED(CommentID)

	END
GO

-- The SecPatch_Location table contains URLs and file and registry locations used
-- by security patches and bulletins and is referred to by file and registry entries within a patch.
IF NOT EXISTS (
	SELECT * FROM INFORMATION_SCHEMA.TABLES
	WHERE TABLE_NAME = N'SecPatch_Location'
	AND TABLE_SCHEMA = 'dbo'
)
BEGIN
	CREATE TABLE [dbo].[SecPatch_Location] (
		-- Primary key.
		LocationID		int NOT NULL,
		-- URL to a file or registry location used by a security patch or bulletin.
		Path			nvarchar(256) NOT NULL,
		-- Whether the location is an absolute path or not.
		AbsolutePath		nvarchar(6) NOT NULL,
		CONSTRAINT PK_Location
			PRIMARY KEY CLUSTERED(LocationID)
	);

END
ELSE BEGIN
	PRINT 'Using existing SecPatch_Location table'
	IF EXISTS (
		SELECT * FROM sysindexes
		WHERE name = N'PK_Location'
		AND ((status & 6162) <> 2066)
	)
	BEGIN
		PRINT 'Recreating existing PK_Location index'
		ALTER TABLE [dbo].[SecPatch_Location]
		DROP CONSTRAINT PK_Location
	END

	IF NOT EXISTS (
		SELECT * FROM sysindexes
		WHERE name = N'PK_Location'
	)
		ALTER TABLE [dbo].[SecPatch_Location]
		ADD CONSTRAINT PK_Location
			PRIMARY KEY CLUSTERED(LocationID)

	END
GO

-- The SecPatch_Product table contains all the products that Microsoft has published security patches for.
IF NOT EXISTS (
	SELECT * FROM INFORMATION_SCHEMA.TABLES
	WHERE TABLE_NAME = N'SecPatch_Product'
	AND TABLE_SCHEMA = 'dbo'
)
BEGIN
	CREATE TABLE [dbo].[SecPatch_Product] (
		-- Primary key.
		ProductID		int IDENTITY NOT NULL,
		-- Name of this product.
		[Name]			nvarchar(80) NOT NULL,
		-- The least recent service pack that is supported for this product.
		MinimumSupportedServicePackID int NULL,
		-- The most recently released service pack for this product.
		CurrentServicePackID	int NULL,
		-- The most recently released version of this product.
		CurrentVersion		nvarchar(50) NULL,
		CONSTRAINT PK_Product
			PRIMARY KEY NONCLUSTERED([Name]),
		CONSTRAINT UQ_Product
			UNIQUE CLUSTERED(ProductID)
	);

END
ELSE BEGIN
	PRINT 'Using existing SecPatch_Product table'
	IF EXISTS (
		SELECT * FROM sysindexes
		WHERE name = N'UQ_Product'
		AND ((status & 6162) <> 4114)
	)
	BEGIN
		PRINT 'Recreating existing UQ_Product index'
		ALTER TABLE [dbo].[SecPatch_Product]
		DROP CONSTRAINT UQ_Product
	END

	IF EXISTS (
		SELECT * FROM sysindexes
		WHERE name = N'PK_Product'
		AND ((status & 6162) <> 2050)
	)
	BEGIN
		PRINT 'Recreating existing PK_Product index'
		ALTER TABLE [dbo].[SecPatch_Product]
		DROP CONSTRAINT PK_Product
	END

	IF NOT EXISTS (
		SELECT * FROM sysindexes
		WHERE name = N'UQ_Product'
	)
		ALTER TABLE [dbo].[SecPatch_Product]
		ADD CONSTRAINT UQ_Product
			UNIQUE CLUSTERED(ProductID)

	IF NOT EXISTS (
		SELECT * FROM sysindexes
		WHERE name = N'PK_Product'
	)
		ALTER TABLE [dbo].[SecPatch_Product]
		ADD CONSTRAINT PK_Product
			PRIMARY KEY NONCLUSTERED([Name])

	END
GO

-- The SecPatch_ServicePack table contains all the service packs for products that have
-- security patches published for them.
IF NOT EXISTS (
	SELECT * FROM INFORMATION_SCHEMA.TABLES
	WHERE TABLE_NAME = N'SecPatch_ServicePack'
	AND TABLE_SCHEMA = 'dbo'
)
BEGIN
	CREATE TABLE [dbo].[SecPatch_ServicePack] (
		-- Primary key.
		ServicePackID		int NOT NULL,
		-- The name of the service pack.
		[Name]			nvarchar(50) NOT NULL,
		-- The URL from which the service pack can be retrieved.
		URL			nvarchar(256) NULL,
		-- The release date of the service pack.
		ReleaseDate		datetime NULL,
		CONSTRAINT PK_ServicePack
			PRIMARY KEY CLUSTERED(ServicePackID)
	);

END
ELSE BEGIN
	PRINT 'Using existing SecPatch_ServicePack table'
	IF EXISTS (
		SELECT * FROM sysindexes
		WHERE name = N'PK_ServicePack'
		AND ((status & 6162) <> 2066)
	)
	BEGIN
		PRINT 'Recreating existing PK_ServicePack index'
		ALTER TABLE [dbo].[SecPatch_ServicePack]
		DROP CONSTRAINT PK_ServicePack
	END

	IF NOT EXISTS (
		SELECT * FROM sysindexes
		WHERE name = N'PK_ServicePack'
	)
		ALTER TABLE [dbo].[SecPatch_ServicePack]
		ADD CONSTRAINT PK_ServicePack
			PRIMARY KEY CLUSTERED(ServicePackID)

	END
GO

-- The SecPatch_Severity table contains descriptions of the severity of security bulletins.
IF NOT EXISTS (
	SELECT * FROM INFORMATION_SCHEMA.TABLES
	WHERE TABLE_NAME = N'SecPatch_Severity'
	AND TABLE_SCHEMA = 'dbo'
)
BEGIN
	CREATE TABLE [dbo].[SecPatch_Severity] (
		-- Primary key.
		SeverityID		char(10) NOT NULL,
		-- Descriptive text of the severity of a security bulletin.
		Description		nvarchar(50) NULL,
		CONSTRAINT PK_Severity
			PRIMARY KEY CLUSTERED(SeverityID)
	);

END
ELSE BEGIN
	PRINT 'Using existing SecPatch_Severity table'
	IF EXISTS (
		SELECT * FROM sysindexes
		WHERE name = N'PK_Severity'
		AND ((status & 6162) <> 2066)
	)
	BEGIN
		PRINT 'Recreating existing PK_Severity index'
		ALTER TABLE [dbo].[SecPatch_Severity]
		DROP CONSTRAINT PK_Severity
	END

	IF NOT EXISTS (
		SELECT * FROM sysindexes
		WHERE name = N'PK_Severity'
	)
		ALTER TABLE [dbo].[SecPatch_Severity]
		ADD CONSTRAINT PK_Severity
			PRIMARY KEY CLUSTERED(SeverityID)

	END
GO

-- The SecurityPatchSupersede table contains a list of patches that supersede other patches.
IF NOT EXISTS (
	SELECT * FROM INFORMATION_SCHEMA.TABLES
	WHERE TABLE_NAME = N'SecurityPatchSupersede'
	AND TABLE_SCHEMA = 'dbo'
)
BEGIN
	CREATE TABLE [dbo].[SecurityPatchSupersede] (
		-- Foreign key for security patch that supersedes another patch.
		NewSecurityPatchID	int NOT NULL,
		-- Foreign key for security patch that has been superseded.
		OldSecurityPatchID	int NOT NULL,
		CONSTRAINT PK_SecurityPatchSupersede
			PRIMARY KEY CLUSTERED(NewSecurityPatchID, OldSecurityPatchID)
	);

	CREATE NONCLUSTERED INDEX SecurityPatchSupersedeByOldSecurityPatch
		ON dbo.SecurityPatchSupersede(OldSecurityPatchID)

END
ELSE BEGIN
	PRINT 'Using existing SecurityPatchSupersede table'
	IF EXISTS (
		SELECT * FROM sysindexes
		WHERE name = N'PK_SecurityPatchSupersede'
		AND ((status & 6162) <> 2066)
	)
	BEGIN
		PRINT 'Recreating existing PK_SecurityPatchSupersede index'
		ALTER TABLE [dbo].[SecurityPatchSupersede]
		DROP CONSTRAINT PK_SecurityPatchSupersede
	END

	IF EXISTS (
		SELECT * FROM sysindexes
		WHERE name = N'SecurityPatchSupersedeByOldSecurityPatch'
		AND ((status & 6162) <> 0)
	)
	BEGIN
		PRINT 'Recreating existing SecurityPatchSupersedeByOldSecurityPatch index'
		DROP INDEX SecurityPatchSupersedeByOldSecurityPatch
		ON [dbo].[SecurityPatchSupersede]
	END

	IF NOT EXISTS (
		SELECT * FROM sysindexes
		WHERE name = N'PK_SecurityPatchSupersede'
	)
		ALTER TABLE [dbo].[SecurityPatchSupersede]
		ADD CONSTRAINT PK_SecurityPatchSupersede
			PRIMARY KEY CLUSTERED(NewSecurityPatchID, OldSecurityPatchID)

	IF NOT EXISTS (
		SELECT * FROM sysindexes
		WHERE name = N'SecurityPatchSupersedeByOldSecurityPatch'
	)
		CREATE NONCLUSTERED INDEX SecurityPatchSupersedeByOldSecurityPatch
			ON dbo.SecurityPatchSupersede(OldSecurityPatchID)

	END
GO

-- The SecPatch_ProductFamily table defines product groupings used by security patches.
IF NOT EXISTS (
	SELECT * FROM INFORMATION_SCHEMA.TABLES
	WHERE TABLE_NAME = N'SecPatch_ProductFamily'
	AND TABLE_SCHEMA = 'dbo'
)
BEGIN
	CREATE TABLE [dbo].[SecPatch_ProductFamily] (
		-- Primary key.
		ProductFamilyID		int NOT NULL,
		-- The name of the product family.
		[Name]			nvarchar(50) NOT NULL,
		-- Foreign key to the SecPatch_Product table.
		RecentProductVersionID	int NULL,
		CONSTRAINT PK_ProductFamily
			PRIMARY KEY CLUSTERED(ProductFamilyID)
	);

END
ELSE BEGIN
	PRINT 'Using existing SecPatch_ProductFamily table'
	IF EXISTS (
		SELECT * FROM sysindexes
		WHERE name = N'PK_ProductFamily'
		AND ((status & 6162) <> 2066)
	)
	BEGIN
		PRINT 'Recreating existing PK_ProductFamily index'
		ALTER TABLE [dbo].[SecPatch_ProductFamily]
		DROP CONSTRAINT PK_ProductFamily
	END

	IF NOT EXISTS (
		SELECT * FROM sysindexes
		WHERE name = N'PK_ProductFamily'
	)
		ALTER TABLE [dbo].[SecPatch_ProductFamily]
		ADD CONSTRAINT PK_ProductFamily
			PRIMARY KEY CLUSTERED(ProductFamilyID)

	END
GO

-- The SecPatch_ProductProductFamily table defines product groupings used by security patches.
IF NOT EXISTS (
	SELECT * FROM INFORMATION_SCHEMA.TABLES
	WHERE TABLE_NAME = N'SecPatch_ProductProductFamily'
	AND TABLE_SCHEMA = 'dbo'
)
BEGIN
	CREATE TABLE [dbo].[SecPatch_ProductProductFamily] (
		-- Primary key.
		ProductProductFamilyID	int IDENTITY NOT NULL,
		-- Foreign key to the SecPatch_Product table.
		ProductID		int NOT NULL,
		-- Foreign key to the SecPatch_ProductFamily table.
		ProductFamilyID		int NOT NULL,
		CONSTRAINT FK_ProductProductFamily_ProductFamily
			FOREIGN KEY(ProductFamilyID)
			REFERENCES SecPatch_ProductFamily(ProductFamilyID) ON DELETE CASCADE,
		CONSTRAINT FK_ProductProductFamily_Product
			FOREIGN KEY(ProductID)
			REFERENCES SecPatch_Product(ProductID) ON DELETE CASCADE,
		CONSTRAINT PK_ProductProductFamily
			PRIMARY KEY CLUSTERED(ProductID, ProductFamilyID)
	);

	CREATE NONCLUSTERED INDEX SecPatch_ProductProductFamilyByProductFamilyID
		ON dbo.SecPatch_ProductProductFamily(ProductFamilyID)

END
ELSE BEGIN
	PRINT 'Using existing SecPatch_ProductProductFamily table'
	IF EXISTS (
		SELECT * FROM sysindexes
		WHERE name = N'PK_ProductProductFamily'
		AND ((status & 6162) <> 2066)
	)
	BEGIN
		PRINT 'Recreating existing PK_ProductProductFamily index'
		ALTER TABLE [dbo].[SecPatch_ProductProductFamily]
		DROP CONSTRAINT PK_ProductProductFamily
	END

	IF EXISTS (
		SELECT * FROM sysindexes
		WHERE name = N'SecPatch_ProductProductFamilyByProductFamilyID'
		AND ((status & 6162) <> 0)
	)
	BEGIN
		PRINT 'Recreating existing SecPatch_ProductProductFamilyByProductFamilyID index'
		DROP INDEX SecPatch_ProductProductFamilyByProductFamilyID
		ON [dbo].[SecPatch_ProductProductFamily]
	END

	IF NOT EXISTS (
		SELECT * FROM sysindexes
		WHERE name = N'PK_ProductProductFamily'
	)
		ALTER TABLE [dbo].[SecPatch_ProductProductFamily]
		ADD CONSTRAINT PK_ProductProductFamily
			PRIMARY KEY CLUSTERED(ProductID, ProductFamilyID)

	IF NOT EXISTS (
		SELECT * FROM sysindexes
		WHERE name = N'SecPatch_ProductProductFamilyByProductFamilyID'
	)
		CREATE NONCLUSTERED INDEX SecPatch_ProductProductFamilyByProductFamilyID
			ON dbo.SecPatch_ProductProductFamily(ProductFamilyID)

	IF EXISTS(
		SELECT *
		FROM INFORMATION_SCHEMA.TABLES
		WHERE TABLE_NAME = 'SecPatch_ProductProductFamily'
		AND TABLE_SCHEMA = 'dbo'
	)
	AND EXISTS(
		SELECT *
		FROM INFORMATION_SCHEMA.TABLES
		WHERE TABLE_NAME = 'SecPatch_ProductFamily'
		AND TABLE_SCHEMA = 'dbo'
	)	
	AND NOT EXISTS (
		SELECT *
		FROM INFORMATION_SCHEMA.CONSTRAINT_COLUMN_USAGE
		WHERE CONSTRAINT_NAME = 'FK_ProductProductFamily_ProductFamily'
	)
	BEGIN
		PRINT 'Create FK_ProductProductFamily_ProductFamily constraint'
	
		ALTER TABLE [dbo].[SecPatch_ProductProductFamily]
			ADD CONSTRAINT FK_ProductProductFamily_ProductFamily
			FOREIGN KEY(ProductFamilyID)
			REFERENCES SecPatch_ProductFamily(ProductFamilyID) ON DELETE CASCADE
	END
	
	IF EXISTS(
		SELECT *
		FROM INFORMATION_SCHEMA.TABLES
		WHERE TABLE_NAME = 'SecPatch_ProductProductFamily'
		AND TABLE_SCHEMA = 'dbo'
	)
	AND EXISTS(
		SELECT *
		FROM INFORMATION_SCHEMA.TABLES
		WHERE TABLE_NAME = 'SecPatch_Product'
		AND TABLE_SCHEMA = 'dbo'
	)	
	AND NOT EXISTS (
		SELECT *
		FROM INFORMATION_SCHEMA.CONSTRAINT_COLUMN_USAGE
		WHERE CONSTRAINT_NAME = 'FK_ProductProductFamily_Product'
	)
	BEGIN
		PRINT 'Create FK_ProductProductFamily_Product constraint'
	
		ALTER TABLE [dbo].[SecPatch_ProductProductFamily]
			ADD CONSTRAINT FK_ProductProductFamily_Product
			FOREIGN KEY(ProductID)
			REFERENCES SecPatch_Product(ProductID) ON DELETE CASCADE
	END
END
GO

-- The SecPatch_BulletinComment table is a mapping between SecPatch_Bulletin and the SecPatch_Comment table.
IF NOT EXISTS (
	SELECT * FROM INFORMATION_SCHEMA.TABLES
	WHERE TABLE_NAME = N'SecPatch_BulletinComment'
	AND TABLE_SCHEMA = 'dbo'
)
BEGIN
	CREATE TABLE [dbo].[SecPatch_BulletinComment] (
		-- Primary key.
		BulletinCommentID	int IDENTITY NOT NULL,
		-- Foreign key to the SecPatch_Comment table.
		CommentID		int NOT NULL,
		-- Foreign key to the SecPatch_Bulletin table.
		BulletinID		nvarchar(50) NOT NULL,
		CONSTRAINT FK_BulletinComment_Comment
			FOREIGN KEY(CommentID)
			REFERENCES SecPatch_Comment(CommentID) ON DELETE CASCADE,
		CONSTRAINT PK_BulletinComment
			PRIMARY KEY CLUSTERED(BulletinCommentID),
		CONSTRAINT UQ_BulletinComment
			UNIQUE NONCLUSTERED(CommentID, BulletinID)
	);

	CREATE NONCLUSTERED INDEX SecPatch_BulletinCommentByBulletinID
		ON dbo.SecPatch_BulletinComment(BulletinID)

END
ELSE BEGIN
	PRINT 'Using existing SecPatch_BulletinComment table'
	IF EXISTS (
		SELECT * FROM sysindexes
		WHERE name = N'PK_BulletinComment'
		AND ((status & 6162) <> 2066)
	)
	BEGIN
		PRINT 'Recreating existing PK_BulletinComment index'
		ALTER TABLE [dbo].[SecPatch_BulletinComment]
		DROP CONSTRAINT PK_BulletinComment
	END

	IF EXISTS (
		SELECT * FROM sysindexes
		WHERE name = N'UQ_BulletinComment'
		AND ((status & 6162) <> 4098)
	)
	BEGIN
		PRINT 'Recreating existing UQ_BulletinComment index'
		ALTER TABLE [dbo].[SecPatch_BulletinComment]
		DROP CONSTRAINT UQ_BulletinComment
	END

	IF EXISTS (
		SELECT * FROM sysindexes
		WHERE name = N'SecPatch_BulletinCommentByBulletinID'
		AND ((status & 6162) <> 0)
	)
	BEGIN
		PRINT 'Recreating existing SecPatch_BulletinCommentByBulletinID index'
		DROP INDEX SecPatch_BulletinCommentByBulletinID
		ON [dbo].[SecPatch_BulletinComment]
	END

	IF NOT EXISTS (
		SELECT * FROM sysindexes
		WHERE name = N'PK_BulletinComment'
	)
		ALTER TABLE [dbo].[SecPatch_BulletinComment]
		ADD CONSTRAINT PK_BulletinComment
			PRIMARY KEY CLUSTERED(BulletinCommentID)

	IF NOT EXISTS (
		SELECT * FROM sysindexes
		WHERE name = N'UQ_BulletinComment'
	)
		ALTER TABLE [dbo].[SecPatch_BulletinComment]
		ADD CONSTRAINT UQ_BulletinComment
			UNIQUE NONCLUSTERED(CommentID, BulletinID)

	IF NOT EXISTS (
		SELECT * FROM sysindexes
		WHERE name = N'SecPatch_BulletinCommentByBulletinID'
	)
		CREATE NONCLUSTERED INDEX SecPatch_BulletinCommentByBulletinID
			ON dbo.SecPatch_BulletinComment(BulletinID)

	IF EXISTS(
		SELECT *
		FROM INFORMATION_SCHEMA.TABLES
		WHERE TABLE_NAME = 'SecPatch_BulletinComment'
		AND TABLE_SCHEMA = 'dbo'
	)
	AND EXISTS(
		SELECT *
		FROM INFORMATION_SCHEMA.TABLES
		WHERE TABLE_NAME = 'SecPatch_Comment'
		AND TABLE_SCHEMA = 'dbo'
	)	
	AND NOT EXISTS (
		SELECT *
		FROM INFORMATION_SCHEMA.CONSTRAINT_COLUMN_USAGE
		WHERE CONSTRAINT_NAME = 'FK_BulletinComment_Comment'
	)
	BEGIN
		PRINT 'Create FK_BulletinComment_Comment constraint'
	
		ALTER TABLE [dbo].[SecPatch_BulletinComment]
			ADD CONSTRAINT FK_BulletinComment_Comment
			FOREIGN KEY(CommentID)
			REFERENCES SecPatch_Comment(CommentID) ON DELETE CASCADE
	END
END
GO

-- The SecPatch_QNumber table contains "Qnumbers" which are the identifier that Microsoft uses for security bulletins.
IF NOT EXISTS (
	SELECT * FROM INFORMATION_SCHEMA.TABLES
	WHERE TABLE_NAME = N'SecPatch_QNumber'
	AND TABLE_SCHEMA = 'dbo'
)
BEGIN
	CREATE TABLE [dbo].[SecPatch_QNumber] (
		-- The "Qnumber" for this security bulletin.
		qNumber			nvarchar(50) NOT NULL,
		-- Foreign key to the SecPatch_Bulletin table.
		BulletinID		nvarchar(50) NOT NULL,
		CONSTRAINT PK_qNumber
			PRIMARY KEY CLUSTERED(qNumber)
	);

	CREATE NONCLUSTERED INDEX SecPatch_QNumberByBulletinID
		ON dbo.SecPatch_QNumber(BulletinID)

END
ELSE BEGIN
	PRINT 'Using existing SecPatch_QNumber table'
	IF EXISTS (
		SELECT * FROM sysindexes
		WHERE name = N'PK_qNumber'
		AND ((status & 6162) <> 2066)
	)
	BEGIN
		PRINT 'Recreating existing PK_qNumber index'
		ALTER TABLE [dbo].[SecPatch_QNumber]
		DROP CONSTRAINT PK_qNumber
	END

	IF EXISTS (
		SELECT * FROM sysindexes
		WHERE name = N'SecPatch_QNumberByBulletinID'
		AND ((status & 6162) <> 0)
	)
	BEGIN
		PRINT 'Recreating existing SecPatch_QNumberByBulletinID index'
		DROP INDEX SecPatch_QNumberByBulletinID
		ON [dbo].[SecPatch_QNumber]
	END

	IF NOT EXISTS (
		SELECT * FROM sysindexes
		WHERE name = N'PK_qNumber'
	)
		ALTER TABLE [dbo].[SecPatch_QNumber]
		ADD CONSTRAINT PK_qNumber
			PRIMARY KEY CLUSTERED(qNumber)

	IF NOT EXISTS (
		SELECT * FROM sysindexes
		WHERE name = N'SecPatch_QNumberByBulletinID'
	)
		CREATE NONCLUSTERED INDEX SecPatch_QNumberByBulletinID
			ON dbo.SecPatch_QNumber(BulletinID)

	END
GO

-- The SecPatch_Patch table contains security patches.
IF NOT EXISTS (
	SELECT * FROM INFORMATION_SCHEMA.TABLES
	WHERE TABLE_NAME = N'SecPatch_Patch'
	AND TABLE_SCHEMA = 'dbo'
)
BEGIN
	CREATE TABLE [dbo].[SecPatch_Patch] (
		-- Primary key.
		PatchID			int IDENTITY NOT NULL,
		-- The name of this security patch.
		PatchName		nvarchar(128) COLLATE Latin1_General_CS_AS NOT NULL,
		-- Foreign key to the SecPatch_Bulletin table.
		BulletinID		nvarchar(50) NOT NULL,
		-- Foreign key to the SecPatch_Location table.
		PatchLocationID		int NOT NULL,
		-- The SQNumber for this security patch.
		SQNumber		nvarchar(50) NOT NULL,
		-- Whether this security patch requires a reboot or not.
		NoReboot		int NOT NULL,
		-- Foreign key to the SecurityPatch table.
		SecurityPatchID		int NOT NULL,
		CONSTRAINT FK_Patch_Location
			FOREIGN KEY(PatchLocationID)
			REFERENCES SecPatch_Location(LocationID) ON DELETE CASCADE,
		CONSTRAINT PK_Patch
			PRIMARY KEY CLUSTERED(PatchID),
		CONSTRAINT UQ_Patch
			UNIQUE NONCLUSTERED(BulletinID, PatchName),
		CONSTRAINT UQ_PatchSecurityPatchID
			UNIQUE NONCLUSTERED(SecurityPatchID)
	);

	CREATE NONCLUSTERED INDEX IN_Patch
		ON dbo.SecPatch_Patch(PatchName)

	CREATE NONCLUSTERED INDEX SecPatch_PatchByPatchLocationID
		ON dbo.SecPatch_Patch(PatchLocationID)

END
ELSE BEGIN
	PRINT 'Using existing SecPatch_Patch table'
	IF EXISTS (
		SELECT * FROM sysindexes
		WHERE name = N'PK_Patch'
		AND ((status & 6162) <> 2066)
	)
	BEGIN
		PRINT 'Recreating existing PK_Patch index'
		ALTER TABLE [dbo].[SecPatch_Patch]
		DROP CONSTRAINT PK_Patch
	END

	IF EXISTS (
		SELECT * FROM sysindexes
		WHERE name = N'UQ_Patch'
		AND ((status & 6162) <> 4098)
	)
	BEGIN
		PRINT 'Recreating existing UQ_Patch index'
		ALTER TABLE [dbo].[SecPatch_Patch]
		DROP CONSTRAINT UQ_Patch
	END

	IF EXISTS (
		SELECT * FROM sysindexes
		WHERE name = N'UQ_PatchSecurityPatchID'
		AND ((status & 6162) <> 4098)
	)
	BEGIN
		PRINT 'Recreating existing UQ_PatchSecurityPatchID index'
		ALTER TABLE [dbo].[SecPatch_Patch]
		DROP CONSTRAINT UQ_PatchSecurityPatchID
	END

	IF EXISTS (
		SELECT * FROM sysindexes
		WHERE name = N'IN_Patch'
		AND ((status & 6162) <> 0)
	)
	BEGIN
		PRINT 'Recreating existing IN_Patch index'
		DROP INDEX IN_Patch
		ON [dbo].[SecPatch_Patch]
	END

	IF EXISTS (
		SELECT * FROM sysindexes
		WHERE name = N'SecPatch_PatchByPatchLocationID'
		AND ((status & 6162) <> 0)
	)
	BEGIN
		PRINT 'Recreating existing SecPatch_PatchByPatchLocationID index'
		DROP INDEX SecPatch_PatchByPatchLocationID
		ON [dbo].[SecPatch_Patch]
	END

	IF NOT EXISTS (
		SELECT * FROM sysindexes
		WHERE name = N'PK_Patch'
	)
		ALTER TABLE [dbo].[SecPatch_Patch]
		ADD CONSTRAINT PK_Patch
			PRIMARY KEY CLUSTERED(PatchID)

	IF NOT EXISTS (
		SELECT * FROM sysindexes
		WHERE name = N'UQ_Patch'
	)
		ALTER TABLE [dbo].[SecPatch_Patch]
		ADD CONSTRAINT UQ_Patch
			UNIQUE NONCLUSTERED(BulletinID, PatchName)

	IF NOT EXISTS (
		SELECT * FROM sysindexes
		WHERE name = N'UQ_PatchSecurityPatchID'
	)
		ALTER TABLE [dbo].[SecPatch_Patch]
		ADD CONSTRAINT UQ_PatchSecurityPatchID
			UNIQUE NONCLUSTERED(SecurityPatchID)

	IF NOT EXISTS (
		SELECT * FROM sysindexes
		WHERE name = N'IN_Patch'
	)
		CREATE NONCLUSTERED INDEX IN_Patch
			ON dbo.SecPatch_Patch(PatchName)

	IF NOT EXISTS (
		SELECT * FROM sysindexes
		WHERE name = N'SecPatch_PatchByPatchLocationID'
	)
		CREATE NONCLUSTERED INDEX SecPatch_PatchByPatchLocationID
			ON dbo.SecPatch_Patch(PatchLocationID)

	IF EXISTS(
		SELECT *
		FROM INFORMATION_SCHEMA.TABLES
		WHERE TABLE_NAME = 'SecPatch_Patch'
		AND TABLE_SCHEMA = 'dbo'
	)
	AND EXISTS(
		SELECT *
		FROM INFORMATION_SCHEMA.TABLES
		WHERE TABLE_NAME = 'SecPatch_Location'
		AND TABLE_SCHEMA = 'dbo'
	)	
	AND NOT EXISTS (
		SELECT *
		FROM INFORMATION_SCHEMA.CONSTRAINT_COLUMN_USAGE
		WHERE CONSTRAINT_NAME = 'FK_Patch_Location'
	)
	BEGIN
		PRINT 'Create FK_Patch_Location constraint'
	
		ALTER TABLE [dbo].[SecPatch_Patch]
			ADD CONSTRAINT FK_Patch_Location
			FOREIGN KEY(PatchLocationID)
			REFERENCES SecPatch_Location(LocationID) ON DELETE CASCADE
	END
END
GO

-- The SecPatch_PatchComment table is a mapping between SecPatch_Patch and the SecPatch_Comment table.
IF NOT EXISTS (
	SELECT * FROM INFORMATION_SCHEMA.TABLES
	WHERE TABLE_NAME = N'SecPatch_PatchComment'
	AND TABLE_SCHEMA = 'dbo'
)
BEGIN
	CREATE TABLE [dbo].[SecPatch_PatchComment] (
		-- Primary key.
		PatchCommentID		int IDENTITY NOT NULL,
		-- Foreign key to the SecPatch_Comment table.
		CommentID		int NOT NULL,
		-- Foreign key to the SecPatch_Patch table.
		PatchID			int NOT NULL,
		CONSTRAINT FK_PatchComment_Comment
			FOREIGN KEY(CommentID)
			REFERENCES SecPatch_Comment(CommentID) ON DELETE CASCADE,
		CONSTRAINT FK_PatchComment_Patch
			FOREIGN KEY(PatchID)
			REFERENCES SecPatch_Patch(PatchID) ON DELETE CASCADE,
		CONSTRAINT PK_PatchComment
			PRIMARY KEY CLUSTERED(PatchCommentID),
		CONSTRAINT UQ_PatchComment
			UNIQUE NONCLUSTERED(CommentID, PatchID)
	);

	CREATE NONCLUSTERED INDEX SecPatch_PatchCommentByPatchID
		ON dbo.SecPatch_PatchComment(PatchID)

END
ELSE BEGIN
	PRINT 'Using existing SecPatch_PatchComment table'
	IF EXISTS (
		SELECT * FROM sysindexes
		WHERE name = N'PK_PatchComment'
		AND ((status & 6162) <> 2066)
	)
	BEGIN
		PRINT 'Recreating existing PK_PatchComment index'
		ALTER TABLE [dbo].[SecPatch_PatchComment]
		DROP CONSTRAINT PK_PatchComment
	END

	IF EXISTS (
		SELECT * FROM sysindexes
		WHERE name = N'UQ_PatchComment'
		AND ((status & 6162) <> 4098)
	)
	BEGIN
		PRINT 'Recreating existing UQ_PatchComment index'
		ALTER TABLE [dbo].[SecPatch_PatchComment]
		DROP CONSTRAINT UQ_PatchComment
	END

	IF EXISTS (
		SELECT * FROM sysindexes
		WHERE name = N'SecPatch_PatchCommentByPatchID'
		AND ((status & 6162) <> 0)
	)
	BEGIN
		PRINT 'Recreating existing SecPatch_PatchCommentByPatchID index'
		DROP INDEX SecPatch_PatchCommentByPatchID
		ON [dbo].[SecPatch_PatchComment]
	END

	IF NOT EXISTS (
		SELECT * FROM sysindexes
		WHERE name = N'PK_PatchComment'
	)
		ALTER TABLE [dbo].[SecPatch_PatchComment]
		ADD CONSTRAINT PK_PatchComment
			PRIMARY KEY CLUSTERED(PatchCommentID)

	IF NOT EXISTS (
		SELECT * FROM sysindexes
		WHERE name = N'UQ_PatchComment'
	)
		ALTER TABLE [dbo].[SecPatch_PatchComment]
		ADD CONSTRAINT UQ_PatchComment
			UNIQUE NONCLUSTERED(CommentID, PatchID)

	IF NOT EXISTS (
		SELECT * FROM sysindexes
		WHERE name = N'SecPatch_PatchCommentByPatchID'
	)
		CREATE NONCLUSTERED INDEX SecPatch_PatchCommentByPatchID
			ON dbo.SecPatch_PatchComment(PatchID)

	IF EXISTS(
		SELECT *
		FROM INFORMATION_SCHEMA.TABLES
		WHERE TABLE_NAME = 'SecPatch_PatchComment'
		AND TABLE_SCHEMA = 'dbo'
	)
	AND EXISTS(
		SELECT *
		FROM INFORMATION_SCHEMA.TABLES
		WHERE TABLE_NAME = 'SecPatch_Comment'
		AND TABLE_SCHEMA = 'dbo'
	)	
	AND NOT EXISTS (
		SELECT *
		FROM INFORMATION_SCHEMA.CONSTRAINT_COLUMN_USAGE
		WHERE CONSTRAINT_NAME = 'FK_PatchComment_Comment'
	)
	BEGIN
		PRINT 'Create FK_PatchComment_Comment constraint'
	
		ALTER TABLE [dbo].[SecPatch_PatchComment]
			ADD CONSTRAINT FK_PatchComment_Comment
			FOREIGN KEY(CommentID)
			REFERENCES SecPatch_Comment(CommentID) ON DELETE CASCADE
	END
	
	IF EXISTS(
		SELECT *
		FROM INFORMATION_SCHEMA.TABLES
		WHERE TABLE_NAME = 'SecPatch_PatchComment'
		AND TABLE_SCHEMA = 'dbo'
	)
	AND EXISTS(
		SELECT *
		FROM INFORMATION_SCHEMA.TABLES
		WHERE TABLE_NAME = 'SecPatch_Patch'
		AND TABLE_SCHEMA = 'dbo'
	)	
	AND NOT EXISTS (
		SELECT *
		FROM INFORMATION_SCHEMA.CONSTRAINT_COLUMN_USAGE
		WHERE CONSTRAINT_NAME = 'FK_PatchComment_Patch'
	)
	BEGIN
		PRINT 'Create FK_PatchComment_Patch constraint'
	
		ALTER TABLE [dbo].[SecPatch_PatchComment]
			ADD CONSTRAINT FK_PatchComment_Patch
			FOREIGN KEY(PatchID)
			REFERENCES SecPatch_Patch(PatchID) ON DELETE CASCADE
	END
END
GO

-- The SecPatch_Bulletin table describes security bulletins.
IF NOT EXISTS (
	SELECT * FROM INFORMATION_SCHEMA.TABLES
	WHERE TABLE_NAME = N'SecPatch_Bulletin'
	AND TABLE_SCHEMA = 'dbo'
)
BEGIN
	CREATE TABLE [dbo].[SecPatch_Bulletin] (
		-- Primary key.
		BulletinID		nvarchar(50) NOT NULL,
		-- Foreign key to the SecPatch_Location table.
		BulletinLocationID	int NOT NULL,
		-- Foreign key to the SecPatch_Location table. This is the location of the FAQ document for this security bulletin.
		FAQLocationID		int NULL,
		-- The title text of this bulletin.
		Title			nvarchar(128) NULL,
		-- The date this security bulletin was posted.
		DatePosted		datetime NULL,
		-- The date this security bulletin was last revised.
		DateRevised		datetime NULL,
		-- Whether this security bulletin is still supported or not.
		Supported		nvarchar(10) NOT NULL,
		-- The summary text for this security bulletin.
		Summary			ntext NULL,
		-- The text describing the issue solved by this security bulletin.
		Issue			ntext NULL,
		-- Foreign key to the SecPatch_Severity table.
		ImpactSeverityID	char(10) NOT NULL,
		-- Foreign key to the SecPatch_Severity table.
		PreReqSeverityID	char(10) NOT NULL,
		-- Foreign key to the SecPatch_Severity table.
		MitigationSeverityID	char(10) NOT NULL,
		-- Foreign key to the SecPatch_Severity table.
		PopularitySeverityID	char(10) NOT NULL,
		CONSTRAINT FK_Bulletin_Location
			FOREIGN KEY(BulletinLocationID)
			REFERENCES SecPatch_Location(LocationID) ON DELETE NO ACTION,
		CONSTRAINT FK_Bulletin_Location1
			FOREIGN KEY(FAQLocationID)
			REFERENCES SecPatch_Location(LocationID) ON DELETE NO ACTION,
		CONSTRAINT PK_Bulletin
			PRIMARY KEY CLUSTERED(BulletinID)
	);

END
ELSE BEGIN
	PRINT 'Using existing SecPatch_Bulletin table'
	IF EXISTS (
		SELECT * FROM sysindexes
		WHERE name = N'PK_Bulletin'
		AND ((status & 6162) <> 2066)
	)
	BEGIN
		PRINT 'Recreating existing PK_Bulletin index'
		ALTER TABLE [dbo].[SecPatch_Bulletin]
		DROP CONSTRAINT PK_Bulletin
	END

	IF NOT EXISTS (
		SELECT * FROM sysindexes
		WHERE name = N'PK_Bulletin'
	)
		ALTER TABLE [dbo].[SecPatch_Bulletin]
		ADD CONSTRAINT PK_Bulletin
			PRIMARY KEY CLUSTERED(BulletinID)

	IF EXISTS(
		SELECT *
		FROM INFORMATION_SCHEMA.TABLES
		WHERE TABLE_NAME = 'SecPatch_Bulletin'
		AND TABLE_SCHEMA = 'dbo'
	)
	AND EXISTS(
		SELECT *
		FROM INFORMATION_SCHEMA.TABLES
		WHERE TABLE_NAME = 'SecPatch_Location'
		AND TABLE_SCHEMA = 'dbo'
	)	
	AND NOT EXISTS (
		SELECT *
		FROM INFORMATION_SCHEMA.CONSTRAINT_COLUMN_USAGE
		WHERE CONSTRAINT_NAME = 'FK_Bulletin_Location'
	)
	BEGIN
		PRINT 'Create FK_Bulletin_Location constraint'
	
		ALTER TABLE [dbo].[SecPatch_Bulletin]
			ADD CONSTRAINT FK_Bulletin_Location
			FOREIGN KEY(BulletinLocationID)
			REFERENCES SecPatch_Location(LocationID) ON DELETE NO ACTION
	END
	
	IF EXISTS(
		SELECT *
		FROM INFORMATION_SCHEMA.TABLES
		WHERE TABLE_NAME = 'SecPatch_Bulletin'
		AND TABLE_SCHEMA = 'dbo'
	)
	AND EXISTS(
		SELECT *
		FROM INFORMATION_SCHEMA.TABLES
		WHERE TABLE_NAME = 'SecPatch_Location'
		AND TABLE_SCHEMA = 'dbo'
	)	
	AND NOT EXISTS (
		SELECT *
		FROM INFORMATION_SCHEMA.CONSTRAINT_COLUMN_USAGE
		WHERE CONSTRAINT_NAME = 'FK_Bulletin_Location1'
	)
	BEGIN
		PRINT 'Create FK_Bulletin_Location1 constraint'
	
		ALTER TABLE [dbo].[SecPatch_Bulletin]
			ADD CONSTRAINT FK_Bulletin_Location1
			FOREIGN KEY(FAQLocationID)
			REFERENCES SecPatch_Location(LocationID) ON DELETE NO ACTION
	END
END
GO

-- The SecurityBulletin table is the ManageSoft record of known security bulletins.
IF NOT EXISTS (
	SELECT * FROM INFORMATION_SCHEMA.TABLES
	WHERE TABLE_NAME = N'SecurityBulletin'
	AND TABLE_SCHEMA = 'dbo'
)
BEGIN
	CREATE TABLE [dbo].[SecurityBulletin] (
		-- The artificial primary key.
		SecurityBulletinID	int IDENTITY NOT NULL,
		-- The name of the security bulletin. This is a unique key.
		BulletinID		nvarchar(32) NOT NULL,
		-- The severity of the security bulletin.
		Severity		int NOT NULL,
		-- The state of the security bulletin.
		State			int NOT NULL DEFAULT 1,
		-- Operator comments about this bulletin that are entered from the ManageSoft user interface.
		Comments		ntext NULL,
		-- This is the URL to the Microsoft support website for this bulletin. 
		HelpUrl			nvarchar(256) NULL,
		-- The date the bulletin was released.
		DatePosted		datetime NOT NULL,
		-- The date the bulletin was updated.
		DateRevised		datetime NOT NULL,
		-- The version of the bulletin.
		Version			nvarchar(16) NULL DEFAULT '1,0,0,0',
		-- Whether there is an update available.
		UpdateAvailable		int NOT NULL DEFAULT 0,
		-- The state of the bulletin which determines where it is shown in the user interface.
		UIStateID		int NOT NULL DEFAULT 1,
		-- Set to 1 when the bulletin is deployed.
		Hidden			int NOT NULL DEFAULT 0,
		-- Not used.
		PackageVersionID	int NULL,
		-- Not used.
		Errata			nvarchar(64) NULL,
		-- 
		-- Set to one if the bulletin is not to be shown in the user interface.
		-- Only bulletins that have been manually hidden have this value set.
		-- Bulletins that are hidden because of OS/product/language/architecture filters set UIStateID=Hidden instead.
		Ignore			int NOT NULL DEFAULT 0,
		-- Status whether the bulletin is superseded at all , partial or complete.
		SupersedeStatus		int NULL DEFAULT 1,
		-- The custom id of the bulletin.
		CustomID		nvarchar(25) NOT NULL DEFAULT '',
		-- RequiredCount set during the DetermineRequired Action
		RequiredCount		int NULL,
		CONSTRAINT PK_SecurityBulletin
			PRIMARY KEY CLUSTERED(SecurityBulletinID),
		CONSTRAINT UQ_SecurityBulletin
			UNIQUE NONCLUSTERED(BulletinID)
	);

	CREATE NONCLUSTERED INDEX SecurityBulletinByErrata
		ON dbo.SecurityBulletin(Errata)

	CREATE NONCLUSTERED INDEX SecurityBulletinByPackageVersionID
		ON dbo.SecurityBulletin(PackageVersionID)

END
ELSE BEGIN
	PRINT 'Using existing SecurityBulletin table'
	IF EXISTS (
		SELECT * FROM sysindexes
		WHERE name = N'PK_SecurityBulletin'
		AND ((status & 6162) <> 2066)
	)
	BEGIN
		PRINT 'Recreating existing PK_SecurityBulletin index'
		ALTER TABLE [dbo].[SecurityBulletin]
		DROP CONSTRAINT PK_SecurityBulletin
	END

	IF EXISTS (
		SELECT * FROM sysindexes
		WHERE name = N'UQ_SecurityBulletin'
		AND ((status & 6162) <> 4098)
	)
	BEGIN
		PRINT 'Recreating existing UQ_SecurityBulletin index'
		ALTER TABLE [dbo].[SecurityBulletin]
		DROP CONSTRAINT UQ_SecurityBulletin
	END

	IF EXISTS (
		SELECT * FROM sysindexes
		WHERE name = N'SecurityBulletinByErrata'
		AND ((status & 6162) <> 0)
	)
	BEGIN
		PRINT 'Recreating existing SecurityBulletinByErrata index'
		DROP INDEX SecurityBulletinByErrata
		ON [dbo].[SecurityBulletin]
	END

	IF EXISTS (
		SELECT * FROM sysindexes
		WHERE name = N'SecurityBulletinByPackageVersionID'
		AND ((status & 6162) <> 0)
	)
	BEGIN
		PRINT 'Recreating existing SecurityBulletinByPackageVersionID index'
		DROP INDEX SecurityBulletinByPackageVersionID
		ON [dbo].[SecurityBulletin]
	END

	IF NOT EXISTS (
		SELECT * FROM sysindexes
		WHERE name = N'PK_SecurityBulletin'
	)
		ALTER TABLE [dbo].[SecurityBulletin]
		ADD CONSTRAINT PK_SecurityBulletin
			PRIMARY KEY CLUSTERED(SecurityBulletinID)

	IF NOT EXISTS (
		SELECT * FROM sysindexes
		WHERE name = N'UQ_SecurityBulletin'
	)
		ALTER TABLE [dbo].[SecurityBulletin]
		ADD CONSTRAINT UQ_SecurityBulletin
			UNIQUE NONCLUSTERED(BulletinID)

	IF NOT EXISTS (
		SELECT * FROM sysindexes
		WHERE name = N'SecurityBulletinByErrata'
	)
		CREATE NONCLUSTERED INDEX SecurityBulletinByErrata
			ON dbo.SecurityBulletin(Errata)

	IF NOT EXISTS (
		SELECT * FROM sysindexes
		WHERE name = N'SecurityBulletinByPackageVersionID'
	)
		CREATE NONCLUSTERED INDEX SecurityBulletinByPackageVersionID
			ON dbo.SecurityBulletin(PackageVersionID)

	END
GO

-- The SecurityBulletinToUserState table provides a mapping from the SecurityBulletin table to the current state of the bulletin.
IF NOT EXISTS (
	SELECT * FROM INFORMATION_SCHEMA.TABLES
	WHERE TABLE_NAME = N'SecurityBulletinToUserState'
	AND TABLE_SCHEMA = 'dbo'
)
BEGIN
	CREATE TABLE [dbo].[SecurityBulletinToUserState] (
		-- Foreign key to the SecurityBulletin table.
		SecurityBulletinID	int NOT NULL,
		-- Foreign key to the SecurityUserState table.
		UserStateID		int NOT NULL,
		-- The date and time the state changed.
		DateChanged		datetime NOT NULL,
		-- A user comment that accompanies the state change.
		Comment			ntext NULL,
		-- The user who changed the state.
		Username		nvarchar(64) NOT NULL,
		-- Whether the security bulletin passed testing.
		Certified		int NOT NULL,
		CONSTRAINT FK_SecurityBulletinToUserState_SecurityBulletin
			FOREIGN KEY(SecurityBulletinID)
			REFERENCES SecurityBulletin(SecurityBulletinID) ON DELETE CASCADE,
		CONSTRAINT PK_SecurityBulletinToUserState
			PRIMARY KEY CLUSTERED(SecurityBulletinID, UserStateID)
	);

END
ELSE BEGIN
	PRINT 'Using existing SecurityBulletinToUserState table'
	IF EXISTS (
		SELECT * FROM sysindexes
		WHERE name = N'PK_SecurityBulletinToUserState'
		AND ((status & 6162) <> 2066)
	)
	BEGIN
		PRINT 'Recreating existing PK_SecurityBulletinToUserState index'
		ALTER TABLE [dbo].[SecurityBulletinToUserState]
		DROP CONSTRAINT PK_SecurityBulletinToUserState
	END

	IF NOT EXISTS (
		SELECT * FROM sysindexes
		WHERE name = N'PK_SecurityBulletinToUserState'
	)
		ALTER TABLE [dbo].[SecurityBulletinToUserState]
		ADD CONSTRAINT PK_SecurityBulletinToUserState
			PRIMARY KEY CLUSTERED(SecurityBulletinID, UserStateID)

	IF EXISTS(
		SELECT *
		FROM INFORMATION_SCHEMA.TABLES
		WHERE TABLE_NAME = 'SecurityBulletinToUserState'
		AND TABLE_SCHEMA = 'dbo'
	)
	AND EXISTS(
		SELECT *
		FROM INFORMATION_SCHEMA.TABLES
		WHERE TABLE_NAME = 'SecurityBulletin'
		AND TABLE_SCHEMA = 'dbo'
	)	
	AND NOT EXISTS (
		SELECT *
		FROM INFORMATION_SCHEMA.CONSTRAINT_COLUMN_USAGE
		WHERE CONSTRAINT_NAME = 'FK_SecurityBulletinToUserState_SecurityBulletin'
	)
	BEGIN
		PRINT 'Create FK_SecurityBulletinToUserState_SecurityBulletin constraint'
	
		ALTER TABLE [dbo].[SecurityBulletinToUserState]
			ADD CONSTRAINT FK_SecurityBulletinToUserState_SecurityBulletin
			FOREIGN KEY(SecurityBulletinID)
			REFERENCES SecurityBulletin(SecurityBulletinID) ON DELETE CASCADE
	END
END
GO

-- The SecurityBulletinRequires table contains a list of any bulletins that must be installed before this bulletin.
IF NOT EXISTS (
	SELECT * FROM INFORMATION_SCHEMA.TABLES
	WHERE TABLE_NAME = N'SecurityBulletinRequires'
	AND TABLE_SCHEMA = 'dbo'
)
BEGIN
	CREATE TABLE [dbo].[SecurityBulletinRequires] (
		-- Artificial primary key for this table.
		SecurityBulletinRequiresID int IDENTITY NOT NULL,
		-- Foreign key to the SecurityBulletin table.
		SecurityBulletinID	int NOT NULL,
		-- Foreign key to the SecurityBulletin table that the SecurityBulletinID requires.
		RequiredBulletinID	int NOT NULL,
		-- The type of requirement. Only supports 0, which is a dependency requirement.
		Strength		int NOT NULL,
		CONSTRAINT FK_SecurityBulletinRequires_SecurityBulletin
			FOREIGN KEY(SecurityBulletinID)
			REFERENCES SecurityBulletin(SecurityBulletinID) ON DELETE NO ACTION,
		CONSTRAINT FK_SecurityBulletinRequires2_SecurityBulletin
			FOREIGN KEY(RequiredBulletinID)
			REFERENCES SecurityBulletin(SecurityBulletinID) ON DELETE NO ACTION,
		CONSTRAINT PK_SecurityBulletinRequires
			PRIMARY KEY NONCLUSTERED(SecurityBulletinID, RequiredBulletinID)
	);

	CREATE CLUSTERED INDEX SecurityBulletinRequiresByBulletinIDRequiredBulletinID
		ON dbo.SecurityBulletinRequires(SecurityBulletinID, Strength)

	CREATE NONCLUSTERED INDEX SecurityBulletinRequiresByRequiredBulletinID
		ON dbo.SecurityBulletinRequires(RequiredBulletinID)

END
ELSE BEGIN
	PRINT 'Using existing SecurityBulletinRequires table'
	IF EXISTS (
		SELECT * FROM sysindexes
		WHERE name = N'SecurityBulletinRequiresByBulletinIDRequiredBulletinID'
		AND ((status & 6162) <> 16)
	)
	BEGIN
		PRINT 'Recreating existing SecurityBulletinRequiresByBulletinIDRequiredBulletinID index'
		DROP INDEX SecurityBulletinRequiresByBulletinIDRequiredBulletinID
		ON [dbo].[SecurityBulletinRequires]
	END

	IF EXISTS (
		SELECT * FROM sysindexes
		WHERE name = N'PK_SecurityBulletinRequires'
		AND ((status & 6162) <> 2050)
	)
	BEGIN
		PRINT 'Recreating existing PK_SecurityBulletinRequires index'
		ALTER TABLE [dbo].[SecurityBulletinRequires]
		DROP CONSTRAINT PK_SecurityBulletinRequires
	END

	IF EXISTS (
		SELECT * FROM sysindexes
		WHERE name = N'SecurityBulletinRequiresByRequiredBulletinID'
		AND ((status & 6162) <> 0)
	)
	BEGIN
		PRINT 'Recreating existing SecurityBulletinRequiresByRequiredBulletinID index'
		DROP INDEX SecurityBulletinRequiresByRequiredBulletinID
		ON [dbo].[SecurityBulletinRequires]
	END

	IF NOT EXISTS (
		SELECT * FROM sysindexes
		WHERE name = N'SecurityBulletinRequiresByBulletinIDRequiredBulletinID'
	)
		CREATE CLUSTERED INDEX SecurityBulletinRequiresByBulletinIDRequiredBulletinID
			ON dbo.SecurityBulletinRequires(SecurityBulletinID, Strength)

	IF NOT EXISTS (
		SELECT * FROM sysindexes
		WHERE name = N'PK_SecurityBulletinRequires'
	)
		ALTER TABLE [dbo].[SecurityBulletinRequires]
		ADD CONSTRAINT PK_SecurityBulletinRequires
			PRIMARY KEY NONCLUSTERED(SecurityBulletinID, RequiredBulletinID)

	IF NOT EXISTS (
		SELECT * FROM sysindexes
		WHERE name = N'SecurityBulletinRequiresByRequiredBulletinID'
	)
		CREATE NONCLUSTERED INDEX SecurityBulletinRequiresByRequiredBulletinID
			ON dbo.SecurityBulletinRequires(RequiredBulletinID)

	IF EXISTS(
		SELECT *
		FROM INFORMATION_SCHEMA.TABLES
		WHERE TABLE_NAME = 'SecurityBulletinRequires'
		AND TABLE_SCHEMA = 'dbo'
	)
	AND EXISTS(
		SELECT *
		FROM INFORMATION_SCHEMA.TABLES
		WHERE TABLE_NAME = 'SecurityBulletin'
		AND TABLE_SCHEMA = 'dbo'
	)	
	AND NOT EXISTS (
		SELECT *
		FROM INFORMATION_SCHEMA.CONSTRAINT_COLUMN_USAGE
		WHERE CONSTRAINT_NAME = 'FK_SecurityBulletinRequires_SecurityBulletin'
	)
	BEGIN
		PRINT 'Create FK_SecurityBulletinRequires_SecurityBulletin constraint'
	
		ALTER TABLE [dbo].[SecurityBulletinRequires]
			ADD CONSTRAINT FK_SecurityBulletinRequires_SecurityBulletin
			FOREIGN KEY(SecurityBulletinID)
			REFERENCES SecurityBulletin(SecurityBulletinID) ON DELETE NO ACTION
	END
	
	IF EXISTS(
		SELECT *
		FROM INFORMATION_SCHEMA.TABLES
		WHERE TABLE_NAME = 'SecurityBulletinRequires'
		AND TABLE_SCHEMA = 'dbo'
	)
	AND EXISTS(
		SELECT *
		FROM INFORMATION_SCHEMA.TABLES
		WHERE TABLE_NAME = 'SecurityBulletin'
		AND TABLE_SCHEMA = 'dbo'
	)	
	AND NOT EXISTS (
		SELECT *
		FROM INFORMATION_SCHEMA.CONSTRAINT_COLUMN_USAGE
		WHERE CONSTRAINT_NAME = 'FK_SecurityBulletinRequires2_SecurityBulletin'
	)
	BEGIN
		PRINT 'Create FK_SecurityBulletinRequires2_SecurityBulletin constraint'
	
		ALTER TABLE [dbo].[SecurityBulletinRequires]
			ADD CONSTRAINT FK_SecurityBulletinRequires2_SecurityBulletin
			FOREIGN KEY(RequiredBulletinID)
			REFERENCES SecurityBulletin(SecurityBulletinID) ON DELETE NO ACTION
	END
END
GO

-- The Linked Server to be used in SPM.
IF NOT EXISTS (
	SELECT * FROM INFORMATION_SCHEMA.TABLES
	WHERE TABLE_NAME = N'SecurityPatchManagerLinkedServer'
	AND TABLE_SCHEMA = 'dbo'
)
BEGIN
	CREATE TABLE [dbo].[SecurityPatchManagerLinkedServer] (
		-- Synthetic primary key for this table.
		LinkedServerID		int IDENTITY NOT NULL,
		-- The name of the linked server.
		LinkedServerName	nvarchar(128) COLLATE Latin1_General_CS_AS NOT NULL,
		-- The database name to be used of the server the linked server is pointing to.
		DatabaseName		nvarchar(128) NOT NULL,
		CONSTRAINT PK_LinkedServerID
			PRIMARY KEY CLUSTERED(LinkedServerID),
		CONSTRAINT UQ_LinkedServerConnection
			UNIQUE NONCLUSTERED(LinkedServerName, DatabaseName)
	);

END
ELSE BEGIN
	PRINT 'Using existing SecurityPatchManagerLinkedServer table'
	IF EXISTS (
		SELECT * FROM sysindexes
		WHERE name = N'PK_LinkedServerID'
		AND ((status & 6162) <> 2066)
	)
	BEGIN
		PRINT 'Recreating existing PK_LinkedServerID index'
		ALTER TABLE [dbo].[SecurityPatchManagerLinkedServer]
		DROP CONSTRAINT PK_LinkedServerID
	END

	IF EXISTS (
		SELECT * FROM sysindexes
		WHERE name = N'UQ_LinkedServerConnection'
		AND ((status & 6162) <> 4098)
	)
	BEGIN
		PRINT 'Recreating existing UQ_LinkedServerConnection index'
		ALTER TABLE [dbo].[SecurityPatchManagerLinkedServer]
		DROP CONSTRAINT UQ_LinkedServerConnection
	END

	IF NOT EXISTS (
		SELECT * FROM sysindexes
		WHERE name = N'PK_LinkedServerID'
	)
		ALTER TABLE [dbo].[SecurityPatchManagerLinkedServer]
		ADD CONSTRAINT PK_LinkedServerID
			PRIMARY KEY CLUSTERED(LinkedServerID)

	IF NOT EXISTS (
		SELECT * FROM sysindexes
		WHERE name = N'UQ_LinkedServerConnection'
	)
		ALTER TABLE [dbo].[SecurityPatchManagerLinkedServer]
		ADD CONSTRAINT UQ_LinkedServerConnection
			UNIQUE NONCLUSTERED(LinkedServerName, DatabaseName)

	END
GO

-- The SecurityCookie table contains a list of cookies to be used when downloading patches.
IF NOT EXISTS (
	SELECT * FROM INFORMATION_SCHEMA.TABLES
	WHERE TABLE_NAME = N'SecurityCookie'
	AND TABLE_SCHEMA = 'dbo'
)
BEGIN
	CREATE TABLE [dbo].[SecurityCookie] (
		-- Synthetic primary key for this table.
		CookieID		int IDENTITY NOT NULL,
		-- The name of the security Cookie.
		[Name]			nvarchar(128) COLLATE Latin1_General_CS_AS NOT NULL,
		-- The value of the security Cookie.
		Value			nvarchar(128) NOT NULL,
		-- The domain of the security Cookie.
		Domain			nvarchar(128) NOT NULL,
		CONSTRAINT PK_SecurityCookie
			PRIMARY KEY CLUSTERED(CookieID),
		CONSTRAINT UQ_SecurityCookie
			UNIQUE NONCLUSTERED([Name], Value, Domain)
	);

END
ELSE BEGIN
	PRINT 'Using existing SecurityCookie table'
	IF EXISTS (
		SELECT * FROM sysindexes
		WHERE name = N'PK_SecurityCookie'
		AND ((status & 6162) <> 2066)
	)
	BEGIN
		PRINT 'Recreating existing PK_SecurityCookie index'
		ALTER TABLE [dbo].[SecurityCookie]
		DROP CONSTRAINT PK_SecurityCookie
	END

	IF EXISTS (
		SELECT * FROM sysindexes
		WHERE name = N'UQ_SecurityCookie'
		AND ((status & 6162) <> 4098)
	)
	BEGIN
		PRINT 'Recreating existing UQ_SecurityCookie index'
		ALTER TABLE [dbo].[SecurityCookie]
		DROP CONSTRAINT UQ_SecurityCookie
	END

	IF NOT EXISTS (
		SELECT * FROM sysindexes
		WHERE name = N'PK_SecurityCookie'
	)
		ALTER TABLE [dbo].[SecurityCookie]
		ADD CONSTRAINT PK_SecurityCookie
			PRIMARY KEY CLUSTERED(CookieID)

	IF NOT EXISTS (
		SELECT * FROM sysindexes
		WHERE name = N'UQ_SecurityCookie'
	)
		ALTER TABLE [dbo].[SecurityCookie]
		ADD CONSTRAINT UQ_SecurityCookie
			UNIQUE NONCLUSTERED([Name], Value, Domain)

	END
GO

-- The SecurityPatch table is the ManageSoft record of known security patches.
IF NOT EXISTS (
	SELECT * FROM INFORMATION_SCHEMA.TABLES
	WHERE TABLE_NAME = N'SecurityPatch'
	AND TABLE_SCHEMA = 'dbo'
)
BEGIN
	CREATE TABLE [dbo].[SecurityPatch] (
		-- Synthetic primary key for this table.
		PatchID			int IDENTITY NOT NULL,
		-- Foreign key to the parent bulletin in the SecurityBulletin table.
		BulletinID		nvarchar(32) NOT NULL,
		-- The name of the security patch.
		PatchName		nvarchar(128) COLLATE Latin1_General_CS_AS NOT NULL,
		-- The title of the security patch.
		Title			nvarchar(256) NOT NULL,
		-- The QNumber of the patch.
		QNumber			nvarchar(32) NOT NULL,
		-- The state of the patch.
		State			int NOT NULL DEFAULT 1,
		-- The command line options to be used when installing the security patch.
		InstallCommandLineOptions nvarchar(256) NULL,
		-- The command line options to be used when uninstalling the security patch.
		UnInstallCommandLineOptions nvarchar(256) NULL,
		-- The path of the patch executables that has been staged in the ManageSoft administration server.
		StagedFile		nvarchar(512) NULL,
		-- Whether a patch should be included as part of the bulletin. Patches that are not incorporated are ignored by ManageSoft.
		Incorporate		int NOT NULL DEFAULT 1,
		-- Whether this patch is supported by the ManageSoft Security Patch Management System.
		Supported		int NOT NULL DEFAULT 0,
		-- Whether a patch should be displayed as part of the bulletin.
		Ignore			int NOT NULL DEFAULT 0,
		-- The default command line options to be used when installing the security patch.
		DefaultInstallCmdLineOptions nvarchar(256) NULL,
		-- The default command line options to be used when uninstalling the security patch.
		DefaultUnInstallCmdLineOptions nvarchar(256) NULL,
		-- The id of the security cookie.
		CookieID		int NULL,
		-- The id of the security cookie.
		UpdateAvailable		bit NULL DEFAULT 0,
		-- The id of the model.
		ModelID			int NOT NULL DEFAULT 0,
		CONSTRAINT FK_SecurityPatch_SecurityBulletin
			FOREIGN KEY(BulletinID)
			REFERENCES SecurityBulletin(BulletinID) ON DELETE CASCADE,
		CONSTRAINT PK_SecurityPatch
			PRIMARY KEY CLUSTERED(PatchID),
		CONSTRAINT UQ_SecurityPatch
			UNIQUE NONCLUSTERED(BulletinID, PatchName)
	);

	CREATE NONCLUSTERED INDEX SecurityPatchByCookieID
		ON dbo.SecurityPatch(CookieID)

END
ELSE BEGIN
	PRINT 'Using existing SecurityPatch table'
	IF EXISTS (
		SELECT * FROM sysindexes
		WHERE name = N'PK_SecurityPatch'
		AND ((status & 6162) <> 2066)
	)
	BEGIN
		PRINT 'Recreating existing PK_SecurityPatch index'
		ALTER TABLE [dbo].[SecurityPatch]
		DROP CONSTRAINT PK_SecurityPatch
	END

	IF EXISTS (
		SELECT * FROM sysindexes
		WHERE name = N'UQ_SecurityPatch'
		AND ((status & 6162) <> 4098)
	)
	BEGIN
		PRINT 'Recreating existing UQ_SecurityPatch index'
		ALTER TABLE [dbo].[SecurityPatch]
		DROP CONSTRAINT UQ_SecurityPatch
	END

	IF EXISTS (
		SELECT * FROM sysindexes
		WHERE name = N'SecurityPatchByCookieID'
		AND ((status & 6162) <> 0)
	)
	BEGIN
		PRINT 'Recreating existing SecurityPatchByCookieID index'
		DROP INDEX SecurityPatchByCookieID
		ON [dbo].[SecurityPatch]
	END

	IF NOT EXISTS (
		SELECT * FROM sysindexes
		WHERE name = N'PK_SecurityPatch'
	)
		ALTER TABLE [dbo].[SecurityPatch]
		ADD CONSTRAINT PK_SecurityPatch
			PRIMARY KEY CLUSTERED(PatchID)

	IF NOT EXISTS (
		SELECT * FROM sysindexes
		WHERE name = N'UQ_SecurityPatch'
	)
		ALTER TABLE [dbo].[SecurityPatch]
		ADD CONSTRAINT UQ_SecurityPatch
			UNIQUE NONCLUSTERED(BulletinID, PatchName)

	IF NOT EXISTS (
		SELECT * FROM sysindexes
		WHERE name = N'SecurityPatchByCookieID'
	)
		CREATE NONCLUSTERED INDEX SecurityPatchByCookieID
			ON dbo.SecurityPatch(CookieID)

	IF EXISTS(
		SELECT *
		FROM INFORMATION_SCHEMA.TABLES
		WHERE TABLE_NAME = 'SecurityPatch'
		AND TABLE_SCHEMA = 'dbo'
	)
	AND EXISTS(
		SELECT *
		FROM INFORMATION_SCHEMA.TABLES
		WHERE TABLE_NAME = 'SecurityBulletin'
		AND TABLE_SCHEMA = 'dbo'
	)	
	AND NOT EXISTS (
		SELECT *
		FROM INFORMATION_SCHEMA.CONSTRAINT_COLUMN_USAGE
		WHERE CONSTRAINT_NAME = 'FK_SecurityPatch_SecurityBulletin'
	)
	BEGIN
		PRINT 'Create FK_SecurityPatch_SecurityBulletin constraint'
	
		ALTER TABLE [dbo].[SecurityPatch]
			ADD CONSTRAINT FK_SecurityPatch_SecurityBulletin
			FOREIGN KEY(BulletinID)
			REFERENCES SecurityBulletin(BulletinID) ON DELETE CASCADE
	END
END
GO

-- The SecurityPatchAlias table contains a list of any aliases for a security patch.
IF NOT EXISTS (
	SELECT * FROM INFORMATION_SCHEMA.TABLES
	WHERE TABLE_NAME = N'SecurityPatchAlias'
	AND TABLE_SCHEMA = 'dbo'
)
BEGIN
	CREATE TABLE [dbo].[SecurityPatchAlias] (
		-- Foreign key to the parent bulletin in the SecurityBulletin table.
		BulletinID		nvarchar(32) NOT NULL,
		-- The alias of the security patch.
		PatchName		nvarchar(128) COLLATE Latin1_General_CS_AS NOT NULL,
		-- Foreign key to the patch in the SecurityPatch table with alias.
		SecurityPatchID		int NOT NULL,
		CONSTRAINT FK_SecurityPatchAlias_SecurityBulletin
			FOREIGN KEY(BulletinID)
			REFERENCES SecurityBulletin(BulletinID) ON DELETE NO ACTION,
		CONSTRAINT FK_SecurityPatchAlias_SecurityPatch
			FOREIGN KEY(SecurityPatchID)
			REFERENCES SecurityPatch(PatchID) ON DELETE CASCADE,
		CONSTRAINT PK_SecurityPatchAlias
			PRIMARY KEY CLUSTERED(BulletinID, PatchName)
	);

	CREATE NONCLUSTERED INDEX SecurityPatchAliasBySecurityPatchID
		ON dbo.SecurityPatchAlias(SecurityPatchID)

END
ELSE BEGIN
	PRINT 'Using existing SecurityPatchAlias table'
	IF EXISTS (
		SELECT * FROM sysindexes
		WHERE name = N'PK_SecurityPatchAlias'
		AND ((status & 6162) <> 2066)
	)
	BEGIN
		PRINT 'Recreating existing PK_SecurityPatchAlias index'
		ALTER TABLE [dbo].[SecurityPatchAlias]
		DROP CONSTRAINT PK_SecurityPatchAlias
	END

	IF EXISTS (
		SELECT * FROM sysindexes
		WHERE name = N'SecurityPatchAliasBySecurityPatchID'
		AND ((status & 6162) <> 0)
	)
	BEGIN
		PRINT 'Recreating existing SecurityPatchAliasBySecurityPatchID index'
		DROP INDEX SecurityPatchAliasBySecurityPatchID
		ON [dbo].[SecurityPatchAlias]
	END

	IF NOT EXISTS (
		SELECT * FROM sysindexes
		WHERE name = N'PK_SecurityPatchAlias'
	)
		ALTER TABLE [dbo].[SecurityPatchAlias]
		ADD CONSTRAINT PK_SecurityPatchAlias
			PRIMARY KEY CLUSTERED(BulletinID, PatchName)

	IF NOT EXISTS (
		SELECT * FROM sysindexes
		WHERE name = N'SecurityPatchAliasBySecurityPatchID'
	)
		CREATE NONCLUSTERED INDEX SecurityPatchAliasBySecurityPatchID
			ON dbo.SecurityPatchAlias(SecurityPatchID)

	IF EXISTS(
		SELECT *
		FROM INFORMATION_SCHEMA.TABLES
		WHERE TABLE_NAME = 'SecurityPatchAlias'
		AND TABLE_SCHEMA = 'dbo'
	)
	AND EXISTS(
		SELECT *
		FROM INFORMATION_SCHEMA.TABLES
		WHERE TABLE_NAME = 'SecurityBulletin'
		AND TABLE_SCHEMA = 'dbo'
	)	
	AND NOT EXISTS (
		SELECT *
		FROM INFORMATION_SCHEMA.CONSTRAINT_COLUMN_USAGE
		WHERE CONSTRAINT_NAME = 'FK_SecurityPatchAlias_SecurityBulletin'
	)
	BEGIN
		PRINT 'Create FK_SecurityPatchAlias_SecurityBulletin constraint'
	
		ALTER TABLE [dbo].[SecurityPatchAlias]
			ADD CONSTRAINT FK_SecurityPatchAlias_SecurityBulletin
			FOREIGN KEY(BulletinID)
			REFERENCES SecurityBulletin(BulletinID) ON DELETE NO ACTION
	END
	
	IF EXISTS(
		SELECT *
		FROM INFORMATION_SCHEMA.TABLES
		WHERE TABLE_NAME = 'SecurityPatchAlias'
		AND TABLE_SCHEMA = 'dbo'
	)
	AND EXISTS(
		SELECT *
		FROM INFORMATION_SCHEMA.TABLES
		WHERE TABLE_NAME = 'SecurityPatch'
		AND TABLE_SCHEMA = 'dbo'
	)	
	AND NOT EXISTS (
		SELECT *
		FROM INFORMATION_SCHEMA.CONSTRAINT_COLUMN_USAGE
		WHERE CONSTRAINT_NAME = 'FK_SecurityPatchAlias_SecurityPatch'
	)
	BEGIN
		PRINT 'Create FK_SecurityPatchAlias_SecurityPatch constraint'
	
		ALTER TABLE [dbo].[SecurityPatchAlias]
			ADD CONSTRAINT FK_SecurityPatchAlias_SecurityPatch
			FOREIGN KEY(SecurityPatchID)
			REFERENCES SecurityPatch(PatchID) ON DELETE CASCADE
	END
END
GO

-- The SecurityPatchRequires table contains a list of any patches that must be installed before this patch.
IF NOT EXISTS (
	SELECT * FROM INFORMATION_SCHEMA.TABLES
	WHERE TABLE_NAME = N'SecurityPatchRequires'
	AND TABLE_SCHEMA = 'dbo'
)
BEGIN
	CREATE TABLE [dbo].[SecurityPatchRequires] (
		-- Synthetic primary key.
		SecurityPatchRequiresID	int IDENTITY NOT NULL,
		-- Foreign key to the SecurityPatch table. 
		PatchID			int NOT NULL,
		-- This is the patch that the PatchID requires. It is a foreign key to the SecurityPatch table.
		RequiredPatchID		int NOT NULL,
		-- The type of requirement. Only supports 0, which is a dependency requirement.
		Strength		int NOT NULL,
		CONSTRAINT FK_SecurityPatchRequires_SecurityPatch
			FOREIGN KEY(PatchID)
			REFERENCES SecurityPatch(PatchID) ON DELETE CASCADE,
		CONSTRAINT FK_SecurityPatchRequires2_SecurityPatch
			FOREIGN KEY(RequiredPatchID)
			REFERENCES SecurityPatch(PatchID) ON DELETE NO ACTION,
		CONSTRAINT PK_SecurityPatchRequires
			PRIMARY KEY NONCLUSTERED(SecurityPatchRequiresID)
	);

	CREATE CLUSTERED INDEX SecurityPatchRequiresByPatchIDRequiredPatchID
		ON dbo.SecurityPatchRequires(PatchID, Strength)

END
ELSE BEGIN
	PRINT 'Using existing SecurityPatchRequires table'
	IF EXISTS (
		SELECT * FROM sysindexes
		WHERE name = N'SecurityPatchRequiresByPatchIDRequiredPatchID'
		AND ((status & 6162) <> 16)
	)
	BEGIN
		PRINT 'Recreating existing SecurityPatchRequiresByPatchIDRequiredPatchID index'
		DROP INDEX SecurityPatchRequiresByPatchIDRequiredPatchID
		ON [dbo].[SecurityPatchRequires]
	END

	IF EXISTS (
		SELECT * FROM sysindexes
		WHERE name = N'PK_SecurityPatchRequires'
		AND ((status & 6162) <> 2050)
	)
	BEGIN
		PRINT 'Recreating existing PK_SecurityPatchRequires index'
		ALTER TABLE [dbo].[SecurityPatchRequires]
		DROP CONSTRAINT PK_SecurityPatchRequires
	END

	IF NOT EXISTS (
		SELECT * FROM sysindexes
		WHERE name = N'SecurityPatchRequiresByPatchIDRequiredPatchID'
	)
		CREATE CLUSTERED INDEX SecurityPatchRequiresByPatchIDRequiredPatchID
			ON dbo.SecurityPatchRequires(PatchID, Strength)

	IF NOT EXISTS (
		SELECT * FROM sysindexes
		WHERE name = N'PK_SecurityPatchRequires'
	)
		ALTER TABLE [dbo].[SecurityPatchRequires]
		ADD CONSTRAINT PK_SecurityPatchRequires
			PRIMARY KEY NONCLUSTERED(SecurityPatchRequiresID)

	IF EXISTS(
		SELECT *
		FROM INFORMATION_SCHEMA.TABLES
		WHERE TABLE_NAME = 'SecurityPatchRequires'
		AND TABLE_SCHEMA = 'dbo'
	)
	AND EXISTS(
		SELECT *
		FROM INFORMATION_SCHEMA.TABLES
		WHERE TABLE_NAME = 'SecurityPatch'
		AND TABLE_SCHEMA = 'dbo'
	)	
	AND NOT EXISTS (
		SELECT *
		FROM INFORMATION_SCHEMA.CONSTRAINT_COLUMN_USAGE
		WHERE CONSTRAINT_NAME = 'FK_SecurityPatchRequires_SecurityPatch'
	)
	BEGIN
		PRINT 'Create FK_SecurityPatchRequires_SecurityPatch constraint'
	
		ALTER TABLE [dbo].[SecurityPatchRequires]
			ADD CONSTRAINT FK_SecurityPatchRequires_SecurityPatch
			FOREIGN KEY(PatchID)
			REFERENCES SecurityPatch(PatchID) ON DELETE CASCADE
	END
	
	IF EXISTS(
		SELECT *
		FROM INFORMATION_SCHEMA.TABLES
		WHERE TABLE_NAME = 'SecurityPatchRequires'
		AND TABLE_SCHEMA = 'dbo'
	)
	AND EXISTS(
		SELECT *
		FROM INFORMATION_SCHEMA.TABLES
		WHERE TABLE_NAME = 'SecurityPatch'
		AND TABLE_SCHEMA = 'dbo'
	)	
	AND NOT EXISTS (
		SELECT *
		FROM INFORMATION_SCHEMA.CONSTRAINT_COLUMN_USAGE
		WHERE CONSTRAINT_NAME = 'FK_SecurityPatchRequires2_SecurityPatch'
	)
	BEGIN
		PRINT 'Create FK_SecurityPatchRequires2_SecurityPatch constraint'
	
		ALTER TABLE [dbo].[SecurityPatchRequires]
			ADD CONSTRAINT FK_SecurityPatchRequires2_SecurityPatch
			FOREIGN KEY(RequiredPatchID)
			REFERENCES SecurityPatch(PatchID) ON DELETE NO ACTION
	END
END
GO

-- The SecurityPatchToPackageVersion table provides a mapping between security patches and packages in the Software Library.
IF NOT EXISTS (
	SELECT * FROM INFORMATION_SCHEMA.TABLES
	WHERE TABLE_NAME = N'SecurityPatchToPackageVersion'
	AND TABLE_SCHEMA = 'dbo'
)
BEGIN
	CREATE TABLE [dbo].[SecurityPatchToPackageVersion] (
		-- Foreign key to the SecurityPatch table.
		PatchID			int NOT NULL,
		-- Foreign key to the PackageVersion table. This is the package that contains the security patch.
		PackageFullName		nvarchar(256) NOT NULL,
		-- The name of the package.
		PackageName		nvarchar(64) NOT NULL,
		-- The version of the package.
		Version			nvarchar(32) NULL,
		CONSTRAINT FK_SecurityPatchToPackageVersion_SecurityPatch
			FOREIGN KEY(PatchID)
			REFERENCES SecurityPatch(PatchID) ON DELETE CASCADE,
		CONSTRAINT PK_SecurityPatchToPackageVersion
			PRIMARY KEY CLUSTERED(PatchID)
	);

END
ELSE BEGIN
	PRINT 'Using existing SecurityPatchToPackageVersion table'
	IF EXISTS (
		SELECT * FROM sysindexes
		WHERE name = N'PK_SecurityPatchToPackageVersion'
		AND ((status & 6162) <> 2066)
	)
	BEGIN
		PRINT 'Recreating existing PK_SecurityPatchToPackageVersion index'
		ALTER TABLE [dbo].[SecurityPatchToPackageVersion]
		DROP CONSTRAINT PK_SecurityPatchToPackageVersion
	END

	IF NOT EXISTS (
		SELECT * FROM sysindexes
		WHERE name = N'PK_SecurityPatchToPackageVersion'
	)
		ALTER TABLE [dbo].[SecurityPatchToPackageVersion]
		ADD CONSTRAINT PK_SecurityPatchToPackageVersion
			PRIMARY KEY CLUSTERED(PatchID)

	IF EXISTS(
		SELECT *
		FROM INFORMATION_SCHEMA.TABLES
		WHERE TABLE_NAME = 'SecurityPatchToPackageVersion'
		AND TABLE_SCHEMA = 'dbo'
	)
	AND EXISTS(
		SELECT *
		FROM INFORMATION_SCHEMA.TABLES
		WHERE TABLE_NAME = 'SecurityPatch'
		AND TABLE_SCHEMA = 'dbo'
	)	
	AND NOT EXISTS (
		SELECT *
		FROM INFORMATION_SCHEMA.CONSTRAINT_COLUMN_USAGE
		WHERE CONSTRAINT_NAME = 'FK_SecurityPatchToPackageVersion_SecurityPatch'
	)
	BEGIN
		PRINT 'Create FK_SecurityPatchToPackageVersion_SecurityPatch constraint'
	
		ALTER TABLE [dbo].[SecurityPatchToPackageVersion]
			ADD CONSTRAINT FK_SecurityPatchToPackageVersion_SecurityPatch
			FOREIGN KEY(PatchID)
			REFERENCES SecurityPatch(PatchID) ON DELETE CASCADE
	END
END
GO

-- The SecurityState table describes the state of a security patch or bulletin.
IF NOT EXISTS (
	SELECT * FROM INFORMATION_SCHEMA.TABLES
	WHERE TABLE_NAME = N'SecurityState'
	AND TABLE_SCHEMA = 'dbo'
)
BEGIN
	CREATE TABLE [dbo].[SecurityState] (
		-- Primary key.
		StateID			int IDENTITY NOT NULL,
		-- The description of the state.
		Description		nvarchar(64) NOT NULL,
		CONSTRAINT PK_SecurityState
			PRIMARY KEY CLUSTERED(StateID),
		CONSTRAINT UQ_SecurityStateByDescription
			UNIQUE NONCLUSTERED(Description)
	);

END
ELSE BEGIN
	PRINT 'Using existing SecurityState table'
	IF EXISTS (
		SELECT * FROM sysindexes
		WHERE name = N'PK_SecurityState'
		AND ((status & 6162) <> 2066)
	)
	BEGIN
		PRINT 'Recreating existing PK_SecurityState index'
		ALTER TABLE [dbo].[SecurityState]
		DROP CONSTRAINT PK_SecurityState
	END

	IF EXISTS (
		SELECT * FROM sysindexes
		WHERE name = N'UQ_SecurityStateByDescription'
		AND ((status & 6162) <> 4098)
	)
	BEGIN
		PRINT 'Recreating existing UQ_SecurityStateByDescription index'
		ALTER TABLE [dbo].[SecurityState]
		DROP CONSTRAINT UQ_SecurityStateByDescription
	END

	IF NOT EXISTS (
		SELECT * FROM sysindexes
		WHERE name = N'PK_SecurityState'
	)
		ALTER TABLE [dbo].[SecurityState]
		ADD CONSTRAINT PK_SecurityState
			PRIMARY KEY CLUSTERED(StateID)

	IF NOT EXISTS (
		SELECT * FROM sysindexes
		WHERE name = N'UQ_SecurityStateByDescription'
	)
		ALTER TABLE [dbo].[SecurityState]
		ADD CONSTRAINT UQ_SecurityStateByDescription
			UNIQUE NONCLUSTERED(Description)

	END
GO

-- The SecurityUserState table describes the state of a security patch or bulletin.
-- These are the user-signoff states (such as "patch tested", "package tested", and "signoff")
-- which appear in the user interface and allow for identification of the authorizing user.
IF NOT EXISTS (
	SELECT * FROM INFORMATION_SCHEMA.TABLES
	WHERE TABLE_NAME = N'SecurityUserState'
	AND TABLE_SCHEMA = 'dbo'
)
BEGIN
	CREATE TABLE [dbo].[SecurityUserState] (
		-- Primary key.
		UserStateID		int IDENTITY NOT NULL,
		-- The description of the user state.
		Description		nvarchar(64) NOT NULL,
		CONSTRAINT PK_SecurityUserState
			PRIMARY KEY CLUSTERED(UserStateID)
	);

END
ELSE BEGIN
	PRINT 'Using existing SecurityUserState table'
	IF EXISTS (
		SELECT * FROM sysindexes
		WHERE name = N'PK_SecurityUserState'
		AND ((status & 6162) <> 2066)
	)
	BEGIN
		PRINT 'Recreating existing PK_SecurityUserState index'
		ALTER TABLE [dbo].[SecurityUserState]
		DROP CONSTRAINT PK_SecurityUserState
	END

	IF NOT EXISTS (
		SELECT * FROM sysindexes
		WHERE name = N'PK_SecurityUserState'
	)
		ALTER TABLE [dbo].[SecurityUserState]
		ADD CONSTRAINT PK_SecurityUserState
			PRIMARY KEY CLUSTERED(UserStateID)

	END
GO

-- The SecurityUIState table describes the progress of a security patch or bulletin through the
-- various conceptual folders in the user interface (New, In Progress, and the like).
IF NOT EXISTS (
	SELECT * FROM INFORMATION_SCHEMA.TABLES
	WHERE TABLE_NAME = N'SecurityUIState'
	AND TABLE_SCHEMA = 'dbo'
)
BEGIN
	CREATE TABLE [dbo].[SecurityUIState] (
		-- Primary key.
		UIStateID		int IDENTITY NOT NULL,
		-- The description of the user interface state.
		Description		nvarchar(32) NOT NULL,
		CONSTRAINT PK_SecurityUIState
			PRIMARY KEY CLUSTERED(UIStateID),
		CONSTRAINT UQ_SecurityUIStateByDescription
			UNIQUE NONCLUSTERED(Description)
	);

END
ELSE BEGIN
	PRINT 'Using existing SecurityUIState table'
	IF EXISTS (
		SELECT * FROM sysindexes
		WHERE name = N'PK_SecurityUIState'
		AND ((status & 6162) <> 2066)
	)
	BEGIN
		PRINT 'Recreating existing PK_SecurityUIState index'
		ALTER TABLE [dbo].[SecurityUIState]
		DROP CONSTRAINT PK_SecurityUIState
	END

	IF EXISTS (
		SELECT * FROM sysindexes
		WHERE name = N'UQ_SecurityUIStateByDescription'
		AND ((status & 6162) <> 4098)
	)
	BEGIN
		PRINT 'Recreating existing UQ_SecurityUIStateByDescription index'
		ALTER TABLE [dbo].[SecurityUIState]
		DROP CONSTRAINT UQ_SecurityUIStateByDescription
	END

	IF NOT EXISTS (
		SELECT * FROM sysindexes
		WHERE name = N'PK_SecurityUIState'
	)
		ALTER TABLE [dbo].[SecurityUIState]
		ADD CONSTRAINT PK_SecurityUIState
			PRIMARY KEY CLUSTERED(UIStateID)

	IF NOT EXISTS (
		SELECT * FROM sysindexes
		WHERE name = N'UQ_SecurityUIStateByDescription'
	)
		ALTER TABLE [dbo].[SecurityUIState]
		ADD CONSTRAINT UQ_SecurityUIStateByDescription
			UNIQUE NONCLUSTERED(Description)

	END
GO

-- The SecurityStateChange table records changes in the SecurityState table for security patches and bulletins.
IF NOT EXISTS (
	SELECT * FROM INFORMATION_SCHEMA.TABLES
	WHERE TABLE_NAME = N'SecurityStateChange'
	AND TABLE_SCHEMA = 'dbo'
)
BEGIN
	CREATE TABLE [dbo].[SecurityStateChange] (
		-- Primary key.
		SecurityStateChangeID	int IDENTITY NOT NULL,
		-- The id of the bulletin whose state was changed.
		BulletinID		nvarchar(32) NOT NULL,
		-- The state after it was changed.
		StateTo			int NOT NULL,
		-- The state before it was changed.
		StateFrom		int NOT NULL,
		-- The date the state was changed.
		DateChanged		datetime NOT NULL,
		-- The user comment that accompanies the state change.
		Comment			ntext NULL,
		-- The user who performed the change.
		Username		nvarchar(64) NOT NULL,
		CONSTRAINT FK_SecurityStateChange_SecurityState
			FOREIGN KEY(StateTo)
			REFERENCES SecurityState(StateID) ON DELETE NO ACTION,
		CONSTRAINT FK_SecurityStateChange_SecurityState_From
			FOREIGN KEY(StateFrom)
			REFERENCES SecurityState(StateID) ON DELETE NO ACTION,
		CONSTRAINT FK_SecurityStateChange_SecurityBulletin
			FOREIGN KEY(BulletinID)
			REFERENCES SecurityBulletin(BulletinID) ON DELETE CASCADE,
		CONSTRAINT PK_SecurityStateChange
			PRIMARY KEY CLUSTERED(SecurityStateChangeID),
		CONSTRAINT UQ_SecurityStateChange
			UNIQUE NONCLUSTERED(BulletinID, DateChanged)
	);

END
ELSE BEGIN
	PRINT 'Using existing SecurityStateChange table'
	IF EXISTS (
		SELECT * FROM sysindexes
		WHERE name = N'PK_SecurityStateChange'
		AND ((status & 6162) <> 2066)
	)
	BEGIN
		PRINT 'Recreating existing PK_SecurityStateChange index'
		ALTER TABLE [dbo].[SecurityStateChange]
		DROP CONSTRAINT PK_SecurityStateChange
	END

	IF EXISTS (
		SELECT * FROM sysindexes
		WHERE name = N'UQ_SecurityStateChange'
		AND ((status & 6162) <> 4098)
	)
	BEGIN
		PRINT 'Recreating existing UQ_SecurityStateChange index'
		ALTER TABLE [dbo].[SecurityStateChange]
		DROP CONSTRAINT UQ_SecurityStateChange
	END

	IF NOT EXISTS (
		SELECT * FROM sysindexes
		WHERE name = N'PK_SecurityStateChange'
	)
		ALTER TABLE [dbo].[SecurityStateChange]
		ADD CONSTRAINT PK_SecurityStateChange
			PRIMARY KEY CLUSTERED(SecurityStateChangeID)

	IF NOT EXISTS (
		SELECT * FROM sysindexes
		WHERE name = N'UQ_SecurityStateChange'
	)
		ALTER TABLE [dbo].[SecurityStateChange]
		ADD CONSTRAINT UQ_SecurityStateChange
			UNIQUE NONCLUSTERED(BulletinID, DateChanged)

	IF EXISTS(
		SELECT *
		FROM INFORMATION_SCHEMA.TABLES
		WHERE TABLE_NAME = 'SecurityStateChange'
		AND TABLE_SCHEMA = 'dbo'
	)
	AND EXISTS(
		SELECT *
		FROM INFORMATION_SCHEMA.TABLES
		WHERE TABLE_NAME = 'SecurityState'
		AND TABLE_SCHEMA = 'dbo'
	)	
	AND NOT EXISTS (
		SELECT *
		FROM INFORMATION_SCHEMA.CONSTRAINT_COLUMN_USAGE
		WHERE CONSTRAINT_NAME = 'FK_SecurityStateChange_SecurityState'
	)
	BEGIN
		PRINT 'Create FK_SecurityStateChange_SecurityState constraint'
	
		ALTER TABLE [dbo].[SecurityStateChange]
			ADD CONSTRAINT FK_SecurityStateChange_SecurityState
			FOREIGN KEY(StateTo)
			REFERENCES SecurityState(StateID) ON DELETE NO ACTION
	END
	
	IF EXISTS(
		SELECT *
		FROM INFORMATION_SCHEMA.TABLES
		WHERE TABLE_NAME = 'SecurityStateChange'
		AND TABLE_SCHEMA = 'dbo'
	)
	AND EXISTS(
		SELECT *
		FROM INFORMATION_SCHEMA.TABLES
		WHERE TABLE_NAME = 'SecurityState'
		AND TABLE_SCHEMA = 'dbo'
	)	
	AND NOT EXISTS (
		SELECT *
		FROM INFORMATION_SCHEMA.CONSTRAINT_COLUMN_USAGE
		WHERE CONSTRAINT_NAME = 'FK_SecurityStateChange_SecurityState_From'
	)
	BEGIN
		PRINT 'Create FK_SecurityStateChange_SecurityState_From constraint'
	
		ALTER TABLE [dbo].[SecurityStateChange]
			ADD CONSTRAINT FK_SecurityStateChange_SecurityState_From
			FOREIGN KEY(StateFrom)
			REFERENCES SecurityState(StateID) ON DELETE NO ACTION
	END
	
	IF EXISTS(
		SELECT *
		FROM INFORMATION_SCHEMA.TABLES
		WHERE TABLE_NAME = 'SecurityStateChange'
		AND TABLE_SCHEMA = 'dbo'
	)
	AND EXISTS(
		SELECT *
		FROM INFORMATION_SCHEMA.TABLES
		WHERE TABLE_NAME = 'SecurityBulletin'
		AND TABLE_SCHEMA = 'dbo'
	)	
	AND NOT EXISTS (
		SELECT *
		FROM INFORMATION_SCHEMA.CONSTRAINT_COLUMN_USAGE
		WHERE CONSTRAINT_NAME = 'FK_SecurityStateChange_SecurityBulletin'
	)
	BEGIN
		PRINT 'Create FK_SecurityStateChange_SecurityBulletin constraint'
	
		ALTER TABLE [dbo].[SecurityStateChange]
			ADD CONSTRAINT FK_SecurityStateChange_SecurityBulletin
			FOREIGN KEY(BulletinID)
			REFERENCES SecurityBulletin(BulletinID) ON DELETE CASCADE
	END
END
GO

-- The SecurityUserStateChange table records changes in the SecurityUserState table for security bulletins.
IF NOT EXISTS (
	SELECT * FROM INFORMATION_SCHEMA.TABLES
	WHERE TABLE_NAME = N'SecurityUserStateChange'
	AND TABLE_SCHEMA = 'dbo'
)
BEGIN
	CREATE TABLE [dbo].[SecurityUserStateChange] (
		-- Primary key.
		SecurityUserStateChangeID int IDENTITY NOT NULL,
		-- The id of the bulletin whose state was changed.
		SecurityBulletinID	int NOT NULL,
		-- The UserState after it was changed.
		UserStateID		int NOT NULL,
		-- The date of the change.
		DateChanged		datetime NOT NULL,
		-- The user comment that accompanies the state change.
		Comment			ntext NULL,
		-- The user who performed the change.
		Username		nvarchar(64) NOT NULL,
		-- Whether the state was signed off after testing.
		Certified		int NOT NULL,
		CONSTRAINT FK_SecurityUserStateChange_SecurityState
			FOREIGN KEY(UserStateID)
			REFERENCES SecurityUserState(UserStateID) ON DELETE NO ACTION,
		CONSTRAINT FK_SecurityUserStateChange_SecurityBulletin
			FOREIGN KEY(SecurityBulletinID)
			REFERENCES SecurityBulletin(SecurityBulletinID) ON DELETE CASCADE,
		CONSTRAINT PK_SecurityUserStateChange
			PRIMARY KEY CLUSTERED(SecurityUserStateChangeID)
	);

	CREATE NONCLUSTERED INDEX UQ_SecurityUserStateChange
		ON dbo.SecurityUserStateChange(SecurityBulletinID, DateChanged)

END
ELSE BEGIN
	PRINT 'Using existing SecurityUserStateChange table'
	IF EXISTS (
		SELECT * FROM sysindexes
		WHERE name = N'PK_SecurityUserStateChange'
		AND ((status & 6162) <> 2066)
	)
	BEGIN
		PRINT 'Recreating existing PK_SecurityUserStateChange index'
		ALTER TABLE [dbo].[SecurityUserStateChange]
		DROP CONSTRAINT PK_SecurityUserStateChange
	END

	IF EXISTS (
		SELECT * FROM sysindexes
		WHERE name = N'UQ_SecurityUserStateChange'
		AND ((status & 6162) <> 0)
	)
	BEGIN
		PRINT 'Recreating existing UQ_SecurityUserStateChange index'
		DROP INDEX UQ_SecurityUserStateChange
		ON [dbo].[SecurityUserStateChange]
	END

	IF NOT EXISTS (
		SELECT * FROM sysindexes
		WHERE name = N'PK_SecurityUserStateChange'
	)
		ALTER TABLE [dbo].[SecurityUserStateChange]
		ADD CONSTRAINT PK_SecurityUserStateChange
			PRIMARY KEY CLUSTERED(SecurityUserStateChangeID)

	IF NOT EXISTS (
		SELECT * FROM sysindexes
		WHERE name = N'UQ_SecurityUserStateChange'
	)
		CREATE NONCLUSTERED INDEX UQ_SecurityUserStateChange
			ON dbo.SecurityUserStateChange(SecurityBulletinID, DateChanged)

	IF EXISTS(
		SELECT *
		FROM INFORMATION_SCHEMA.TABLES
		WHERE TABLE_NAME = 'SecurityUserStateChange'
		AND TABLE_SCHEMA = 'dbo'
	)
	AND EXISTS(
		SELECT *
		FROM INFORMATION_SCHEMA.TABLES
		WHERE TABLE_NAME = 'SecurityUserState'
		AND TABLE_SCHEMA = 'dbo'
	)	
	AND NOT EXISTS (
		SELECT *
		FROM INFORMATION_SCHEMA.CONSTRAINT_COLUMN_USAGE
		WHERE CONSTRAINT_NAME = 'FK_SecurityUserStateChange_SecurityState'
	)
	BEGIN
		PRINT 'Create FK_SecurityUserStateChange_SecurityState constraint'
	
		ALTER TABLE [dbo].[SecurityUserStateChange]
			ADD CONSTRAINT FK_SecurityUserStateChange_SecurityState
			FOREIGN KEY(UserStateID)
			REFERENCES SecurityUserState(UserStateID) ON DELETE NO ACTION
	END
	
	IF EXISTS(
		SELECT *
		FROM INFORMATION_SCHEMA.TABLES
		WHERE TABLE_NAME = 'SecurityUserStateChange'
		AND TABLE_SCHEMA = 'dbo'
	)
	AND EXISTS(
		SELECT *
		FROM INFORMATION_SCHEMA.TABLES
		WHERE TABLE_NAME = 'SecurityBulletin'
		AND TABLE_SCHEMA = 'dbo'
	)	
	AND NOT EXISTS (
		SELECT *
		FROM INFORMATION_SCHEMA.CONSTRAINT_COLUMN_USAGE
		WHERE CONSTRAINT_NAME = 'FK_SecurityUserStateChange_SecurityBulletin'
	)
	BEGIN
		PRINT 'Create FK_SecurityUserStateChange_SecurityBulletin constraint'
	
		ALTER TABLE [dbo].[SecurityUserStateChange]
			ADD CONSTRAINT FK_SecurityUserStateChange_SecurityBulletin
			FOREIGN KEY(SecurityBulletinID)
			REFERENCES SecurityBulletin(SecurityBulletinID) ON DELETE CASCADE
	END
END
GO

-- The SecurityAnalysisReport table records the last time that a computer reported security patch compliance data.
IF NOT EXISTS (
	SELECT * FROM INFORMATION_SCHEMA.TABLES
	WHERE TABLE_NAME = N'SecurityAnalysisReport'
	AND TABLE_SCHEMA = 'dbo'
)
BEGIN
	CREATE TABLE [dbo].[SecurityAnalysisReport] (
		-- The id of the computer. Foreign key into the Computer table.
		ComputerID		int NOT NULL,
		-- The date that the version of mssecure.xml was last updated on the client.
		DataSourceDate		datetime NOT NULL DEFAULT getdate(),
		-- The version of mssecure.xml that was last updated on the client.
		DataSourceVersion	nvarchar(32) NULL,
		-- The time the compliance was performed on the client.
		AnalysisDate		datetime NOT NULL DEFAULT getdate(),
		-- This field is currently unused.<!-- When the Administration Server received the compliance report. -->
		ReceiveDate		datetime NOT NULL DEFAULT getdate(),
		CONSTRAINT PK_SecurityAnalysisReport_ByComputer
			PRIMARY KEY CLUSTERED(ComputerID)
	);

	CREATE NONCLUSTERED INDEX SecurityAnalysisReport_ByDataSourceDate
		ON dbo.SecurityAnalysisReport(DataSourceDate)

END
ELSE BEGIN
	PRINT 'Using existing SecurityAnalysisReport table'
	IF EXISTS (
		SELECT * FROM sysindexes
		WHERE name = N'PK_SecurityAnalysisReport_ByComputer'
		AND ((status & 6162) <> 2066)
	)
	BEGIN
		PRINT 'Recreating existing PK_SecurityAnalysisReport_ByComputer index'
		ALTER TABLE [dbo].[SecurityAnalysisReport]
		DROP CONSTRAINT PK_SecurityAnalysisReport_ByComputer
	END

	IF EXISTS (
		SELECT * FROM sysindexes
		WHERE name = N'SecurityAnalysisReport_ByDataSourceDate'
		AND ((status & 6162) <> 0)
	)
	BEGIN
		PRINT 'Recreating existing SecurityAnalysisReport_ByDataSourceDate index'
		DROP INDEX SecurityAnalysisReport_ByDataSourceDate
		ON [dbo].[SecurityAnalysisReport]
	END

	IF NOT EXISTS (
		SELECT * FROM sysindexes
		WHERE name = N'PK_SecurityAnalysisReport_ByComputer'
	)
		ALTER TABLE [dbo].[SecurityAnalysisReport]
		ADD CONSTRAINT PK_SecurityAnalysisReport_ByComputer
			PRIMARY KEY CLUSTERED(ComputerID)

	IF NOT EXISTS (
		SELECT * FROM sysindexes
		WHERE name = N'SecurityAnalysisReport_ByDataSourceDate'
	)
		CREATE NONCLUSTERED INDEX SecurityAnalysisReport_ByDataSourceDate
			ON dbo.SecurityAnalysisReport(DataSourceDate)

	END
GO

-- The SecurityPatchAnalysis table contains all the security patch compliance status entries for a computer.
IF NOT EXISTS (
	SELECT * FROM INFORMATION_SCHEMA.TABLES
	WHERE TABLE_NAME = N'SecurityPatchAnalysis'
	AND TABLE_SCHEMA = 'dbo'
)
BEGIN
	CREATE TABLE [dbo].[SecurityPatchAnalysis] (
		-- Foreign key to the Computer table.
		ComputerID		int NOT NULL,
		-- The id string of the bulletin from mssecure.
		BulletinID		nvarchar(32) NOT NULL,
		-- Foreign key into the SecurityPatch table.
		SecurityPatchID		int NOT NULL,
		-- The mssecure id of the product that this patch applies to.
		ProductID		int NULL,
		-- The mssecure id of the service pack that this patch applies to.
		ServicePackID		int NULL,
		-- Foreign key into the SecurityPatchStatus table.
		Status			int NOT NULL,
		-- Compliance message from the client.
		StatusMessage		nvarchar(3000) NULL,
		-- Whether this patch was installed by ManageSoft.
		IsManaged		bit NOT NULL DEFAULT 0,
		CONSTRAINT FK_SecurityPatchAnalysis_SecurityPatch
			FOREIGN KEY(SecurityPatchID)
			REFERENCES SecurityPatch(PatchID) ON DELETE CASCADE,
		CONSTRAINT PK_SecurityPatchAnalysis_ByComputer
			PRIMARY KEY CLUSTERED(ComputerID, BulletinID, SecurityPatchID)
	);

	CREATE NONCLUSTERED INDEX SecurityPatchAnalysis_ByBulletin
		ON dbo.SecurityPatchAnalysis(BulletinID, ComputerID)

	CREATE NONCLUSTERED INDEX SecurityPatchAnalysis_ByStatus
		ON dbo.SecurityPatchAnalysis(Status)

	CREATE NONCLUSTERED INDEX SecurityPatchAnalysisBySecurityPatchID
		ON dbo.SecurityPatchAnalysis(SecurityPatchID)

END
ELSE BEGIN
	PRINT 'Using existing SecurityPatchAnalysis table'
	IF EXISTS (
		SELECT * FROM sysindexes
		WHERE name = N'PK_SecurityPatchAnalysis_ByComputer'
		AND ((status & 6162) <> 2066)
	)
	BEGIN
		PRINT 'Recreating existing PK_SecurityPatchAnalysis_ByComputer index'
		ALTER TABLE [dbo].[SecurityPatchAnalysis]
		DROP CONSTRAINT PK_SecurityPatchAnalysis_ByComputer
	END

	IF EXISTS (
		SELECT * FROM sysindexes
		WHERE name = N'SecurityPatchAnalysis_ByBulletin'
		AND ((status & 6162) <> 0)
	)
	BEGIN
		PRINT 'Recreating existing SecurityPatchAnalysis_ByBulletin index'
		DROP INDEX SecurityPatchAnalysis_ByBulletin
		ON [dbo].[SecurityPatchAnalysis]
	END

	IF EXISTS (
		SELECT * FROM sysindexes
		WHERE name = N'SecurityPatchAnalysis_ByStatus'
		AND ((status & 6162) <> 0)
	)
	BEGIN
		PRINT 'Recreating existing SecurityPatchAnalysis_ByStatus index'
		DROP INDEX SecurityPatchAnalysis_ByStatus
		ON [dbo].[SecurityPatchAnalysis]
	END

	IF EXISTS (
		SELECT * FROM sysindexes
		WHERE name = N'SecurityPatchAnalysisBySecurityPatchID'
		AND ((status & 6162) <> 0)
	)
	BEGIN
		PRINT 'Recreating existing SecurityPatchAnalysisBySecurityPatchID index'
		DROP INDEX SecurityPatchAnalysisBySecurityPatchID
		ON [dbo].[SecurityPatchAnalysis]
	END

	IF NOT EXISTS (
		SELECT * FROM sysindexes
		WHERE name = N'PK_SecurityPatchAnalysis_ByComputer'
	)
		ALTER TABLE [dbo].[SecurityPatchAnalysis]
		ADD CONSTRAINT PK_SecurityPatchAnalysis_ByComputer
			PRIMARY KEY CLUSTERED(ComputerID, BulletinID, SecurityPatchID)

	IF NOT EXISTS (
		SELECT * FROM sysindexes
		WHERE name = N'SecurityPatchAnalysis_ByBulletin'
	)
		CREATE NONCLUSTERED INDEX SecurityPatchAnalysis_ByBulletin
			ON dbo.SecurityPatchAnalysis(BulletinID, ComputerID)

	IF NOT EXISTS (
		SELECT * FROM sysindexes
		WHERE name = N'SecurityPatchAnalysis_ByStatus'
	)
		CREATE NONCLUSTERED INDEX SecurityPatchAnalysis_ByStatus
			ON dbo.SecurityPatchAnalysis(Status)

	IF NOT EXISTS (
		SELECT * FROM sysindexes
		WHERE name = N'SecurityPatchAnalysisBySecurityPatchID'
	)
		CREATE NONCLUSTERED INDEX SecurityPatchAnalysisBySecurityPatchID
			ON dbo.SecurityPatchAnalysis(SecurityPatchID)

	IF EXISTS(
		SELECT *
		FROM INFORMATION_SCHEMA.TABLES
		WHERE TABLE_NAME = 'SecurityPatchAnalysis'
		AND TABLE_SCHEMA = 'dbo'
	)
	AND EXISTS(
		SELECT *
		FROM INFORMATION_SCHEMA.TABLES
		WHERE TABLE_NAME = 'SecurityPatch'
		AND TABLE_SCHEMA = 'dbo'
	)	
	AND NOT EXISTS (
		SELECT *
		FROM INFORMATION_SCHEMA.CONSTRAINT_COLUMN_USAGE
		WHERE CONSTRAINT_NAME = 'FK_SecurityPatchAnalysis_SecurityPatch'
	)
	BEGIN
		PRINT 'Create FK_SecurityPatchAnalysis_SecurityPatch constraint'
	
		ALTER TABLE [dbo].[SecurityPatchAnalysis]
			ADD CONSTRAINT FK_SecurityPatchAnalysis_SecurityPatch
			FOREIGN KEY(SecurityPatchID)
			REFERENCES SecurityPatch(PatchID) ON DELETE CASCADE
	END
END
GO

-- A list of statuses that are used in the SecurityPatchAnalysis table. Populated at database creation.
IF NOT EXISTS (
	SELECT * FROM INFORMATION_SCHEMA.TABLES
	WHERE TABLE_NAME = N'SecurityPatchStatus'
	AND TABLE_SCHEMA = 'dbo'
)
BEGIN
	CREATE TABLE [dbo].[SecurityPatchStatus] (
		-- Primary key.
		SecurityPatchStatusID	int IDENTITY NOT NULL,
		-- The text representing the patch compliance status that is matched against the MSA file (client security patch compliance log).
		InternalText		nvarchar(64) NOT NULL,
		-- Friendly version of the InternalText.
		DisplayText		nvarchar(64) NOT NULL,
		-- Priority of each status value for ordering in the reports.
		Severity		int NOT NULL,
		CONSTRAINT PK_SecurityPatchStatus
			PRIMARY KEY CLUSTERED(SecurityPatchStatusID),
		CONSTRAINT UQ_SecurityPatchStatus
			UNIQUE NONCLUSTERED(InternalText)
	);

END
ELSE BEGIN
	PRINT 'Using existing SecurityPatchStatus table'
	IF EXISTS (
		SELECT * FROM sysindexes
		WHERE name = N'PK_SecurityPatchStatus'
		AND ((status & 6162) <> 2066)
	)
	BEGIN
		PRINT 'Recreating existing PK_SecurityPatchStatus index'
		ALTER TABLE [dbo].[SecurityPatchStatus]
		DROP CONSTRAINT PK_SecurityPatchStatus
	END

	IF EXISTS (
		SELECT * FROM sysindexes
		WHERE name = N'UQ_SecurityPatchStatus'
		AND ((status & 6162) <> 4098)
	)
	BEGIN
		PRINT 'Recreating existing UQ_SecurityPatchStatus index'
		ALTER TABLE [dbo].[SecurityPatchStatus]
		DROP CONSTRAINT UQ_SecurityPatchStatus
	END

	IF NOT EXISTS (
		SELECT * FROM sysindexes
		WHERE name = N'PK_SecurityPatchStatus'
	)
		ALTER TABLE [dbo].[SecurityPatchStatus]
		ADD CONSTRAINT PK_SecurityPatchStatus
			PRIMARY KEY CLUSTERED(SecurityPatchStatusID)

	IF NOT EXISTS (
		SELECT * FROM sysindexes
		WHERE name = N'UQ_SecurityPatchStatus'
	)
		ALTER TABLE [dbo].[SecurityPatchStatus]
		ADD CONSTRAINT UQ_SecurityPatchStatus
			UNIQUE NONCLUSTERED(InternalText)

	END
GO

-- The SecurityPatchUIProductGroup table maps products to the logical product groupings that appear in the Security Patch Management UI.
IF NOT EXISTS (
	SELECT * FROM INFORMATION_SCHEMA.TABLES
	WHERE TABLE_NAME = N'SecurityPatchUIProductGroup'
	AND TABLE_SCHEMA = 'dbo'
)
BEGIN
	CREATE TABLE [dbo].[SecurityPatchUIProductGroup] (
		-- Primary key.
		SecurityPatchUIProductGroupID int IDENTITY NOT NULL,
		-- The name of the logical product group.
		GroupName		nvarchar(64) NOT NULL,
		-- The product name. This is a foreign key into the SecPatch_Product table.
		Product			nvarchar(128) NOT NULL,
		CONSTRAINT PK_SecurityPatchUIProductGroup
			PRIMARY KEY NONCLUSTERED(SecurityPatchUIProductGroupID),
		CONSTRAINT UQ_SecurityPatchUIProductGroupByProduct
			UNIQUE NONCLUSTERED(Product)
	);

	CREATE CLUSTERED INDEX SecurityPatchUIProductGroupByGroup
		ON dbo.SecurityPatchUIProductGroup(GroupName)

END
ELSE BEGIN
	PRINT 'Using existing SecurityPatchUIProductGroup table'
	IF EXISTS (
		SELECT * FROM sysindexes
		WHERE name = N'SecurityPatchUIProductGroupByGroup'
		AND ((status & 6162) <> 16)
	)
	BEGIN
		PRINT 'Recreating existing SecurityPatchUIProductGroupByGroup index'
		DROP INDEX SecurityPatchUIProductGroupByGroup
		ON [dbo].[SecurityPatchUIProductGroup]
	END

	IF EXISTS (
		SELECT * FROM sysindexes
		WHERE name = N'PK_SecurityPatchUIProductGroup'
		AND ((status & 6162) <> 2050)
	)
	BEGIN
		PRINT 'Recreating existing PK_SecurityPatchUIProductGroup index'
		ALTER TABLE [dbo].[SecurityPatchUIProductGroup]
		DROP CONSTRAINT PK_SecurityPatchUIProductGroup
	END

	IF EXISTS (
		SELECT * FROM sysindexes
		WHERE name = N'UQ_SecurityPatchUIProductGroupByProduct'
		AND ((status & 6162) <> 4098)
	)
	BEGIN
		PRINT 'Recreating existing UQ_SecurityPatchUIProductGroupByProduct index'
		ALTER TABLE [dbo].[SecurityPatchUIProductGroup]
		DROP CONSTRAINT UQ_SecurityPatchUIProductGroupByProduct
	END

	IF NOT EXISTS (
		SELECT * FROM sysindexes
		WHERE name = N'SecurityPatchUIProductGroupByGroup'
	)
		CREATE CLUSTERED INDEX SecurityPatchUIProductGroupByGroup
			ON dbo.SecurityPatchUIProductGroup(GroupName)

	IF NOT EXISTS (
		SELECT * FROM sysindexes
		WHERE name = N'PK_SecurityPatchUIProductGroup'
	)
		ALTER TABLE [dbo].[SecurityPatchUIProductGroup]
		ADD CONSTRAINT PK_SecurityPatchUIProductGroup
			PRIMARY KEY NONCLUSTERED(SecurityPatchUIProductGroupID)

	IF NOT EXISTS (
		SELECT * FROM sysindexes
		WHERE name = N'UQ_SecurityPatchUIProductGroupByProduct'
	)
		ALTER TABLE [dbo].[SecurityPatchUIProductGroup]
		ADD CONSTRAINT UQ_SecurityPatchUIProductGroupByProduct
			UNIQUE NONCLUSTERED(Product)

	END
GO

-- The SecurityPatchProductGroup table provides a mapping between products and ManageSoft environments that are used in the Packer.
IF NOT EXISTS (
	SELECT * FROM INFORMATION_SCHEMA.TABLES
	WHERE TABLE_NAME = N'SecurityPatchProductGroup'
	AND TABLE_SCHEMA = 'dbo'
)
BEGIN
	CREATE TABLE [dbo].[SecurityPatchProductGroup] (
		-- Primary key.
		SecurityPatchProductGroupID int IDENTITY NOT NULL,
		-- The name of the ManageSoft environment to be used when a security patch affects the associated product.
		Environment		nvarchar(64) NOT NULL,
		-- The product that the environment is used for.
		Product			nvarchar(128) NOT NULL,
		CONSTRAINT PK_SecurityPatchProductGroup
			PRIMARY KEY NONCLUSTERED(SecurityPatchProductGroupID)
	);

	CREATE NONCLUSTERED INDEX SecurityPatchProductGroupByEnvironment
		ON dbo.SecurityPatchProductGroup(Environment)

	CREATE CLUSTERED INDEX SecurityPatchProductGroupByProduct
		ON dbo.SecurityPatchProductGroup(Product)

END
ELSE BEGIN
	PRINT 'Using existing SecurityPatchProductGroup table'
	IF EXISTS (
		SELECT * FROM sysindexes
		WHERE name = N'SecurityPatchProductGroupByProduct'
		AND ((status & 6162) <> 16)
	)
	BEGIN
		PRINT 'Recreating existing SecurityPatchProductGroupByProduct index'
		DROP INDEX SecurityPatchProductGroupByProduct
		ON [dbo].[SecurityPatchProductGroup]
	END

	IF EXISTS (
		SELECT * FROM sysindexes
		WHERE name = N'PK_SecurityPatchProductGroup'
		AND ((status & 6162) <> 2050)
	)
	BEGIN
		PRINT 'Recreating existing PK_SecurityPatchProductGroup index'
		ALTER TABLE [dbo].[SecurityPatchProductGroup]
		DROP CONSTRAINT PK_SecurityPatchProductGroup
	END

	IF EXISTS (
		SELECT * FROM sysindexes
		WHERE name = N'SecurityPatchProductGroupByEnvironment'
		AND ((status & 6162) <> 0)
	)
	BEGIN
		PRINT 'Recreating existing SecurityPatchProductGroupByEnvironment index'
		DROP INDEX SecurityPatchProductGroupByEnvironment
		ON [dbo].[SecurityPatchProductGroup]
	END

	IF NOT EXISTS (
		SELECT * FROM sysindexes
		WHERE name = N'SecurityPatchProductGroupByProduct'
	)
		CREATE CLUSTERED INDEX SecurityPatchProductGroupByProduct
			ON dbo.SecurityPatchProductGroup(Product)

	IF NOT EXISTS (
		SELECT * FROM sysindexes
		WHERE name = N'PK_SecurityPatchProductGroup'
	)
		ALTER TABLE [dbo].[SecurityPatchProductGroup]
		ADD CONSTRAINT PK_SecurityPatchProductGroup
			PRIMARY KEY NONCLUSTERED(SecurityPatchProductGroupID)

	IF NOT EXISTS (
		SELECT * FROM sysindexes
		WHERE name = N'SecurityPatchProductGroupByEnvironment'
	)
		CREATE NONCLUSTERED INDEX SecurityPatchProductGroupByEnvironment
			ON dbo.SecurityPatchProductGroup(Environment)

	END
GO

-- The SecurityPatchProductUninstallValues table contains default uninstall commands for products.
IF NOT EXISTS (
	SELECT * FROM INFORMATION_SCHEMA.TABLES
	WHERE TABLE_NAME = N'SecurityPatchProductUninstallValues'
	AND TABLE_SCHEMA = 'dbo'
)
BEGIN
	CREATE TABLE [dbo].[SecurityPatchProductUninstallValues] (
		-- Primary key.
		SecurityPatchProductUninstallValuesID int IDENTITY NOT NULL,
		-- The name of the product to set uninstall values for.
		Product			nvarchar(128) NOT NULL,
		-- The RegistryKey on the target system that contains the uninstall command line.
		RegistryKey		nvarchar(256) NOT NULL,
		-- The command line to uninstall the product. 
		AbsoluteCommandLine	nvarchar(256) NOT NULL,
		-- The registry key in Add/Remove Programs that applies to the product.
		UninstallKey		nvarchar(128) NOT NULL,
		CONSTRAINT PK_SecurityPatchProductUninstallValues
			PRIMARY KEY CLUSTERED(SecurityPatchProductUninstallValuesID)
	);

	CREATE NONCLUSTERED INDEX SecurityPatchProductUninstallValuesByProduct
		ON dbo.SecurityPatchProductUninstallValues(Product)

END
ELSE BEGIN
	PRINT 'Using existing SecurityPatchProductUninstallValues table'
	IF EXISTS (
		SELECT * FROM sysindexes
		WHERE name = N'PK_SecurityPatchProductUninstallValues'
		AND ((status & 6162) <> 2066)
	)
	BEGIN
		PRINT 'Recreating existing PK_SecurityPatchProductUninstallValues index'
		ALTER TABLE [dbo].[SecurityPatchProductUninstallValues]
		DROP CONSTRAINT PK_SecurityPatchProductUninstallValues
	END

	IF EXISTS (
		SELECT * FROM sysindexes
		WHERE name = N'SecurityPatchProductUninstallValuesByProduct'
		AND ((status & 6162) <> 0)
	)
	BEGIN
		PRINT 'Recreating existing SecurityPatchProductUninstallValuesByProduct index'
		DROP INDEX SecurityPatchProductUninstallValuesByProduct
		ON [dbo].[SecurityPatchProductUninstallValues]
	END

	IF NOT EXISTS (
		SELECT * FROM sysindexes
		WHERE name = N'PK_SecurityPatchProductUninstallValues'
	)
		ALTER TABLE [dbo].[SecurityPatchProductUninstallValues]
		ADD CONSTRAINT PK_SecurityPatchProductUninstallValues
			PRIMARY KEY CLUSTERED(SecurityPatchProductUninstallValuesID)

	IF NOT EXISTS (
		SELECT * FROM sysindexes
		WHERE name = N'SecurityPatchProductUninstallValuesByProduct'
	)
		CREATE NONCLUSTERED INDEX SecurityPatchProductUninstallValuesByProduct
			ON dbo.SecurityPatchProductUninstallValues(Product)

	END
GO

-- The SecurityPatchUninstallValues table contains uninstall commands that override the defaults for a security patch.
IF NOT EXISTS (
	SELECT * FROM INFORMATION_SCHEMA.TABLES
	WHERE TABLE_NAME = N'SecurityPatchUninstallValues'
	AND TABLE_SCHEMA = 'dbo'
)
BEGIN
	CREATE TABLE [dbo].[SecurityPatchUninstallValues] (
		-- Primary key.
		SecurityPatchUninstallValuesID int IDENTITY NOT NULL,
		-- Foreign key to the SecurityPatch table.
		SecurityPatchID		int NOT NULL,
		-- The registry key on the target system that contains the uninstall command line.
		RegistryKey		nvarchar(256) NOT NULL,
		-- The command line to uninstall the product. 
		AbsoluteCommandLine	nvarchar(256) NOT NULL,
		-- The registry key in Add/Remove Programs that applies to the product.
		UninstallKey		nvarchar(128) NOT NULL,
		CONSTRAINT FK_SecurityPatchUninstallValues_SecurityPatch
			FOREIGN KEY(SecurityPatchID)
			REFERENCES SecurityPatch(PatchID) ON DELETE CASCADE,
		CONSTRAINT PK_SecurityPatchUninstallValues
			PRIMARY KEY NONCLUSTERED(SecurityPatchUninstallValuesID)
	);

	CREATE CLUSTERED INDEX SecurityPatchUninstallValuesBySecurityPatchID
		ON dbo.SecurityPatchUninstallValues(SecurityPatchID)

END
ELSE BEGIN
	PRINT 'Using existing SecurityPatchUninstallValues table'
	IF EXISTS (
		SELECT * FROM sysindexes
		WHERE name = N'SecurityPatchUninstallValuesBySecurityPatchID'
		AND ((status & 6162) <> 16)
	)
	BEGIN
		PRINT 'Recreating existing SecurityPatchUninstallValuesBySecurityPatchID index'
		DROP INDEX SecurityPatchUninstallValuesBySecurityPatchID
		ON [dbo].[SecurityPatchUninstallValues]
	END

	IF EXISTS (
		SELECT * FROM sysindexes
		WHERE name = N'PK_SecurityPatchUninstallValues'
		AND ((status & 6162) <> 2050)
	)
	BEGIN
		PRINT 'Recreating existing PK_SecurityPatchUninstallValues index'
		ALTER TABLE [dbo].[SecurityPatchUninstallValues]
		DROP CONSTRAINT PK_SecurityPatchUninstallValues
	END

	IF NOT EXISTS (
		SELECT * FROM sysindexes
		WHERE name = N'SecurityPatchUninstallValuesBySecurityPatchID'
	)
		CREATE CLUSTERED INDEX SecurityPatchUninstallValuesBySecurityPatchID
			ON dbo.SecurityPatchUninstallValues(SecurityPatchID)

	IF NOT EXISTS (
		SELECT * FROM sysindexes
		WHERE name = N'PK_SecurityPatchUninstallValues'
	)
		ALTER TABLE [dbo].[SecurityPatchUninstallValues]
		ADD CONSTRAINT PK_SecurityPatchUninstallValues
			PRIMARY KEY NONCLUSTERED(SecurityPatchUninstallValuesID)

	IF EXISTS(
		SELECT *
		FROM INFORMATION_SCHEMA.TABLES
		WHERE TABLE_NAME = 'SecurityPatchUninstallValues'
		AND TABLE_SCHEMA = 'dbo'
	)
	AND EXISTS(
		SELECT *
		FROM INFORMATION_SCHEMA.TABLES
		WHERE TABLE_NAME = 'SecurityPatch'
		AND TABLE_SCHEMA = 'dbo'
	)	
	AND NOT EXISTS (
		SELECT *
		FROM INFORMATION_SCHEMA.CONSTRAINT_COLUMN_USAGE
		WHERE CONSTRAINT_NAME = 'FK_SecurityPatchUninstallValues_SecurityPatch'
	)
	BEGIN
		PRINT 'Create FK_SecurityPatchUninstallValues_SecurityPatch constraint'
	
		ALTER TABLE [dbo].[SecurityPatchUninstallValues]
			ADD CONSTRAINT FK_SecurityPatchUninstallValues_SecurityPatch
			FOREIGN KEY(SecurityPatchID)
			REFERENCES SecurityPatch(PatchID) ON DELETE CASCADE
	END
END
GO

-- A resource (usually a file) that constitutes part of or the whole of a security patch.
IF NOT EXISTS (
	SELECT * FROM INFORMATION_SCHEMA.TABLES
	WHERE TABLE_NAME = N'SecurityPatchResource'
	AND TABLE_SCHEMA = 'dbo'
)
BEGIN
	CREATE TABLE [dbo].[SecurityPatchResource] (
		-- Artificial key.
		SecurityPatchResourceID	int IDENTITY NOT NULL,
		-- The security patch that the resource belongs to.
		SecurityPatchID		int NOT NULL,
		-- Original URL of the resource.
		ResourceURL		nvarchar(2000) NOT NULL,
		-- The architecture that the resource is built for.
		ArchitectureID		int NOT NULL,
		-- The language that the resource is built for.
		LanguageID		nvarchar(6) NOT NULL DEFAULT 'eng',
		-- The checksum for the resource.
		Checksum		nvarchar(256) NULL,
		-- The actual URL of the resource.
		ActualResourceURL	nvarchar(2000) NULL,
		-- The local path of the resource.
		LocalPath		nvarchar(256) NULL,
		-- Whether this resource should be included as part of the patch.
		Incorporate		bit NOT NULL DEFAULT 1,
		-- The name of the security bulletin.
		BulletinID		nvarchar(32) NOT NULL,
		-- The name of the security patch resource.
		PatchResourceName	nvarchar(512) NULL,
		CONSTRAINT FK_SecurityPatchResource_SecurityPatchID
			FOREIGN KEY(SecurityPatchID)
			REFERENCES SecurityPatch(PatchID) ON DELETE CASCADE,
		CONSTRAINT FK_SecurityPatchResource_BulletinID
			FOREIGN KEY(BulletinID)
			REFERENCES SecurityBulletin(BulletinID) ON DELETE NO ACTION,
		CONSTRAINT PK_SecurityPatchResource
			PRIMARY KEY NONCLUSTERED
			(SecurityPatchID, ResourceURL, ArchitectureID, LanguageID),
		CONSTRAINT UQ_SecurityPatchResource
			UNIQUE CLUSTERED(SecurityPatchResourceID)
	);

	CREATE NONCLUSTERED INDEX SecurityPatchResourceByBulletinID
		ON dbo.SecurityPatchResource(BulletinID)

	CREATE NONCLUSTERED INDEX SecurityPatchResourceByLanguageID
		ON dbo.SecurityPatchResource(LanguageID)

	CREATE NONCLUSTERED INDEX SecurityPatchResourceByArchitectureID
		ON dbo.SecurityPatchResource(ArchitectureID)

END
ELSE BEGIN
	PRINT 'Using existing SecurityPatchResource table'
	IF EXISTS (
		SELECT * FROM sysindexes
		WHERE name = N'UQ_SecurityPatchResource'
		AND ((status & 6162) <> 4114)
	)
	BEGIN
		PRINT 'Recreating existing UQ_SecurityPatchResource index'
		ALTER TABLE [dbo].[SecurityPatchResource]
		DROP CONSTRAINT UQ_SecurityPatchResource
	END

	IF EXISTS (
		SELECT * FROM sysindexes
		WHERE name = N'PK_SecurityPatchResource'
		AND ((status & 6162) <> 2050)
	)
	BEGIN
		PRINT 'Recreating existing PK_SecurityPatchResource index'
		ALTER TABLE [dbo].[SecurityPatchResource]
		DROP CONSTRAINT PK_SecurityPatchResource
	END

	IF EXISTS (
		SELECT * FROM sysindexes
		WHERE name = N'SecurityPatchResourceByBulletinID'
		AND ((status & 6162) <> 0)
	)
	BEGIN
		PRINT 'Recreating existing SecurityPatchResourceByBulletinID index'
		DROP INDEX SecurityPatchResourceByBulletinID
		ON [dbo].[SecurityPatchResource]
	END

	IF EXISTS (
		SELECT * FROM sysindexes
		WHERE name = N'SecurityPatchResourceByLanguageID'
		AND ((status & 6162) <> 0)
	)
	BEGIN
		PRINT 'Recreating existing SecurityPatchResourceByLanguageID index'
		DROP INDEX SecurityPatchResourceByLanguageID
		ON [dbo].[SecurityPatchResource]
	END

	IF EXISTS (
		SELECT * FROM sysindexes
		WHERE name = N'SecurityPatchResourceByArchitectureID'
		AND ((status & 6162) <> 0)
	)
	BEGIN
		PRINT 'Recreating existing SecurityPatchResourceByArchitectureID index'
		DROP INDEX SecurityPatchResourceByArchitectureID
		ON [dbo].[SecurityPatchResource]
	END

	IF NOT EXISTS (
		SELECT * FROM sysindexes
		WHERE name = N'UQ_SecurityPatchResource'
	)
		ALTER TABLE [dbo].[SecurityPatchResource]
		ADD CONSTRAINT UQ_SecurityPatchResource
			UNIQUE CLUSTERED(SecurityPatchResourceID)

	IF NOT EXISTS (
		SELECT * FROM sysindexes
		WHERE name = N'PK_SecurityPatchResource'
	)
		ALTER TABLE [dbo].[SecurityPatchResource]
		ADD CONSTRAINT PK_SecurityPatchResource
			PRIMARY KEY NONCLUSTERED
			(SecurityPatchID, ResourceURL, ArchitectureID, LanguageID)

	IF NOT EXISTS (
		SELECT * FROM sysindexes
		WHERE name = N'SecurityPatchResourceByBulletinID'
	)
		CREATE NONCLUSTERED INDEX SecurityPatchResourceByBulletinID
			ON dbo.SecurityPatchResource(BulletinID)

	IF NOT EXISTS (
		SELECT * FROM sysindexes
		WHERE name = N'SecurityPatchResourceByLanguageID'
	)
		CREATE NONCLUSTERED INDEX SecurityPatchResourceByLanguageID
			ON dbo.SecurityPatchResource(LanguageID)

	IF NOT EXISTS (
		SELECT * FROM sysindexes
		WHERE name = N'SecurityPatchResourceByArchitectureID'
	)
		CREATE NONCLUSTERED INDEX SecurityPatchResourceByArchitectureID
			ON dbo.SecurityPatchResource(ArchitectureID)

	IF EXISTS(
		SELECT *
		FROM INFORMATION_SCHEMA.TABLES
		WHERE TABLE_NAME = 'SecurityPatchResource'
		AND TABLE_SCHEMA = 'dbo'
	)
	AND EXISTS(
		SELECT *
		FROM INFORMATION_SCHEMA.TABLES
		WHERE TABLE_NAME = 'SecurityPatch'
		AND TABLE_SCHEMA = 'dbo'
	)	
	AND NOT EXISTS (
		SELECT *
		FROM INFORMATION_SCHEMA.CONSTRAINT_COLUMN_USAGE
		WHERE CONSTRAINT_NAME = 'FK_SecurityPatchResource_SecurityPatchID'
	)
	BEGIN
		PRINT 'Create FK_SecurityPatchResource_SecurityPatchID constraint'
	
		ALTER TABLE [dbo].[SecurityPatchResource]
			ADD CONSTRAINT FK_SecurityPatchResource_SecurityPatchID
			FOREIGN KEY(SecurityPatchID)
			REFERENCES SecurityPatch(PatchID) ON DELETE CASCADE
	END
	
	IF EXISTS(
		SELECT *
		FROM INFORMATION_SCHEMA.TABLES
		WHERE TABLE_NAME = 'SecurityPatchResource'
		AND TABLE_SCHEMA = 'dbo'
	)
	AND EXISTS(
		SELECT *
		FROM INFORMATION_SCHEMA.TABLES
		WHERE TABLE_NAME = 'SecurityBulletin'
		AND TABLE_SCHEMA = 'dbo'
	)	
	AND NOT EXISTS (
		SELECT *
		FROM INFORMATION_SCHEMA.CONSTRAINT_COLUMN_USAGE
		WHERE CONSTRAINT_NAME = 'FK_SecurityPatchResource_BulletinID'
	)
	BEGIN
		PRINT 'Create FK_SecurityPatchResource_BulletinID constraint'
	
		ALTER TABLE [dbo].[SecurityPatchResource]
			ADD CONSTRAINT FK_SecurityPatchResource_BulletinID
			FOREIGN KEY(BulletinID)
			REFERENCES SecurityBulletin(BulletinID) ON DELETE NO ACTION
	END
END
GO

-- The SecurityPatchLanguage table contains the list of supported and visible languages.
IF NOT EXISTS (
	SELECT * FROM INFORMATION_SCHEMA.TABLES
	WHERE TABLE_NAME = N'SecurityPatchLanguage'
	AND TABLE_SCHEMA = 'dbo'
)
BEGIN
	CREATE TABLE [dbo].[SecurityPatchLanguage] (
		-- Primary key.
		LanguageName		nvarchar(6) NOT NULL,
		-- A friendly name for the language.
		Description		nvarchar(128) NOT NULL,
		-- If it is set to true then it is supported by ManageSoft Security Patch Management.
		Selected		bit NOT NULL,
		-- If it is set to true then it is shown on the properties dialog.
		Visible			bit NOT NULL,
		CONSTRAINT PK_SecurityPatchLanguage
			PRIMARY KEY CLUSTERED(LanguageName)
	);

	CREATE NONCLUSTERED INDEX SecurityPatchLanguageBySelected
		ON dbo.SecurityPatchLanguage(Selected)

END
ELSE BEGIN
	PRINT 'Using existing SecurityPatchLanguage table'
	IF EXISTS (
		SELECT * FROM sysindexes
		WHERE name = N'PK_SecurityPatchLanguage'
		AND ((status & 6162) <> 2066)
	)
	BEGIN
		PRINT 'Recreating existing PK_SecurityPatchLanguage index'
		ALTER TABLE [dbo].[SecurityPatchLanguage]
		DROP CONSTRAINT PK_SecurityPatchLanguage
	END

	IF EXISTS (
		SELECT * FROM sysindexes
		WHERE name = N'SecurityPatchLanguageBySelected'
		AND ((status & 6162) <> 0)
	)
	BEGIN
		PRINT 'Recreating existing SecurityPatchLanguageBySelected index'
		DROP INDEX SecurityPatchLanguageBySelected
		ON [dbo].[SecurityPatchLanguage]
	END

	IF NOT EXISTS (
		SELECT * FROM sysindexes
		WHERE name = N'PK_SecurityPatchLanguage'
	)
		ALTER TABLE [dbo].[SecurityPatchLanguage]
		ADD CONSTRAINT PK_SecurityPatchLanguage
			PRIMARY KEY CLUSTERED(LanguageName)

	IF NOT EXISTS (
		SELECT * FROM sysindexes
		WHERE name = N'SecurityPatchLanguageBySelected'
	)
		CREATE NONCLUSTERED INDEX SecurityPatchLanguageBySelected
			ON dbo.SecurityPatchLanguage(Selected)

	END
GO

-- The SecurityPatchOperatingSystem table contains the list of supported and visible operating systems.
IF NOT EXISTS (
	SELECT * FROM INFORMATION_SCHEMA.TABLES
	WHERE TABLE_NAME = N'SecurityPatchOperatingSystem'
	AND TABLE_SCHEMA = 'dbo'
)
BEGIN
	CREATE TABLE [dbo].[SecurityPatchOperatingSystem] (
		-- Operating system id.
		OperatingSystemID	int NOT NULL,
		-- A friendly name for the operating system.
		Description		nvarchar(128) NOT NULL,
		-- If it is set to true then it is supported by ManageSoft Security Patch Management.
		Selected		bit NOT NULL,
		-- If it is set to true then it is shown on the properties dialog.
		Visible			bit NOT NULL,
		CONSTRAINT FK_SecurityPatchOperatingSystem_SecurityPatchUIProductGroup
			FOREIGN KEY(OperatingSystemID)
			REFERENCES SecurityPatchUIProductGroup(SecurityPatchUIProductGroupID) ON DELETE CASCADE,
		CONSTRAINT PK_SecurityPatchOperatingSystem
			PRIMARY KEY CLUSTERED(OperatingSystemID)
	);

END
ELSE BEGIN
	PRINT 'Using existing SecurityPatchOperatingSystem table'
	IF EXISTS (
		SELECT * FROM sysindexes
		WHERE name = N'PK_SecurityPatchOperatingSystem'
		AND ((status & 6162) <> 2066)
	)
	BEGIN
		PRINT 'Recreating existing PK_SecurityPatchOperatingSystem index'
		ALTER TABLE [dbo].[SecurityPatchOperatingSystem]
		DROP CONSTRAINT PK_SecurityPatchOperatingSystem
	END

	IF NOT EXISTS (
		SELECT * FROM sysindexes
		WHERE name = N'PK_SecurityPatchOperatingSystem'
	)
		ALTER TABLE [dbo].[SecurityPatchOperatingSystem]
		ADD CONSTRAINT PK_SecurityPatchOperatingSystem
			PRIMARY KEY CLUSTERED(OperatingSystemID)

	IF EXISTS(
		SELECT *
		FROM INFORMATION_SCHEMA.TABLES
		WHERE TABLE_NAME = 'SecurityPatchOperatingSystem'
		AND TABLE_SCHEMA = 'dbo'
	)
	AND EXISTS(
		SELECT *
		FROM INFORMATION_SCHEMA.TABLES
		WHERE TABLE_NAME = 'SecurityPatchUIProductGroup'
		AND TABLE_SCHEMA = 'dbo'
	)	
	AND NOT EXISTS (
		SELECT *
		FROM INFORMATION_SCHEMA.CONSTRAINT_COLUMN_USAGE
		WHERE CONSTRAINT_NAME = 'FK_SecurityPatchOperatingSystem_SecurityPatchUIProductGroup'
	)
	BEGIN
		PRINT 'Create FK_SecurityPatchOperatingSystem_SecurityPatchUIProductGroup constraint'
	
		ALTER TABLE [dbo].[SecurityPatchOperatingSystem]
			ADD CONSTRAINT FK_SecurityPatchOperatingSystem_SecurityPatchUIProductGroup
			FOREIGN KEY(OperatingSystemID)
			REFERENCES SecurityPatchUIProductGroup(SecurityPatchUIProductGroupID) ON DELETE CASCADE
	END
END
GO

-- The SecurityPatchArchitecture table contains the list of supported and visible architectures.
IF NOT EXISTS (
	SELECT * FROM INFORMATION_SCHEMA.TABLES
	WHERE TABLE_NAME = N'SecurityPatchArchitecture'
	AND TABLE_SCHEMA = 'dbo'
)
BEGIN
	CREATE TABLE [dbo].[SecurityPatchArchitecture] (
		-- Architecture id.
		ArchitectureID		int NOT NULL,
		-- A friendly name for the architecture.
		Description		nvarchar(128) NOT NULL,
		-- If it is set to true then it is supported by ManageSoft Security Patch Management.
		Selected		bit NOT NULL,
		-- If it is set to true then it is shown on the properties dialog.
		Visible			bit NOT NULL,
		CONSTRAINT PK_SecurityPatchArchitecture
			PRIMARY KEY CLUSTERED(ArchitectureID)
	);

	CREATE NONCLUSTERED INDEX SecurityPatchArchitectureBySelected
		ON dbo.SecurityPatchArchitecture(Selected)

END
ELSE BEGIN
	PRINT 'Using existing SecurityPatchArchitecture table'
	IF EXISTS (
		SELECT * FROM sysindexes
		WHERE name = N'PK_SecurityPatchArchitecture'
		AND ((status & 6162) <> 2066)
	)
	BEGIN
		PRINT 'Recreating existing PK_SecurityPatchArchitecture index'
		ALTER TABLE [dbo].[SecurityPatchArchitecture]
		DROP CONSTRAINT PK_SecurityPatchArchitecture
	END

	IF EXISTS (
		SELECT * FROM sysindexes
		WHERE name = N'SecurityPatchArchitectureBySelected'
		AND ((status & 6162) <> 0)
	)
	BEGIN
		PRINT 'Recreating existing SecurityPatchArchitectureBySelected index'
		DROP INDEX SecurityPatchArchitectureBySelected
		ON [dbo].[SecurityPatchArchitecture]
	END

	IF NOT EXISTS (
		SELECT * FROM sysindexes
		WHERE name = N'PK_SecurityPatchArchitecture'
	)
		ALTER TABLE [dbo].[SecurityPatchArchitecture]
		ADD CONSTRAINT PK_SecurityPatchArchitecture
			PRIMARY KEY CLUSTERED(ArchitectureID)

	IF NOT EXISTS (
		SELECT * FROM sysindexes
		WHERE name = N'SecurityPatchArchitectureBySelected'
	)
		CREATE NONCLUSTERED INDEX SecurityPatchArchitectureBySelected
			ON dbo.SecurityPatchArchitecture(Selected)

	END
GO

-- The SecurityPatchModel table contains the list of supported and visible update models.
IF NOT EXISTS (
	SELECT * FROM INFORMATION_SCHEMA.TABLES
	WHERE TABLE_NAME = N'SecurityPatchModel'
	AND TABLE_SCHEMA = 'dbo'
)
BEGIN
	CREATE TABLE [dbo].[SecurityPatchModel] (
		-- Model id.
		ModelID			int NOT NULL,
		-- A friendly name for the model.
		Description		nvarchar(128) NOT NULL,
		-- If it is set to true then it is supported by ManageSoft Security Patch Management.
		Selected		bit NOT NULL,
		-- If it is set to true then it is shown on the properties dialog.
		Visible			bit NOT NULL,
		CONSTRAINT PK_SecurityPatchModel
			PRIMARY KEY CLUSTERED(ModelID)
	);

	CREATE NONCLUSTERED INDEX SecurityPatchModelBySelected
		ON dbo.SecurityPatchModel(Selected)

END
ELSE BEGIN
	PRINT 'Using existing SecurityPatchModel table'
	IF EXISTS (
		SELECT * FROM sysindexes
		WHERE name = N'PK_SecurityPatchModel'
		AND ((status & 6162) <> 2066)
	)
	BEGIN
		PRINT 'Recreating existing PK_SecurityPatchModel index'
		ALTER TABLE [dbo].[SecurityPatchModel]
		DROP CONSTRAINT PK_SecurityPatchModel
	END

	IF EXISTS (
		SELECT * FROM sysindexes
		WHERE name = N'SecurityPatchModelBySelected'
		AND ((status & 6162) <> 0)
	)
	BEGIN
		PRINT 'Recreating existing SecurityPatchModelBySelected index'
		DROP INDEX SecurityPatchModelBySelected
		ON [dbo].[SecurityPatchModel]
	END

	IF NOT EXISTS (
		SELECT * FROM sysindexes
		WHERE name = N'PK_SecurityPatchModel'
	)
		ALTER TABLE [dbo].[SecurityPatchModel]
		ADD CONSTRAINT PK_SecurityPatchModel
			PRIMARY KEY CLUSTERED(ModelID)

	IF NOT EXISTS (
		SELECT * FROM sysindexes
		WHERE name = N'SecurityPatchModelBySelected'
	)
		CREATE NONCLUSTERED INDEX SecurityPatchModelBySelected
			ON dbo.SecurityPatchModel(Selected)

	END
GO

-- The SecurityPatchApplication table contains the list of supported and visible applications.
IF NOT EXISTS (
	SELECT * FROM INFORMATION_SCHEMA.TABLES
	WHERE TABLE_NAME = N'SecurityPatchApplication'
	AND TABLE_SCHEMA = 'dbo'
)
BEGIN
	CREATE TABLE [dbo].[SecurityPatchApplication] (
		-- Application id.
		ApplicationID		int NOT NULL,
		-- A friendly name for the application.
		Description		nvarchar(128) NOT NULL,
		-- If it is set to true then it is supported by ManageSoft Security Patch Management.
		Selected		bit NOT NULL,
		-- If it is set to true then it is shown on the properties dialog.
		Visible			bit NOT NULL,
		CONSTRAINT FK_SecurityPatchApplication_SecurityPatchUIProductGroup
			FOREIGN KEY(ApplicationID)
			REFERENCES SecurityPatchUIProductGroup(SecurityPatchUIProductGroupID) ON DELETE CASCADE,
		CONSTRAINT PK_SecurityPatchApplication
			PRIMARY KEY CLUSTERED(ApplicationID)
	);

END
ELSE BEGIN
	PRINT 'Using existing SecurityPatchApplication table'
	IF EXISTS (
		SELECT * FROM sysindexes
		WHERE name = N'PK_SecurityPatchApplication'
		AND ((status & 6162) <> 2066)
	)
	BEGIN
		PRINT 'Recreating existing PK_SecurityPatchApplication index'
		ALTER TABLE [dbo].[SecurityPatchApplication]
		DROP CONSTRAINT PK_SecurityPatchApplication
	END

	IF NOT EXISTS (
		SELECT * FROM sysindexes
		WHERE name = N'PK_SecurityPatchApplication'
	)
		ALTER TABLE [dbo].[SecurityPatchApplication]
		ADD CONSTRAINT PK_SecurityPatchApplication
			PRIMARY KEY CLUSTERED(ApplicationID)

	IF EXISTS(
		SELECT *
		FROM INFORMATION_SCHEMA.TABLES
		WHERE TABLE_NAME = 'SecurityPatchApplication'
		AND TABLE_SCHEMA = 'dbo'
	)
	AND EXISTS(
		SELECT *
		FROM INFORMATION_SCHEMA.TABLES
		WHERE TABLE_NAME = 'SecurityPatchUIProductGroup'
		AND TABLE_SCHEMA = 'dbo'
	)	
	AND NOT EXISTS (
		SELECT *
		FROM INFORMATION_SCHEMA.CONSTRAINT_COLUMN_USAGE
		WHERE CONSTRAINT_NAME = 'FK_SecurityPatchApplication_SecurityPatchUIProductGroup'
	)
	BEGIN
		PRINT 'Create FK_SecurityPatchApplication_SecurityPatchUIProductGroup constraint'
	
		ALTER TABLE [dbo].[SecurityPatchApplication]
			ADD CONSTRAINT FK_SecurityPatchApplication_SecurityPatchUIProductGroup
			FOREIGN KEY(ApplicationID)
			REFERENCES SecurityPatchUIProductGroup(SecurityPatchUIProductGroupID) ON DELETE CASCADE
	END
END
GO

-- The SecurityAutomationTask table contains automation rules.
IF NOT EXISTS (
	SELECT * FROM INFORMATION_SCHEMA.TABLES
	WHERE TABLE_NAME = N'SecurityAutomationTask'
	AND TABLE_SCHEMA = 'dbo'
)
BEGIN
	CREATE TABLE [dbo].[SecurityAutomationTask] (
		-- Primary key.
		ScheduleID		int IDENTITY NOT NULL,
		-- The name of this schedule.
		TaskName		nvarchar(128) COLLATE Latin1_General_CS_AS NOT NULL,
		-- The Id of the Automation Rule.
		GroupID			nvarchar(128) NOT NULL,
		-- The name of the Automation Rule.
		GroupName		nvarchar(128) NOT NULL,
		-- Initial Delay Days of the Add to Policy Automation Schedule.
		ATPDelay		int NOT NULL,
		-- The Delay Days of the Automation Rule.
		DelayDays		int NOT NULL,
		-- The Order of this Scheduled Task within its Automation Task.
		[Order]			int NOT NULL,
		-- Determines whether the task is sync task or not.
		IsSyncTask		bit NOT NULL,
		CONSTRAINT PK_ScheduleID
			PRIMARY KEY CLUSTERED(ScheduleID),
		CONSTRAINT UQ_Task
			UNIQUE NONCLUSTERED(TaskName)
	);

	CREATE NONCLUSTERED INDEX UQ_GroupID
		ON dbo.SecurityAutomationTask(GroupID)

END
ELSE BEGIN
	PRINT 'Using existing SecurityAutomationTask table'
	IF EXISTS (
		SELECT * FROM sysindexes
		WHERE name = N'PK_ScheduleID'
		AND ((status & 6162) <> 2066)
	)
	BEGIN
		PRINT 'Recreating existing PK_ScheduleID index'
		ALTER TABLE [dbo].[SecurityAutomationTask]
		DROP CONSTRAINT PK_ScheduleID
	END

	IF EXISTS (
		SELECT * FROM sysindexes
		WHERE name = N'UQ_Task'
		AND ((status & 6162) <> 4098)
	)
	BEGIN
		PRINT 'Recreating existing UQ_Task index'
		ALTER TABLE [dbo].[SecurityAutomationTask]
		DROP CONSTRAINT UQ_Task
	END

	IF EXISTS (
		SELECT * FROM sysindexes
		WHERE name = N'UQ_GroupID'
		AND ((status & 6162) <> 0)
	)
	BEGIN
		PRINT 'Recreating existing UQ_GroupID index'
		DROP INDEX UQ_GroupID
		ON [dbo].[SecurityAutomationTask]
	END

	IF NOT EXISTS (
		SELECT * FROM sysindexes
		WHERE name = N'PK_ScheduleID'
	)
		ALTER TABLE [dbo].[SecurityAutomationTask]
		ADD CONSTRAINT PK_ScheduleID
			PRIMARY KEY CLUSTERED(ScheduleID)

	IF NOT EXISTS (
		SELECT * FROM sysindexes
		WHERE name = N'UQ_Task'
	)
		ALTER TABLE [dbo].[SecurityAutomationTask]
		ADD CONSTRAINT UQ_Task
			UNIQUE NONCLUSTERED(TaskName)

	IF NOT EXISTS (
		SELECT * FROM sysindexes
		WHERE name = N'UQ_GroupID'
	)
		CREATE NONCLUSTERED INDEX UQ_GroupID
			ON dbo.SecurityAutomationTask(GroupID)

	END
GO

-- The SecurityBulletinChange table contains the list of changes made to a bulletin by the automatic bulletin update.
IF NOT EXISTS (
	SELECT * FROM INFORMATION_SCHEMA.TABLES
	WHERE TABLE_NAME = N'SecurityBulletinChange'
	AND TABLE_SCHEMA = 'dbo'
)
BEGIN
	CREATE TABLE [dbo].[SecurityBulletinChange] (
		-- ID of modified bulletin.
		BulletinID		nvarchar(32) NOT NULL,
		-- Type of change that was made.
		ChangeTypeID		int NOT NULL,
		CONSTRAINT PK_SecurityBulletinChange
			PRIMARY KEY CLUSTERED(BulletinID, ChangeTypeID)
	);

	CREATE NONCLUSTERED INDEX SecurityBulletinChangeByChangeTypeID
		ON dbo.SecurityBulletinChange(ChangeTypeID)

END
ELSE BEGIN
	PRINT 'Using existing SecurityBulletinChange table'
	IF EXISTS (
		SELECT * FROM sysindexes
		WHERE name = N'PK_SecurityBulletinChange'
		AND ((status & 6162) <> 2066)
	)
	BEGIN
		PRINT 'Recreating existing PK_SecurityBulletinChange index'
		ALTER TABLE [dbo].[SecurityBulletinChange]
		DROP CONSTRAINT PK_SecurityBulletinChange
	END

	IF EXISTS (
		SELECT * FROM sysindexes
		WHERE name = N'SecurityBulletinChangeByChangeTypeID'
		AND ((status & 6162) <> 0)
	)
	BEGIN
		PRINT 'Recreating existing SecurityBulletinChangeByChangeTypeID index'
		DROP INDEX SecurityBulletinChangeByChangeTypeID
		ON [dbo].[SecurityBulletinChange]
	END

	IF NOT EXISTS (
		SELECT * FROM sysindexes
		WHERE name = N'PK_SecurityBulletinChange'
	)
		ALTER TABLE [dbo].[SecurityBulletinChange]
		ADD CONSTRAINT PK_SecurityBulletinChange
			PRIMARY KEY CLUSTERED(BulletinID, ChangeTypeID)

	IF NOT EXISTS (
		SELECT * FROM sysindexes
		WHERE name = N'SecurityBulletinChangeByChangeTypeID'
	)
		CREATE NONCLUSTERED INDEX SecurityBulletinChangeByChangeTypeID
			ON dbo.SecurityBulletinChange(ChangeTypeID)

	END
GO

-- The SecurityBulletinChangeType table contains the list of available changes that can be made to a bulletin by the automatic bulletin update.
IF NOT EXISTS (
	SELECT * FROM INFORMATION_SCHEMA.TABLES
	WHERE TABLE_NAME = N'SecurityBulletinChangeType'
	AND TABLE_SCHEMA = 'dbo'
)
BEGIN
	CREATE TABLE [dbo].[SecurityBulletinChangeType] (
		-- Bulletin change type ID.
		SecurityBulletinChangeTypeID int IDENTITY NOT NULL,
		-- Description of change to a bulletin.
		Description		nvarchar(64) NOT NULL,
		CONSTRAINT PK_SecurityBulletinChangeType
			PRIMARY KEY CLUSTERED(SecurityBulletinChangeTypeID),
		CONSTRAINT UQ_SecurityBulletinChangeTypeByDescription
			UNIQUE NONCLUSTERED(Description)
	);

END
ELSE BEGIN
	PRINT 'Using existing SecurityBulletinChangeType table'
	IF EXISTS (
		SELECT * FROM sysindexes
		WHERE name = N'PK_SecurityBulletinChangeType'
		AND ((status & 6162) <> 2066)
	)
	BEGIN
		PRINT 'Recreating existing PK_SecurityBulletinChangeType index'
		ALTER TABLE [dbo].[SecurityBulletinChangeType]
		DROP CONSTRAINT PK_SecurityBulletinChangeType
	END

	IF EXISTS (
		SELECT * FROM sysindexes
		WHERE name = N'UQ_SecurityBulletinChangeTypeByDescription'
		AND ((status & 6162) <> 4098)
	)
	BEGIN
		PRINT 'Recreating existing UQ_SecurityBulletinChangeTypeByDescription index'
		ALTER TABLE [dbo].[SecurityBulletinChangeType]
		DROP CONSTRAINT UQ_SecurityBulletinChangeTypeByDescription
	END

	IF NOT EXISTS (
		SELECT * FROM sysindexes
		WHERE name = N'PK_SecurityBulletinChangeType'
	)
		ALTER TABLE [dbo].[SecurityBulletinChangeType]
		ADD CONSTRAINT PK_SecurityBulletinChangeType
			PRIMARY KEY CLUSTERED(SecurityBulletinChangeTypeID)

	IF NOT EXISTS (
		SELECT * FROM sysindexes
		WHERE name = N'UQ_SecurityBulletinChangeTypeByDescription'
	)
		ALTER TABLE [dbo].[SecurityBulletinChangeType]
		ADD CONSTRAINT UQ_SecurityBulletinChangeTypeByDescription
			UNIQUE NONCLUSTERED(Description)

	END
GO

-- List of threat types.
IF NOT EXISTS (
	SELECT * FROM INFORMATION_SCHEMA.TABLES
	WHERE TABLE_NAME = N'ThreatType'
	AND TABLE_SCHEMA = 'dbo'
)
BEGIN
	CREATE TABLE [dbo].[ThreatType] (
		-- Threat type ID
		ThreatTypeID		int IDENTITY NOT NULL,
		-- Threat type name
		[Name]			nvarchar(64) NOT NULL,
		CONSTRAINT PK_ThreatType
			PRIMARY KEY NONCLUSTERED([Name]),
		CONSTRAINT UQ_ThreatType
			UNIQUE CLUSTERED(ThreatTypeID)
	);

END
ELSE BEGIN
	PRINT 'Using existing ThreatType table'
	IF EXISTS (
		SELECT * FROM sysindexes
		WHERE name = N'UQ_ThreatType'
		AND ((status & 6162) <> 4114)
	)
	BEGIN
		PRINT 'Recreating existing UQ_ThreatType index'
		ALTER TABLE [dbo].[ThreatType]
		DROP CONSTRAINT UQ_ThreatType
	END

	IF EXISTS (
		SELECT * FROM sysindexes
		WHERE name = N'PK_ThreatType'
		AND ((status & 6162) <> 2050)
	)
	BEGIN
		PRINT 'Recreating existing PK_ThreatType index'
		ALTER TABLE [dbo].[ThreatType]
		DROP CONSTRAINT PK_ThreatType
	END

	IF NOT EXISTS (
		SELECT * FROM sysindexes
		WHERE name = N'UQ_ThreatType'
	)
		ALTER TABLE [dbo].[ThreatType]
		ADD CONSTRAINT UQ_ThreatType
			UNIQUE CLUSTERED(ThreatTypeID)

	IF NOT EXISTS (
		SELECT * FROM sysindexes
		WHERE name = N'PK_ThreatType'
	)
		ALTER TABLE [dbo].[ThreatType]
		ADD CONSTRAINT PK_ThreatType
			PRIMARY KEY NONCLUSTERED([Name])

	END
GO

-- List of threat categories.
IF NOT EXISTS (
	SELECT * FROM INFORMATION_SCHEMA.TABLES
	WHERE TABLE_NAME = N'ThreatCategory'
	AND TABLE_SCHEMA = 'dbo'
)
BEGIN
	CREATE TABLE [dbo].[ThreatCategory] (
		-- Threat category ID
		ThreatCategoryID	int IDENTITY NOT NULL,
		-- Threat category name
		[Name]			nvarchar(64) NOT NULL,
		-- <comment/>
		ThreatTypeID		int NOT NULL,
		CONSTRAINT FK_ThreatCategory_ThreatType
			FOREIGN KEY(ThreatTypeID)
			REFERENCES ThreatType(ThreatTypeID) ON DELETE NO ACTION,
		CONSTRAINT PK_ThreatCategory
			PRIMARY KEY NONCLUSTERED([Name]),
		CONSTRAINT UQ_ThreatCategory
			UNIQUE CLUSTERED(ThreatCategoryID)
	);

	CREATE NONCLUSTERED INDEX ThreatCategoryByThreatTypeID
		ON dbo.ThreatCategory(ThreatTypeID)

END
ELSE BEGIN
	PRINT 'Using existing ThreatCategory table'
	IF EXISTS (
		SELECT * FROM sysindexes
		WHERE name = N'UQ_ThreatCategory'
		AND ((status & 6162) <> 4114)
	)
	BEGIN
		PRINT 'Recreating existing UQ_ThreatCategory index'
		ALTER TABLE [dbo].[ThreatCategory]
		DROP CONSTRAINT UQ_ThreatCategory
	END

	IF EXISTS (
		SELECT * FROM sysindexes
		WHERE name = N'PK_ThreatCategory'
		AND ((status & 6162) <> 2050)
	)
	BEGIN
		PRINT 'Recreating existing PK_ThreatCategory index'
		ALTER TABLE [dbo].[ThreatCategory]
		DROP CONSTRAINT PK_ThreatCategory
	END

	IF EXISTS (
		SELECT * FROM sysindexes
		WHERE name = N'ThreatCategoryByThreatTypeID'
		AND ((status & 6162) <> 0)
	)
	BEGIN
		PRINT 'Recreating existing ThreatCategoryByThreatTypeID index'
		DROP INDEX ThreatCategoryByThreatTypeID
		ON [dbo].[ThreatCategory]
	END

	IF NOT EXISTS (
		SELECT * FROM sysindexes
		WHERE name = N'UQ_ThreatCategory'
	)
		ALTER TABLE [dbo].[ThreatCategory]
		ADD CONSTRAINT UQ_ThreatCategory
			UNIQUE CLUSTERED(ThreatCategoryID)

	IF NOT EXISTS (
		SELECT * FROM sysindexes
		WHERE name = N'PK_ThreatCategory'
	)
		ALTER TABLE [dbo].[ThreatCategory]
		ADD CONSTRAINT PK_ThreatCategory
			PRIMARY KEY NONCLUSTERED([Name])

	IF NOT EXISTS (
		SELECT * FROM sysindexes
		WHERE name = N'ThreatCategoryByThreatTypeID'
	)
		CREATE NONCLUSTERED INDEX ThreatCategoryByThreatTypeID
			ON dbo.ThreatCategory(ThreatTypeID)

	IF EXISTS(
		SELECT *
		FROM INFORMATION_SCHEMA.TABLES
		WHERE TABLE_NAME = 'ThreatCategory'
		AND TABLE_SCHEMA = 'dbo'
	)
	AND EXISTS(
		SELECT *
		FROM INFORMATION_SCHEMA.TABLES
		WHERE TABLE_NAME = 'ThreatType'
		AND TABLE_SCHEMA = 'dbo'
	)	
	AND NOT EXISTS (
		SELECT *
		FROM INFORMATION_SCHEMA.CONSTRAINT_COLUMN_USAGE
		WHERE CONSTRAINT_NAME = 'FK_ThreatCategory_ThreatType'
	)
	BEGIN
		PRINT 'Create FK_ThreatCategory_ThreatType constraint'
	
		ALTER TABLE [dbo].[ThreatCategory]
			ADD CONSTRAINT FK_ThreatCategory_ThreatType
			FOREIGN KEY(ThreatTypeID)
			REFERENCES ThreatType(ThreatTypeID) ON DELETE NO ACTION
	END
END
GO

-- List of threat actions.
IF NOT EXISTS (
	SELECT * FROM INFORMATION_SCHEMA.TABLES
	WHERE TABLE_NAME = N'ThreatAction'
	AND TABLE_SCHEMA = 'dbo'
)
BEGIN
	CREATE TABLE [dbo].[ThreatAction] (
		-- Threat action ID
		ThreatActionID		int IDENTITY NOT NULL,
		-- Threat action name
		[Name]			nvarchar(64) NOT NULL,
		CONSTRAINT PK_ThreatAction
			PRIMARY KEY NONCLUSTERED([Name]),
		CONSTRAINT UQ_ThreatAction
			UNIQUE CLUSTERED(ThreatActionID)
	);

END
ELSE BEGIN
	PRINT 'Using existing ThreatAction table'
	IF EXISTS (
		SELECT * FROM sysindexes
		WHERE name = N'UQ_ThreatAction'
		AND ((status & 6162) <> 4114)
	)
	BEGIN
		PRINT 'Recreating existing UQ_ThreatAction index'
		ALTER TABLE [dbo].[ThreatAction]
		DROP CONSTRAINT UQ_ThreatAction
	END

	IF EXISTS (
		SELECT * FROM sysindexes
		WHERE name = N'PK_ThreatAction'
		AND ((status & 6162) <> 2050)
	)
	BEGIN
		PRINT 'Recreating existing PK_ThreatAction index'
		ALTER TABLE [dbo].[ThreatAction]
		DROP CONSTRAINT PK_ThreatAction
	END

	IF NOT EXISTS (
		SELECT * FROM sysindexes
		WHERE name = N'UQ_ThreatAction'
	)
		ALTER TABLE [dbo].[ThreatAction]
		ADD CONSTRAINT UQ_ThreatAction
			UNIQUE CLUSTERED(ThreatActionID)

	IF NOT EXISTS (
		SELECT * FROM sysindexes
		WHERE name = N'PK_ThreatAction'
	)
		ALTER TABLE [dbo].[ThreatAction]
		ADD CONSTRAINT PK_ThreatAction
			PRIMARY KEY NONCLUSTERED([Name])

	END
GO

-- List of threat actions.
IF NOT EXISTS (
	SELECT * FROM INFORMATION_SCHEMA.TABLES
	WHERE TABLE_NAME = N'ThreatRisk'
	AND TABLE_SCHEMA = 'dbo'
)
BEGIN
	CREATE TABLE [dbo].[ThreatRisk] (
		-- Threat risk ID
		ThreatRiskID		int IDENTITY NOT NULL,
		-- Threat risk name
		[Name]			nvarchar(64) NOT NULL,
		-- Threat risk severity (0 = none, 7 = critical)
		Severity		int NOT NULL,
		CONSTRAINT PK_ThreatRisk
			PRIMARY KEY NONCLUSTERED([Name]),
		CONSTRAINT UQ_ThreatRisk
			UNIQUE CLUSTERED(ThreatRiskID)
	);

END
ELSE BEGIN
	PRINT 'Using existing ThreatRisk table'
	IF EXISTS (
		SELECT * FROM sysindexes
		WHERE name = N'UQ_ThreatRisk'
		AND ((status & 6162) <> 4114)
	)
	BEGIN
		PRINT 'Recreating existing UQ_ThreatRisk index'
		ALTER TABLE [dbo].[ThreatRisk]
		DROP CONSTRAINT UQ_ThreatRisk
	END

	IF EXISTS (
		SELECT * FROM sysindexes
		WHERE name = N'PK_ThreatRisk'
		AND ((status & 6162) <> 2050)
	)
	BEGIN
		PRINT 'Recreating existing PK_ThreatRisk index'
		ALTER TABLE [dbo].[ThreatRisk]
		DROP CONSTRAINT PK_ThreatRisk
	END

	IF NOT EXISTS (
		SELECT * FROM sysindexes
		WHERE name = N'UQ_ThreatRisk'
	)
		ALTER TABLE [dbo].[ThreatRisk]
		ADD CONSTRAINT UQ_ThreatRisk
			UNIQUE CLUSTERED(ThreatRiskID)

	IF NOT EXISTS (
		SELECT * FROM sysindexes
		WHERE name = N'PK_ThreatRisk'
	)
		ALTER TABLE [dbo].[ThreatRisk]
		ADD CONSTRAINT PK_ThreatRisk
			PRIMARY KEY NONCLUSTERED([Name])

	END
GO

-- Applications that scan for threats.
IF NOT EXISTS (
	SELECT * FROM INFORMATION_SCHEMA.TABLES
	WHERE TABLE_NAME = N'ThreatScanner'
	AND TABLE_SCHEMA = 'dbo'
)
BEGIN
	CREATE TABLE [dbo].[ThreatScanner] (
		-- Threat scanner ID
		ThreatScannerID		int IDENTITY NOT NULL,
		-- Threat scanner name
		[Name]			nvarchar(128) NOT NULL,
		-- Threat scanner title or display name
		Title			nvarchar(256) NULL,
		-- Threat scanner version
		Version			nvarchar(256) NULL,
		-- Threat scanner manufacturer
		Manufacturer		nvarchar(256) NULL,
		CONSTRAINT PK_ThreatScanner
			PRIMARY KEY NONCLUSTERED([Name]),
		CONSTRAINT UQ_ThreatScanner
			UNIQUE CLUSTERED(ThreatScannerID)
	);

END
ELSE BEGIN
	PRINT 'Using existing ThreatScanner table'
	IF EXISTS (
		SELECT * FROM sysindexes
		WHERE name = N'UQ_ThreatScanner'
		AND ((status & 6162) <> 4114)
	)
	BEGIN
		PRINT 'Recreating existing UQ_ThreatScanner index'
		ALTER TABLE [dbo].[ThreatScanner]
		DROP CONSTRAINT UQ_ThreatScanner
	END

	IF EXISTS (
		SELECT * FROM sysindexes
		WHERE name = N'PK_ThreatScanner'
		AND ((status & 6162) <> 2050)
	)
	BEGIN
		PRINT 'Recreating existing PK_ThreatScanner index'
		ALTER TABLE [dbo].[ThreatScanner]
		DROP CONSTRAINT PK_ThreatScanner
	END

	IF NOT EXISTS (
		SELECT * FROM sysindexes
		WHERE name = N'UQ_ThreatScanner'
	)
		ALTER TABLE [dbo].[ThreatScanner]
		ADD CONSTRAINT UQ_ThreatScanner
			UNIQUE CLUSTERED(ThreatScannerID)

	IF NOT EXISTS (
		SELECT * FROM sysindexes
		WHERE name = N'PK_ThreatScanner'
	)
		ALTER TABLE [dbo].[ThreatScanner]
		ADD CONSTRAINT PK_ThreatScanner
			PRIMARY KEY NONCLUSTERED([Name])

	END
GO

-- The list of threat types detected by a Scanner.
IF NOT EXISTS (
	SELECT * FROM INFORMATION_SCHEMA.TABLES
	WHERE TABLE_NAME = N'ThreatScannerToType'
	AND TABLE_SCHEMA = 'dbo'
)
BEGIN
	CREATE TABLE [dbo].[ThreatScannerToType] (
		-- Threat scanner
		ThreatScannerID		int NOT NULL,
		-- Type of threat detected by scanner
		ThreatTypeID		int NOT NULL,
		CONSTRAINT FK_ThreatScannerToType_ThreatScanner
			FOREIGN KEY(ThreatScannerID)
			REFERENCES ThreatScanner(ThreatScannerID) ON DELETE NO ACTION,
		CONSTRAINT FK_ThreatScannerToType_ThreatType
			FOREIGN KEY(ThreatTypeID)
			REFERENCES ThreatType(ThreatTypeID) ON DELETE NO ACTION,
		CONSTRAINT PK_ThreatScannerToType
			PRIMARY KEY CLUSTERED(ThreatScannerID, ThreatTypeID)
	);

END
ELSE BEGIN
	PRINT 'Using existing ThreatScannerToType table'
	IF EXISTS (
		SELECT * FROM sysindexes
		WHERE name = N'PK_ThreatScannerToType'
		AND ((status & 6162) <> 2066)
	)
	BEGIN
		PRINT 'Recreating existing PK_ThreatScannerToType index'
		ALTER TABLE [dbo].[ThreatScannerToType]
		DROP CONSTRAINT PK_ThreatScannerToType
	END

	IF NOT EXISTS (
		SELECT * FROM sysindexes
		WHERE name = N'PK_ThreatScannerToType'
	)
		ALTER TABLE [dbo].[ThreatScannerToType]
		ADD CONSTRAINT PK_ThreatScannerToType
			PRIMARY KEY CLUSTERED(ThreatScannerID, ThreatTypeID)

	IF EXISTS(
		SELECT *
		FROM INFORMATION_SCHEMA.TABLES
		WHERE TABLE_NAME = 'ThreatScannerToType'
		AND TABLE_SCHEMA = 'dbo'
	)
	AND EXISTS(
		SELECT *
		FROM INFORMATION_SCHEMA.TABLES
		WHERE TABLE_NAME = 'ThreatScanner'
		AND TABLE_SCHEMA = 'dbo'
	)	
	AND NOT EXISTS (
		SELECT *
		FROM INFORMATION_SCHEMA.CONSTRAINT_COLUMN_USAGE
		WHERE CONSTRAINT_NAME = 'FK_ThreatScannerToType_ThreatScanner'
	)
	BEGIN
		PRINT 'Create FK_ThreatScannerToType_ThreatScanner constraint'
	
		ALTER TABLE [dbo].[ThreatScannerToType]
			ADD CONSTRAINT FK_ThreatScannerToType_ThreatScanner
			FOREIGN KEY(ThreatScannerID)
			REFERENCES ThreatScanner(ThreatScannerID) ON DELETE NO ACTION
	END
	
	IF EXISTS(
		SELECT *
		FROM INFORMATION_SCHEMA.TABLES
		WHERE TABLE_NAME = 'ThreatScannerToType'
		AND TABLE_SCHEMA = 'dbo'
	)
	AND EXISTS(
		SELECT *
		FROM INFORMATION_SCHEMA.TABLES
		WHERE TABLE_NAME = 'ThreatType'
		AND TABLE_SCHEMA = 'dbo'
	)	
	AND NOT EXISTS (
		SELECT *
		FROM INFORMATION_SCHEMA.CONSTRAINT_COLUMN_USAGE
		WHERE CONSTRAINT_NAME = 'FK_ThreatScannerToType_ThreatType'
	)
	BEGIN
		PRINT 'Create FK_ThreatScannerToType_ThreatType constraint'
	
		ALTER TABLE [dbo].[ThreatScannerToType]
			ADD CONSTRAINT FK_ThreatScannerToType_ThreatType
			FOREIGN KEY(ThreatTypeID)
			REFERENCES ThreatType(ThreatTypeID) ON DELETE NO ACTION
	END
END
GO

-- The infected item details from the Threat application.
IF NOT EXISTS (
	SELECT * FROM INFORMATION_SCHEMA.TABLES
	WHERE TABLE_NAME = N'Threat'
	AND TABLE_SCHEMA = 'dbo'
)
BEGIN
	CREATE TABLE [dbo].[Threat] (
		-- Threat ID
		ThreatID		int IDENTITY NOT NULL,
		-- Threat name
		[Name]			nvarchar(128) NOT NULL,
		-- Threat title or display name
		Title			nvarchar(256) NOT NULL,
		-- Type of threat
		ThreatTypeID		int NOT NULL,
		-- Risk of threat
		ThreatRiskID		int NOT NULL,
		-- Comma separated list of categories for threat
		CategorySummary		nvarchar(256) NOT NULL DEFAULT '',
		-- Monitor state of threat (1 = not monitored, 2 = monitored)
		State			int NOT NULL DEFAULT 2,
		-- Date that threat is posted
		DatePosted		datetime NOT NULL DEFAULT getdate(),
		-- Description of threat
		Description		ntext NOT NULL DEFAULT '',
		CONSTRAINT FK_Threat_ThreatType
			FOREIGN KEY(ThreatTypeID)
			REFERENCES ThreatType(ThreatTypeID) ON DELETE NO ACTION,
		CONSTRAINT FK_Threat_ThreatRisk
			FOREIGN KEY(ThreatRiskID)
			REFERENCES ThreatRisk(ThreatRiskID) ON DELETE NO ACTION,
		CONSTRAINT PK_Threat
			PRIMARY KEY NONCLUSTERED([Name]),
		CONSTRAINT UQ_Threat
			UNIQUE CLUSTERED(ThreatID)
	);

	CREATE NONCLUSTERED INDEX ThreatByThreatType
		ON dbo.Threat(ThreatTypeID)

END
ELSE BEGIN
	PRINT 'Using existing Threat table'
	IF EXISTS (
		SELECT * FROM sysindexes
		WHERE name = N'UQ_Threat'
		AND ((status & 6162) <> 4114)
	)
	BEGIN
		PRINT 'Recreating existing UQ_Threat index'
		ALTER TABLE [dbo].[Threat]
		DROP CONSTRAINT UQ_Threat
	END

	IF EXISTS (
		SELECT * FROM sysindexes
		WHERE name = N'PK_Threat'
		AND ((status & 6162) <> 2050)
	)
	BEGIN
		PRINT 'Recreating existing PK_Threat index'
		ALTER TABLE [dbo].[Threat]
		DROP CONSTRAINT PK_Threat
	END

	IF EXISTS (
		SELECT * FROM sysindexes
		WHERE name = N'ThreatByThreatType'
		AND ((status & 6162) <> 0)
	)
	BEGIN
		PRINT 'Recreating existing ThreatByThreatType index'
		DROP INDEX ThreatByThreatType
		ON [dbo].[Threat]
	END

	IF NOT EXISTS (
		SELECT * FROM sysindexes
		WHERE name = N'UQ_Threat'
	)
		ALTER TABLE [dbo].[Threat]
		ADD CONSTRAINT UQ_Threat
			UNIQUE CLUSTERED(ThreatID)

	IF NOT EXISTS (
		SELECT * FROM sysindexes
		WHERE name = N'PK_Threat'
	)
		ALTER TABLE [dbo].[Threat]
		ADD CONSTRAINT PK_Threat
			PRIMARY KEY NONCLUSTERED([Name])

	IF NOT EXISTS (
		SELECT * FROM sysindexes
		WHERE name = N'ThreatByThreatType'
	)
		CREATE NONCLUSTERED INDEX ThreatByThreatType
			ON dbo.Threat(ThreatTypeID)

	IF EXISTS(
		SELECT *
		FROM INFORMATION_SCHEMA.TABLES
		WHERE TABLE_NAME = 'Threat'
		AND TABLE_SCHEMA = 'dbo'
	)
	AND EXISTS(
		SELECT *
		FROM INFORMATION_SCHEMA.TABLES
		WHERE TABLE_NAME = 'ThreatType'
		AND TABLE_SCHEMA = 'dbo'
	)	
	AND NOT EXISTS (
		SELECT *
		FROM INFORMATION_SCHEMA.CONSTRAINT_COLUMN_USAGE
		WHERE CONSTRAINT_NAME = 'FK_Threat_ThreatType'
	)
	BEGIN
		PRINT 'Create FK_Threat_ThreatType constraint'
	
		ALTER TABLE [dbo].[Threat]
			ADD CONSTRAINT FK_Threat_ThreatType
			FOREIGN KEY(ThreatTypeID)
			REFERENCES ThreatType(ThreatTypeID) ON DELETE NO ACTION
	END
	
	IF EXISTS(
		SELECT *
		FROM INFORMATION_SCHEMA.TABLES
		WHERE TABLE_NAME = 'Threat'
		AND TABLE_SCHEMA = 'dbo'
	)
	AND EXISTS(
		SELECT *
		FROM INFORMATION_SCHEMA.TABLES
		WHERE TABLE_NAME = 'ThreatRisk'
		AND TABLE_SCHEMA = 'dbo'
	)	
	AND NOT EXISTS (
		SELECT *
		FROM INFORMATION_SCHEMA.CONSTRAINT_COLUMN_USAGE
		WHERE CONSTRAINT_NAME = 'FK_Threat_ThreatRisk'
	)
	BEGIN
		PRINT 'Create FK_Threat_ThreatRisk constraint'
	
		ALTER TABLE [dbo].[Threat]
			ADD CONSTRAINT FK_Threat_ThreatRisk
			FOREIGN KEY(ThreatRiskID)
			REFERENCES ThreatRisk(ThreatRiskID) ON DELETE NO ACTION
	END
END
GO

-- List of categories per threat.
IF NOT EXISTS (
	SELECT * FROM INFORMATION_SCHEMA.TABLES
	WHERE TABLE_NAME = N'ThreatToCategory'
	AND TABLE_SCHEMA = 'dbo'
)
BEGIN
	CREATE TABLE [dbo].[ThreatToCategory] (
		-- Threat
		ThreatID		int NOT NULL,
		-- Category of threat
		ThreatCategoryID	int NOT NULL,
		CONSTRAINT FK_ThreatToCategory_Threat
			FOREIGN KEY(ThreatID)
			REFERENCES Threat(ThreatID) ON DELETE CASCADE,
		CONSTRAINT FK_ThreatToCategory_ThreatCategory
			FOREIGN KEY(ThreatCategoryID)
			REFERENCES ThreatCategory(ThreatCategoryID) ON DELETE CASCADE,
		CONSTRAINT PK_ThreatToCategory
			PRIMARY KEY CLUSTERED(ThreatID, ThreatCategoryID)
	);

	CREATE NONCLUSTERED INDEX UQ_ThreatToCategory
		ON dbo.ThreatToCategory(ThreatCategoryID)

END
ELSE BEGIN
	PRINT 'Using existing ThreatToCategory table'
	IF EXISTS (
		SELECT * FROM sysindexes
		WHERE name = N'PK_ThreatToCategory'
		AND ((status & 6162) <> 2066)
	)
	BEGIN
		PRINT 'Recreating existing PK_ThreatToCategory index'
		ALTER TABLE [dbo].[ThreatToCategory]
		DROP CONSTRAINT PK_ThreatToCategory
	END

	IF EXISTS (
		SELECT * FROM sysindexes
		WHERE name = N'UQ_ThreatToCategory'
		AND ((status & 6162) <> 0)
	)
	BEGIN
		PRINT 'Recreating existing UQ_ThreatToCategory index'
		DROP INDEX UQ_ThreatToCategory
		ON [dbo].[ThreatToCategory]
	END

	IF NOT EXISTS (
		SELECT * FROM sysindexes
		WHERE name = N'PK_ThreatToCategory'
	)
		ALTER TABLE [dbo].[ThreatToCategory]
		ADD CONSTRAINT PK_ThreatToCategory
			PRIMARY KEY CLUSTERED(ThreatID, ThreatCategoryID)

	IF NOT EXISTS (
		SELECT * FROM sysindexes
		WHERE name = N'UQ_ThreatToCategory'
	)
		CREATE NONCLUSTERED INDEX UQ_ThreatToCategory
			ON dbo.ThreatToCategory(ThreatCategoryID)

	IF EXISTS(
		SELECT *
		FROM INFORMATION_SCHEMA.TABLES
		WHERE TABLE_NAME = 'ThreatToCategory'
		AND TABLE_SCHEMA = 'dbo'
	)
	AND EXISTS(
		SELECT *
		FROM INFORMATION_SCHEMA.TABLES
		WHERE TABLE_NAME = 'Threat'
		AND TABLE_SCHEMA = 'dbo'
	)	
	AND NOT EXISTS (
		SELECT *
		FROM INFORMATION_SCHEMA.CONSTRAINT_COLUMN_USAGE
		WHERE CONSTRAINT_NAME = 'FK_ThreatToCategory_Threat'
	)
	BEGIN
		PRINT 'Create FK_ThreatToCategory_Threat constraint'
	
		ALTER TABLE [dbo].[ThreatToCategory]
			ADD CONSTRAINT FK_ThreatToCategory_Threat
			FOREIGN KEY(ThreatID)
			REFERENCES Threat(ThreatID) ON DELETE CASCADE
	END
	
	IF EXISTS(
		SELECT *
		FROM INFORMATION_SCHEMA.TABLES
		WHERE TABLE_NAME = 'ThreatToCategory'
		AND TABLE_SCHEMA = 'dbo'
	)
	AND EXISTS(
		SELECT *
		FROM INFORMATION_SCHEMA.TABLES
		WHERE TABLE_NAME = 'ThreatCategory'
		AND TABLE_SCHEMA = 'dbo'
	)	
	AND NOT EXISTS (
		SELECT *
		FROM INFORMATION_SCHEMA.CONSTRAINT_COLUMN_USAGE
		WHERE CONSTRAINT_NAME = 'FK_ThreatToCategory_ThreatCategory'
	)
	BEGIN
		PRINT 'Create FK_ThreatToCategory_ThreatCategory constraint'
	
		ALTER TABLE [dbo].[ThreatToCategory]
			ADD CONSTRAINT FK_ThreatToCategory_ThreatCategory
			FOREIGN KEY(ThreatCategoryID)
			REFERENCES ThreatCategory(ThreatCategoryID) ON DELETE CASCADE
	END
END
GO

-- List of scanners that can detect a threat.
IF NOT EXISTS (
	SELECT * FROM INFORMATION_SCHEMA.TABLES
	WHERE TABLE_NAME = N'ThreatToScanner'
	AND TABLE_SCHEMA = 'dbo'
)
BEGIN
	CREATE TABLE [dbo].[ThreatToScanner] (
		-- Threat
		ThreatID		int NOT NULL,
		-- Scanner of threat
		ThreatScannerID		int NOT NULL,
		CONSTRAINT FK_ThreatToScanner_Threat
			FOREIGN KEY(ThreatID)
			REFERENCES Threat(ThreatID) ON DELETE CASCADE,
		CONSTRAINT FK_ThreatToScanner_ThreatScanner
			FOREIGN KEY(ThreatScannerID)
			REFERENCES ThreatScanner(ThreatScannerID) ON DELETE NO ACTION,
		CONSTRAINT PK_ThreatToScanner
			PRIMARY KEY CLUSTERED(ThreatID, ThreatScannerID)
	);

	CREATE NONCLUSTERED INDEX UQ_ThreatToScanner
		ON dbo.ThreatToScanner(ThreatScannerID)

END
ELSE BEGIN
	PRINT 'Using existing ThreatToScanner table'
	IF EXISTS (
		SELECT * FROM sysindexes
		WHERE name = N'PK_ThreatToScanner'
		AND ((status & 6162) <> 2066)
	)
	BEGIN
		PRINT 'Recreating existing PK_ThreatToScanner index'
		ALTER TABLE [dbo].[ThreatToScanner]
		DROP CONSTRAINT PK_ThreatToScanner
	END

	IF EXISTS (
		SELECT * FROM sysindexes
		WHERE name = N'UQ_ThreatToScanner'
		AND ((status & 6162) <> 0)
	)
	BEGIN
		PRINT 'Recreating existing UQ_ThreatToScanner index'
		DROP INDEX UQ_ThreatToScanner
		ON [dbo].[ThreatToScanner]
	END

	IF NOT EXISTS (
		SELECT * FROM sysindexes
		WHERE name = N'PK_ThreatToScanner'
	)
		ALTER TABLE [dbo].[ThreatToScanner]
		ADD CONSTRAINT PK_ThreatToScanner
			PRIMARY KEY CLUSTERED(ThreatID, ThreatScannerID)

	IF NOT EXISTS (
		SELECT * FROM sysindexes
		WHERE name = N'UQ_ThreatToScanner'
	)
		CREATE NONCLUSTERED INDEX UQ_ThreatToScanner
			ON dbo.ThreatToScanner(ThreatScannerID)

	IF EXISTS(
		SELECT *
		FROM INFORMATION_SCHEMA.TABLES
		WHERE TABLE_NAME = 'ThreatToScanner'
		AND TABLE_SCHEMA = 'dbo'
	)
	AND EXISTS(
		SELECT *
		FROM INFORMATION_SCHEMA.TABLES
		WHERE TABLE_NAME = 'Threat'
		AND TABLE_SCHEMA = 'dbo'
	)	
	AND NOT EXISTS (
		SELECT *
		FROM INFORMATION_SCHEMA.CONSTRAINT_COLUMN_USAGE
		WHERE CONSTRAINT_NAME = 'FK_ThreatToScanner_Threat'
	)
	BEGIN
		PRINT 'Create FK_ThreatToScanner_Threat constraint'
	
		ALTER TABLE [dbo].[ThreatToScanner]
			ADD CONSTRAINT FK_ThreatToScanner_Threat
			FOREIGN KEY(ThreatID)
			REFERENCES Threat(ThreatID) ON DELETE CASCADE
	END
	
	IF EXISTS(
		SELECT *
		FROM INFORMATION_SCHEMA.TABLES
		WHERE TABLE_NAME = 'ThreatToScanner'
		AND TABLE_SCHEMA = 'dbo'
	)
	AND EXISTS(
		SELECT *
		FROM INFORMATION_SCHEMA.TABLES
		WHERE TABLE_NAME = 'ThreatScanner'
		AND TABLE_SCHEMA = 'dbo'
	)	
	AND NOT EXISTS (
		SELECT *
		FROM INFORMATION_SCHEMA.CONSTRAINT_COLUMN_USAGE
		WHERE CONSTRAINT_NAME = 'FK_ThreatToScanner_ThreatScanner'
	)
	BEGIN
		PRINT 'Create FK_ThreatToScanner_ThreatScanner constraint'
	
		ALTER TABLE [dbo].[ThreatToScanner]
			ADD CONSTRAINT FK_ThreatToScanner_ThreatScanner
			FOREIGN KEY(ThreatScannerID)
			REFERENCES ThreatScanner(ThreatScannerID) ON DELETE NO ACTION
	END
END
GO

-- Log messages associated with a threat
IF NOT EXISTS (
	SELECT * FROM INFORMATION_SCHEMA.TABLES
	WHERE TABLE_NAME = N'ThreatLog'
	AND TABLE_SCHEMA = 'dbo'
)
BEGIN
	CREATE TABLE [dbo].[ThreatLog] (
		-- Threat log ID
		ThreatLogID		int IDENTITY NOT NULL,
		-- Threat
		ThreatID		int NOT NULL,
		-- Message for threat
		Message			nvarchar(128) NOT NULL,
		-- Date message posted
		DatePosted		datetime NOT NULL,
		CONSTRAINT FK_ThreatMessageLog_Threat
			FOREIGN KEY(ThreatID)
			REFERENCES Threat(ThreatID) ON DELETE CASCADE,
		CONSTRAINT PK_ThreatLog
			PRIMARY KEY CLUSTERED(ThreatLogID)
	);

	CREATE NONCLUSTERED INDEX ThreatMessageLogByThreat
		ON dbo.ThreatLog(ThreatID)

END
ELSE BEGIN
	PRINT 'Using existing ThreatLog table'
	IF EXISTS (
		SELECT * FROM sysindexes
		WHERE name = N'PK_ThreatLog'
		AND ((status & 6162) <> 2066)
	)
	BEGIN
		PRINT 'Recreating existing PK_ThreatLog index'
		ALTER TABLE [dbo].[ThreatLog]
		DROP CONSTRAINT PK_ThreatLog
	END

	IF EXISTS (
		SELECT * FROM sysindexes
		WHERE name = N'ThreatMessageLogByThreat'
		AND ((status & 6162) <> 0)
	)
	BEGIN
		PRINT 'Recreating existing ThreatMessageLogByThreat index'
		DROP INDEX ThreatMessageLogByThreat
		ON [dbo].[ThreatLog]
	END

	IF NOT EXISTS (
		SELECT * FROM sysindexes
		WHERE name = N'PK_ThreatLog'
	)
		ALTER TABLE [dbo].[ThreatLog]
		ADD CONSTRAINT PK_ThreatLog
			PRIMARY KEY CLUSTERED(ThreatLogID)

	IF NOT EXISTS (
		SELECT * FROM sysindexes
		WHERE name = N'ThreatMessageLogByThreat'
	)
		CREATE NONCLUSTERED INDEX ThreatMessageLogByThreat
			ON dbo.ThreatLog(ThreatID)

	IF EXISTS(
		SELECT *
		FROM INFORMATION_SCHEMA.TABLES
		WHERE TABLE_NAME = 'ThreatLog'
		AND TABLE_SCHEMA = 'dbo'
	)
	AND EXISTS(
		SELECT *
		FROM INFORMATION_SCHEMA.TABLES
		WHERE TABLE_NAME = 'Threat'
		AND TABLE_SCHEMA = 'dbo'
	)	
	AND NOT EXISTS (
		SELECT *
		FROM INFORMATION_SCHEMA.CONSTRAINT_COLUMN_USAGE
		WHERE CONSTRAINT_NAME = 'FK_ThreatMessageLog_Threat'
	)
	BEGIN
		PRINT 'Create FK_ThreatMessageLog_Threat constraint'
	
		ALTER TABLE [dbo].[ThreatLog]
			ADD CONSTRAINT FK_ThreatMessageLog_Threat
			FOREIGN KEY(ThreatID)
			REFERENCES Threat(ThreatID) ON DELETE CASCADE
	END
END
GO

-- Threat scans performed on a computer.
IF NOT EXISTS (
	SELECT * FROM INFORMATION_SCHEMA.TABLES
	WHERE TABLE_NAME = N'ThreatScan'
	AND TABLE_SCHEMA = 'dbo'
)
BEGIN
	CREATE TABLE [dbo].[ThreatScan] (
		-- Computer scanned
		ComputerID		int NOT NULL,
		-- User scanned
		UserID			int NOT NULL,
		-- Type of threat scan performed
		ThreatTypeID		int NOT NULL,
		-- Scanner used to detect threats
		ThreatScannerID		int NOT NULL,
		-- Date of data source used by threat scanner
		DataSourceDate		datetime NULL,
		-- Version of data source used by threat scanner
		DataSourceVersion	nvarchar(128) NULL,
		-- Time threat scan started
		ScanStart		datetime NULL,
		-- Time threat scan finished
		ScanEnd			datetime NULL,
		CONSTRAINT FK_ThreatScan_ThreatType
			FOREIGN KEY(ThreatTypeID)
			REFERENCES ThreatType(ThreatTypeID) ON DELETE NO ACTION,
		CONSTRAINT FK_ThreatScan_ThreatScanner
			FOREIGN KEY(ThreatScannerID)
			REFERENCES ThreatScanner(ThreatScannerID) ON DELETE NO ACTION,
		CONSTRAINT PK_ThreatScan
			PRIMARY KEY CLUSTERED
			(ComputerID, UserID, ThreatTypeID, ThreatScannerID)
	);

	CREATE NONCLUSTERED INDEX ThreatScanByUser
		ON dbo.ThreatScan(UserID)

END
ELSE BEGIN
	PRINT 'Using existing ThreatScan table'
	IF EXISTS (
		SELECT * FROM sysindexes
		WHERE name = N'PK_ThreatScan'
		AND ((status & 6162) <> 2066)
	)
	BEGIN
		PRINT 'Recreating existing PK_ThreatScan index'
		ALTER TABLE [dbo].[ThreatScan]
		DROP CONSTRAINT PK_ThreatScan
	END

	IF EXISTS (
		SELECT * FROM sysindexes
		WHERE name = N'ThreatScanByUser'
		AND ((status & 6162) <> 0)
	)
	BEGIN
		PRINT 'Recreating existing ThreatScanByUser index'
		DROP INDEX ThreatScanByUser
		ON [dbo].[ThreatScan]
	END

	IF NOT EXISTS (
		SELECT * FROM sysindexes
		WHERE name = N'PK_ThreatScan'
	)
		ALTER TABLE [dbo].[ThreatScan]
		ADD CONSTRAINT PK_ThreatScan
			PRIMARY KEY CLUSTERED
			(ComputerID, UserID, ThreatTypeID, ThreatScannerID)

	IF NOT EXISTS (
		SELECT * FROM sysindexes
		WHERE name = N'ThreatScanByUser'
	)
		CREATE NONCLUSTERED INDEX ThreatScanByUser
			ON dbo.ThreatScan(UserID)

	IF EXISTS(
		SELECT *
		FROM INFORMATION_SCHEMA.TABLES
		WHERE TABLE_NAME = 'ThreatScan'
		AND TABLE_SCHEMA = 'dbo'
	)
	AND EXISTS(
		SELECT *
		FROM INFORMATION_SCHEMA.TABLES
		WHERE TABLE_NAME = 'ThreatType'
		AND TABLE_SCHEMA = 'dbo'
	)	
	AND NOT EXISTS (
		SELECT *
		FROM INFORMATION_SCHEMA.CONSTRAINT_COLUMN_USAGE
		WHERE CONSTRAINT_NAME = 'FK_ThreatScan_ThreatType'
	)
	BEGIN
		PRINT 'Create FK_ThreatScan_ThreatType constraint'
	
		ALTER TABLE [dbo].[ThreatScan]
			ADD CONSTRAINT FK_ThreatScan_ThreatType
			FOREIGN KEY(ThreatTypeID)
			REFERENCES ThreatType(ThreatTypeID) ON DELETE NO ACTION
	END
	
	IF EXISTS(
		SELECT *
		FROM INFORMATION_SCHEMA.TABLES
		WHERE TABLE_NAME = 'ThreatScan'
		AND TABLE_SCHEMA = 'dbo'
	)
	AND EXISTS(
		SELECT *
		FROM INFORMATION_SCHEMA.TABLES
		WHERE TABLE_NAME = 'ThreatScanner'
		AND TABLE_SCHEMA = 'dbo'
	)	
	AND NOT EXISTS (
		SELECT *
		FROM INFORMATION_SCHEMA.CONSTRAINT_COLUMN_USAGE
		WHERE CONSTRAINT_NAME = 'FK_ThreatScan_ThreatScanner'
	)
	BEGIN
		PRINT 'Create FK_ThreatScan_ThreatScanner constraint'
	
		ALTER TABLE [dbo].[ThreatScan]
			ADD CONSTRAINT FK_ThreatScan_ThreatScanner
			FOREIGN KEY(ThreatScannerID)
			REFERENCES ThreatScanner(ThreatScannerID) ON DELETE NO ACTION
	END
END
GO

-- Details outlining the occurrences of this infected item.
IF NOT EXISTS (
	SELECT * FROM INFORMATION_SCHEMA.TABLES
	WHERE TABLE_NAME = N'ThreatOccurrence'
	AND TABLE_SCHEMA = 'dbo'
)
BEGIN
	CREATE TABLE [dbo].[ThreatOccurrence] (
		-- Computer scanned
		ComputerID		int NOT NULL,
		-- User scanned
		UserID			int NOT NULL,
		-- Type of threat scanned
		ThreatTypeID		int NOT NULL,
		-- Threat detected by scanner
		ThreatID		int NOT NULL,
		-- Threat scanner that detected threat
		ThreatScannerID		int NOT NULL,
		-- Risk of threat to computer
		ThreatOccurrenceRiskID	int NOT NULL,
		-- Action performed by ManageSoft to rectify threat
		ThreatActionID		int NOT NULL,
		-- Details of threat to computer
		Details			ntext NULL,
		CONSTRAINT FK_ThreatOccurrence_ThreatType
			FOREIGN KEY(ThreatTypeID)
			REFERENCES ThreatType(ThreatTypeID) ON DELETE NO ACTION,
		CONSTRAINT FK_ThreatOccurrence_Threat
			FOREIGN KEY(ThreatID)
			REFERENCES Threat(ThreatID) ON DELETE CASCADE,
		CONSTRAINT FK_ThreatOccurrence_ThreatScanner
			FOREIGN KEY(ThreatScannerID)
			REFERENCES ThreatScanner(ThreatScannerID) ON DELETE NO ACTION,
		CONSTRAINT FK_ThreatOccurrence_ThreatRisk
			FOREIGN KEY(ThreatOccurrenceRiskID)
			REFERENCES ThreatRisk(ThreatRiskID) ON DELETE NO ACTION,
		CONSTRAINT FK_ThreatOccurrence_ThreatAction
			FOREIGN KEY(ThreatActionID)
			REFERENCES ThreatAction(ThreatActionID) ON DELETE NO ACTION,
		CONSTRAINT PK_ThreatOccurrence
			PRIMARY KEY CLUSTERED
			(ComputerID, UserID, ThreatTypeID, ThreatID)
	);

	CREATE NONCLUSTERED INDEX ThreatOccurrenceByComputerUserThreatTypeThreatAction
		ON dbo.ThreatOccurrence
		(ThreatTypeID, ThreatActionID, ComputerID, UserID)

	CREATE NONCLUSTERED INDEX ThreatOccurrenceByUser
		ON dbo.ThreatOccurrence(UserID)

	CREATE NONCLUSTERED INDEX ThreatOccurrenceByThreatID
		ON dbo.ThreatOccurrence(ThreatID)

END
ELSE BEGIN
	PRINT 'Using existing ThreatOccurrence table'
	IF EXISTS (
		SELECT * FROM sysindexes
		WHERE name = N'PK_ThreatOccurrence'
		AND ((status & 6162) <> 2066)
	)
	BEGIN
		PRINT 'Recreating existing PK_ThreatOccurrence index'
		ALTER TABLE [dbo].[ThreatOccurrence]
		DROP CONSTRAINT PK_ThreatOccurrence
	END

	IF EXISTS (
		SELECT * FROM sysindexes
		WHERE name = N'ThreatOccurrenceByComputerUserThreatTypeThreatAction'
		AND ((status & 6162) <> 0)
	)
	BEGIN
		PRINT 'Recreating existing ThreatOccurrenceByComputerUserThreatTypeThreatAction index'
		DROP INDEX ThreatOccurrenceByComputerUserThreatTypeThreatAction
		ON [dbo].[ThreatOccurrence]
	END

	IF EXISTS (
		SELECT * FROM sysindexes
		WHERE name = N'ThreatOccurrenceByUser'
		AND ((status & 6162) <> 0)
	)
	BEGIN
		PRINT 'Recreating existing ThreatOccurrenceByUser index'
		DROP INDEX ThreatOccurrenceByUser
		ON [dbo].[ThreatOccurrence]
	END

	IF EXISTS (
		SELECT * FROM sysindexes
		WHERE name = N'ThreatOccurrenceByThreatID'
		AND ((status & 6162) <> 0)
	)
	BEGIN
		PRINT 'Recreating existing ThreatOccurrenceByThreatID index'
		DROP INDEX ThreatOccurrenceByThreatID
		ON [dbo].[ThreatOccurrence]
	END

	IF NOT EXISTS (
		SELECT * FROM sysindexes
		WHERE name = N'PK_ThreatOccurrence'
	)
		ALTER TABLE [dbo].[ThreatOccurrence]
		ADD CONSTRAINT PK_ThreatOccurrence
			PRIMARY KEY CLUSTERED
			(ComputerID, UserID, ThreatTypeID, ThreatID)

	IF NOT EXISTS (
		SELECT * FROM sysindexes
		WHERE name = N'ThreatOccurrenceByComputerUserThreatTypeThreatAction'
	)
		CREATE NONCLUSTERED INDEX ThreatOccurrenceByComputerUserThreatTypeThreatAction
			ON dbo.ThreatOccurrence
			(ThreatTypeID, ThreatActionID, ComputerID, UserID)

	IF NOT EXISTS (
		SELECT * FROM sysindexes
		WHERE name = N'ThreatOccurrenceByUser'
	)
		CREATE NONCLUSTERED INDEX ThreatOccurrenceByUser
			ON dbo.ThreatOccurrence(UserID)

	IF NOT EXISTS (
		SELECT * FROM sysindexes
		WHERE name = N'ThreatOccurrenceByThreatID'
	)
		CREATE NONCLUSTERED INDEX ThreatOccurrenceByThreatID
			ON dbo.ThreatOccurrence(ThreatID)

	IF EXISTS(
		SELECT *
		FROM INFORMATION_SCHEMA.TABLES
		WHERE TABLE_NAME = 'ThreatOccurrence'
		AND TABLE_SCHEMA = 'dbo'
	)
	AND EXISTS(
		SELECT *
		FROM INFORMATION_SCHEMA.TABLES
		WHERE TABLE_NAME = 'ThreatType'
		AND TABLE_SCHEMA = 'dbo'
	)	
	AND NOT EXISTS (
		SELECT *
		FROM INFORMATION_SCHEMA.CONSTRAINT_COLUMN_USAGE
		WHERE CONSTRAINT_NAME = 'FK_ThreatOccurrence_ThreatType'
	)
	BEGIN
		PRINT 'Create FK_ThreatOccurrence_ThreatType constraint'
	
		ALTER TABLE [dbo].[ThreatOccurrence]
			ADD CONSTRAINT FK_ThreatOccurrence_ThreatType
			FOREIGN KEY(ThreatTypeID)
			REFERENCES ThreatType(ThreatTypeID) ON DELETE NO ACTION
	END
	
	IF EXISTS(
		SELECT *
		FROM INFORMATION_SCHEMA.TABLES
		WHERE TABLE_NAME = 'ThreatOccurrence'
		AND TABLE_SCHEMA = 'dbo'
	)
	AND EXISTS(
		SELECT *
		FROM INFORMATION_SCHEMA.TABLES
		WHERE TABLE_NAME = 'Threat'
		AND TABLE_SCHEMA = 'dbo'
	)	
	AND NOT EXISTS (
		SELECT *
		FROM INFORMATION_SCHEMA.CONSTRAINT_COLUMN_USAGE
		WHERE CONSTRAINT_NAME = 'FK_ThreatOccurrence_Threat'
	)
	BEGIN
		PRINT 'Create FK_ThreatOccurrence_Threat constraint'
	
		ALTER TABLE [dbo].[ThreatOccurrence]
			ADD CONSTRAINT FK_ThreatOccurrence_Threat
			FOREIGN KEY(ThreatID)
			REFERENCES Threat(ThreatID) ON DELETE CASCADE
	END
	
	IF EXISTS(
		SELECT *
		FROM INFORMATION_SCHEMA.TABLES
		WHERE TABLE_NAME = 'ThreatOccurrence'
		AND TABLE_SCHEMA = 'dbo'
	)
	AND EXISTS(
		SELECT *
		FROM INFORMATION_SCHEMA.TABLES
		WHERE TABLE_NAME = 'ThreatScanner'
		AND TABLE_SCHEMA = 'dbo'
	)	
	AND NOT EXISTS (
		SELECT *
		FROM INFORMATION_SCHEMA.CONSTRAINT_COLUMN_USAGE
		WHERE CONSTRAINT_NAME = 'FK_ThreatOccurrence_ThreatScanner'
	)
	BEGIN
		PRINT 'Create FK_ThreatOccurrence_ThreatScanner constraint'
	
		ALTER TABLE [dbo].[ThreatOccurrence]
			ADD CONSTRAINT FK_ThreatOccurrence_ThreatScanner
			FOREIGN KEY(ThreatScannerID)
			REFERENCES ThreatScanner(ThreatScannerID) ON DELETE NO ACTION
	END
	
	IF EXISTS(
		SELECT *
		FROM INFORMATION_SCHEMA.TABLES
		WHERE TABLE_NAME = 'ThreatOccurrence'
		AND TABLE_SCHEMA = 'dbo'
	)
	AND EXISTS(
		SELECT *
		FROM INFORMATION_SCHEMA.TABLES
		WHERE TABLE_NAME = 'ThreatRisk'
		AND TABLE_SCHEMA = 'dbo'
	)	
	AND NOT EXISTS (
		SELECT *
		FROM INFORMATION_SCHEMA.CONSTRAINT_COLUMN_USAGE
		WHERE CONSTRAINT_NAME = 'FK_ThreatOccurrence_ThreatRisk'
	)
	BEGIN
		PRINT 'Create FK_ThreatOccurrence_ThreatRisk constraint'
	
		ALTER TABLE [dbo].[ThreatOccurrence]
			ADD CONSTRAINT FK_ThreatOccurrence_ThreatRisk
			FOREIGN KEY(ThreatOccurrenceRiskID)
			REFERENCES ThreatRisk(ThreatRiskID) ON DELETE NO ACTION
	END
	
	IF EXISTS(
		SELECT *
		FROM INFORMATION_SCHEMA.TABLES
		WHERE TABLE_NAME = 'ThreatOccurrence'
		AND TABLE_SCHEMA = 'dbo'
	)
	AND EXISTS(
		SELECT *
		FROM INFORMATION_SCHEMA.TABLES
		WHERE TABLE_NAME = 'ThreatAction'
		AND TABLE_SCHEMA = 'dbo'
	)	
	AND NOT EXISTS (
		SELECT *
		FROM INFORMATION_SCHEMA.CONSTRAINT_COLUMN_USAGE
		WHERE CONSTRAINT_NAME = 'FK_ThreatOccurrence_ThreatAction'
	)
	BEGIN
		PRINT 'Create FK_ThreatOccurrence_ThreatAction constraint'
	
		ALTER TABLE [dbo].[ThreatOccurrence]
			ADD CONSTRAINT FK_ThreatOccurrence_ThreatAction
			FOREIGN KEY(ThreatActionID)
			REFERENCES ThreatAction(ThreatActionID) ON DELETE NO ACTION
	END
END
GO

-- List of applications that can be blocked from running.
IF NOT EXISTS (
	SELECT * FROM INFORMATION_SCHEMA.TABLES
	WHERE TABLE_NAME = N'BlockedApplication'
	AND TABLE_SCHEMA = 'dbo'
)
BEGIN
	CREATE TABLE [dbo].[BlockedApplication] (
		-- Executable filename of application to block
		[FileName]		nvarchar(256) NOT NULL,
		-- Friendly application name containing executable
		ApplicationName		nvarchar(256) NOT NULL DEFAULT '',
		-- Description of application
		Description		ntext NOT NULL DEFAULT '',
		-- Visible status of application information (0 = hidden, 1 = visible)
		Visible			bit NOT NULL DEFAULT 1,
		CONSTRAINT PK_BlockedApplication
			PRIMARY KEY CLUSTERED([FileName])
	);

END
ELSE BEGIN
	PRINT 'Using existing BlockedApplication table'
	IF EXISTS (
		SELECT * FROM sysindexes
		WHERE name = N'PK_BlockedApplication'
		AND ((status & 6162) <> 2066)
	)
	BEGIN
		PRINT 'Recreating existing PK_BlockedApplication index'
		ALTER TABLE [dbo].[BlockedApplication]
		DROP CONSTRAINT PK_BlockedApplication
	END

	IF NOT EXISTS (
		SELECT * FROM sysindexes
		WHERE name = N'PK_BlockedApplication'
	)
		ALTER TABLE [dbo].[BlockedApplication]
		ADD CONSTRAINT PK_BlockedApplication
			PRIMARY KEY CLUSTERED([FileName])

	END
GO


IF NOT EXISTS(SELECT * FROM SecPatch_Location)
BEGIN
	INSERT INTO SecPatch_Location (AbsolutePath, LocationID, Path)
		VALUES ('', 0, 'Other')
END
GO

IF NOT EXISTS(SELECT * FROM SecurityBulletin)
BEGIN
	SET IDENTITY_INSERT SecurityBulletin ON
	INSERT INTO SecurityBulletin (BulletinID, DatePosted, DateRevised, SecurityBulletinID, Severity)
		VALUES ('Unknown', '20030328', '20030328', 0, 0)
	SET IDENTITY_INSERT SecurityBulletin OFF
END
GO

IF NOT EXISTS(SELECT * FROM SecurityPatch)
BEGIN
	SET IDENTITY_INSERT SecurityPatch ON
	INSERT INTO SecurityPatch (BulletinID, PatchID, PatchName, QNumber, Title)
		VALUES ('Unknown', 0, 'Unknown', '', 'Unknown')
	SET IDENTITY_INSERT SecurityPatch OFF
END
GO

IF NOT EXISTS(SELECT * FROM SecurityState)
BEGIN
	SET IDENTITY_INSERT SecurityState ON
	INSERT INTO SecurityState (Description, StateID)
		VALUES ('Posted', 1)
	INSERT INTO SecurityState (Description, StateID)
		VALUES ('Downloaded', 2)
	INSERT INTO SecurityState (Description, StateID)
		VALUES ('Packaged', 3)
	INSERT INTO SecurityState (Description, StateID)
		VALUES ('Released', 4)
	INSERT INTO SecurityState (Description, StateID)
		VALUES ('Required', 5)
	INSERT INTO SecurityState (Description, StateID)
		VALUES ('Not Required', 6)
	INSERT INTO SecurityState (Description, StateID)
		VALUES ('Approved', 7)
	INSERT INTO SecurityState (Description, StateID)
		VALUES ('Distributed', 8)
	SET IDENTITY_INSERT SecurityState OFF
END
GO
IF NOT EXISTS (
	SELECT * FROM SecurityState
	WHERE StateID > 1000
)
BEGIN
	DBCC CHECKIDENT("[dbo].[SecurityState]", RESEED, 1000)
END
GO

IF NOT EXISTS(SELECT * FROM SecurityUserState)
BEGIN
	SET IDENTITY_INSERT SecurityUserState ON
	INSERT INTO SecurityUserState (Description, UserStateID)
		VALUES ('Patch Tested', 1)
	INSERT INTO SecurityUserState (Description, UserStateID)
		VALUES ('Package Tested', 2)
	INSERT INTO SecurityUserState (Description, UserStateID)
		VALUES ('Signoff', 3)
	SET IDENTITY_INSERT SecurityUserState OFF
END
GO
IF NOT EXISTS (
	SELECT * FROM SecurityUserState
	WHERE UserStateID > 1000
)
BEGIN
	DBCC CHECKIDENT("[dbo].[SecurityUserState]", RESEED, 1000)
END
GO

IF NOT EXISTS(SELECT * FROM SecurityUIState)
BEGIN
	SET IDENTITY_INSERT SecurityUIState ON
	INSERT INTO SecurityUIState (Description, UIStateID)
		VALUES ('NewOrUpdated', 1)
	INSERT INTO SecurityUIState (Description, UIStateID)
		VALUES ('InProgress', 2)
	INSERT INTO SecurityUIState (Description, UIStateID)
		VALUES ('Released', 3)
	INSERT INTO SecurityUIState (Description, UIStateID)
		VALUES ('Hidden', 4)
	SET IDENTITY_INSERT SecurityUIState OFF
END
GO
IF NOT EXISTS (
	SELECT * FROM SecurityUIState
	WHERE UIStateID > 1000
)
BEGIN
	DBCC CHECKIDENT("[dbo].[SecurityUIState]", RESEED, 1000)
END
GO

IF NOT EXISTS(SELECT * FROM SecurityPatchStatus)
BEGIN
	SET IDENTITY_INSERT SecurityPatchStatus ON
	INSERT INTO SecurityPatchStatus (DisplayText, InternalText, SecurityPatchStatusID, Severity)
		VALUES ('Not applied', 'NotApplied', 1, 1)
	INSERT INTO SecurityPatchStatus (DisplayText, InternalText, SecurityPatchStatusID, Severity)
		VALUES ('Analyzed but unknown', 'Unknown', 2, 2)
	INSERT INTO SecurityPatchStatus (DisplayText, InternalText, SecurityPatchStatusID, Severity)
		VALUES ('Not analyzed', 'NotAnalyzed', 3, 3)
	INSERT INTO SecurityPatchStatus (DisplayText, InternalText, SecurityPatchStatusID, Severity)
		VALUES ('Applied with warning', 'Warning', 4, 4)
	INSERT INTO SecurityPatchStatus (DisplayText, InternalText, SecurityPatchStatusID, Severity)
		VALUES ('Not applicable', 'NotApplicable', 5, 5)
	INSERT INTO SecurityPatchStatus (DisplayText, InternalText, SecurityPatchStatusID, Severity)
		VALUES ('Superseded', 'Superseded', 6, 6)
	INSERT INTO SecurityPatchStatus (DisplayText, InternalText, SecurityPatchStatusID, Severity)
		VALUES ('Applied', 'Applied', 7, 7)
	SET IDENTITY_INSERT SecurityPatchStatus OFF
END
GO
IF NOT EXISTS (
	SELECT * FROM SecurityPatchStatus
	WHERE SecurityPatchStatusID > 1000
)
BEGIN
	DBCC CHECKIDENT("[dbo].[SecurityPatchStatus]", RESEED, 1000)
END
GO

IF NOT EXISTS(SELECT * FROM SecurityPatchUIProductGroup)
BEGIN
	SET IDENTITY_INSERT SecurityPatchUIProductGroup ON
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('Internet Explorer', 'Internet Explorer 4.0', 3)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('Internet Explorer', 'Internet Explorer 4.01', 4)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('Internet Explorer', 'Internet Explorer 5', 6)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('Internet Explorer', 'Internet Explorer 5.01', 7)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('Internet Explorer', 'Internet Explorer 5.5', 10)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('Internet Explorer', 'Internet Explorer 6', 11)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('Internet Explorer', 'Internet Explorer 6.0 for Windows Server 2003', 12)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('IIS', 'Internet Information Server 3.0', 13)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('IIS', 'Internet Information Server 4.0', 14)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('IIS', 'Internet Information Services 5.0', 15)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('IIS', 'Internet Information Services 5.1', 16)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('IIS', 'Internet Information Services 6.0', 17)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('SQL Server', 'Microsoft SQL Server 2000 Desktop Engine (MSDE 2000)', 18)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('SQL Server', 'SQL Server 2000', 19)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('SQL Server', 'SQL Server 7.0', 20)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('Windows 2000', 'Windows 2000 Advanced Server', 21)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('Windows 2000', 'Windows 2000 Datacenter Server', 22)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('Windows 2000', 'Windows 2000 Professional', 23)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('Windows 2000', 'Windows 2000 Server', 24)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('Windows 95', 'Windows 95', 25)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('Windows 98', 'Windows 98', 26)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('Windows 98', 'Windows 98 Resource Kit', 27)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('Windows 98', 'Windows 98 Resource Kit Sampler', 28)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('Windows 98', 'Windows 98 SE', 29)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('Windows Millennium', 'Windows Me', 30)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('Windows NT 4.0', 'Windows NT Server 4.0', 31)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('Windows NT 4.0', 'Windows NT Server 4.0, Enterprise Edition', 32)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('Windows NT 4.0', 'Windows NT Server 4.0, Terminal Server Edition', 33)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('Windows NT 4.0', 'Windows NT Workstation 4.0', 34)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('Windows Server 2003', 'Windows Server 2003 for Small Business Server', 35)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('Windows Server 2003', 'Windows Server 2003, Datacenter Edition', 36)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('Windows Server 2003', 'Windows Server 2003, Enterprise Edition', 37)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('Windows Server 2003', 'Windows Server 2003, Standard Edition', 38)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('Windows Server 2003', 'Windows Server 2003, Web Edition', 39)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('Windows XP', 'Windows XP Home Edition', 40)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('Windows XP', 'Windows XP Professional', 41)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('Microsoft Office 2000', 'Microsoft Office 2000', 42)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('Microsoft Office XP', 'Microsoft Office XP', 43)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('Microsoft Office 2003', 'Microsoft Office 2003', 44)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('Windows Media Player', 'Windows Media Player 6.4', 45)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('Windows Media Player', 'Windows Media Player 6.4 for Windows NT 4.0', 46)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('Windows Media Player', 'Windows Media Player 6.4 for Windows 2000', 47)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('Windows Media Player', 'Windows Media Player 6.4 for Windows XP', 48)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('Windows Media Player', 'Windows Media Player 6.4 for Windows Server 2003', 49)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('Windows Media Player', 'Windows Media Player 7.0', 50)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('Windows Media Player', 'Windows Media Player 7.1', 51)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('Windows Media Player', 'Windows Media Player 9.0', 52)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('Windows Media Player', 'Windows Media Player 9 Series', 53)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('Windows Media Player', 'Windows Media Player 9 Series for Windows XP', 54)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('Windows Media Player', 'Windows Media Player for Windows XP', 55)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('Windows Media Player', 'Windows Media Player 9 Series for Windows Server 2003', 56)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('Exchange Server', 'Exchange 2000 Enterprise Server', 57)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('Exchange Server', 'Exchange 2000 Server', 58)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('Exchange Server', 'Exchange Server', 59)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('Exchange Server', 'Exchange Server 2003', 60)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('Exchange Server', 'Exchange Server 5.0', 61)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('Exchange Server', 'Exchange Server 5.5', 62)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('MDAC', 'MDAC 2.1', 63)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('MDAC', 'MDAC 2.5', 64)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('MDAC', 'MDAC 2.6', 65)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('MDAC', 'MDAC 2.7', 66)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('MDAC', 'MDAC 2.8', 67)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('Virtual Machine', 'Microsoft Virtual Machine (VM)', 68)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('MSXML', 'MSXML 2.6', 69)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('MSXML', 'MSXML 3.0', 70)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('MSXML', 'MSXML 4.0', 71)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('MSXML', 'MSXML 6.0', 72)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('Visual Studio .NET', 'Visual Studio .NET 2002', 83)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('Visual Studio .NET', 'Visual Studio .NET 2003', 84)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('.NET Framework', '.NET Framework 1.0', 85)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('.NET Framework', '.NET Framework 1.0 SP2', 86)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('.NET Framework', '.NET Framework 1.0 SP3', 87)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('.NET Framework', '.NET Framework 1.1', 88)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('.NET Framework', '.NET Framework 1.1 RTM', 89)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('.NET Framework', '.NET Framework 1.1 RTM for Windows Server 2003', 90)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('.NET Framework', '.NET Framework 1.1 SP1', 91)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('.NET Framework', '.NET Framework 1.1 SP1 for Windows Server 2003', 92)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('Picture It!/Digital Image', 'Picture It! or Digital Image v6', 93)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('Picture It!/Digital Image', 'Picture It! or Digital Image v7', 94)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('Picture It!/Digital Image', 'Picture It! or Digital Image v9', 95)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('Microsoft Producer', 'Producer for 2003', 96)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('Journal Viewer', 'Journal Viewer 1.5', 97)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('Visual FoxPro', 'Visual FoxPro 8.0', 98)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('Windows Sharepoint Services', 'Windows Sharepoint Services', 99)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('MSN Messenger', 'Windows Messenger 4.7.0.2009', 100)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('MSN Messenger', 'Windows Messenger 4.7.0.3000', 101)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('MSN Messenger', 'Windows Messenger 5.0', 102)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('MSN Messenger', 'MSN Messenger', 103)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('Outlook Express', 'Outlook Express 4.01', 104)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('Outlook Express', 'Outlook Express 5.01', 105)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('Outlook Express', 'Outlook Express 5.5', 106)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('Outlook Express', 'Outlook Express 5.5 Service Pack 2', 107)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('Outlook Express', 'Outlook Express 5.5 SP2', 108)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('Outlook Express', 'Outlook Express 6', 109)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('Outlook Express', 'Outlook Express 6 on Windows Server 2003', 110)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('Outlook Express', 'Outlook Express 6 Service Pack 1', 111)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('Outlook Express', 'Outlook Express 6 SP1', 112)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('Outlook Express', 'Outlook Express 6.0', 113)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('Outlook Express', 'Outlook Express for Windows Server 2003', 114)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('Outlook Express', 'Outlook Express for Windows Server 2003 64-bit Itanium Edition', 115)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('BackOffice Server', 'BizTalk Server 2000', 116)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('BackOffice Server', 'BizTalk Server 2002', 117)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('BackOffice Server', 'BackOffice Server 4.0', 118)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('BackOffice Server', 'BackOffice Server 2000', 119)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('BackOffice Server', 'BackOffice Resource Kit SE', 120)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('BackOffice Server', 'Site Server 3.0', 121)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('BackOffice Server', 'Site Server 3.0, Commerce Edition', 122)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('BackOffice Server', 'Microsoft Commercial Internet System 2.0', 123)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('BackOffice Server', 'Microsoft Commercial Internet System 2.5', 124)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('BackOffice Server', 'Commerce Server 2000', 125)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('BackOffice Server', 'Commerce Server 2002', 126)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('BackOffice Server', 'ISA Server 2000', 127)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('BackOffice Server', 'Proxy Server 2.0', 128)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('BackOffice Server', 'Content Management Server 2001', 129)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('BackOffice Server', 'Small Business Server 2000', 130)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('BackOffice Server', 'Windows Small Business Server 2003', 131)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('BackOffice Server', 'Microsoft Business Solutions CRM', 132)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('Microsoft Works', 'Works 2000', 133)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('Microsoft Works', 'Works 2001', 134)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('Microsoft Works', 'Works 2002', 135)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('Microsoft Works', 'Works 2003', 136)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('Microsoft Works', 'Works 2004', 137)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('Microsoft Works', 'Money 2000', 138)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('Microsoft Works', 'Money 2001', 139)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('Microsoft Works', 'Greetings 2000', 140)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('Microsoft Works', 'Home Publishing 2000', 141)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('Microsoft Office 97', 'Office 97', 142)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('Microsoft Office 97', 'Access 97', 143)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('Microsoft Office 97', 'Excel 97', 144)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('Microsoft Office 97', 'PowerPoint 97', 145)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('Microsoft Office 97', 'Word 97', 146)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('Microsoft Office 2000', 'Access 2000', 147)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('Microsoft Office 2000', 'Excel 2000', 148)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('Microsoft Office 2000', 'FrontPage 2000', 149)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('Microsoft Office 2000', 'Office 2000', 150)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('Microsoft Office 2000', 'Office Web Components 2000', 151)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('Microsoft Office 2000', 'Outlook 2000', 152)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('Microsoft Office 2000', 'Powerpoint 2000', 153)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('Microsoft Office 2000', 'Publisher 2000', 154)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('Microsoft Office 2000', 'Word 2000', 155)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('Microsoft Office XP', 'Access 2002', 156)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('Microsoft Office XP', 'Excel 2002', 157)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('Microsoft Office XP', 'FrontPage 2002', 158)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('Microsoft Office XP', 'Office XP', 159)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('Microsoft Office XP', 'Office Web Components XP', 160)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('Microsoft Office XP', 'Office Web Components 2002', 161)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('Microsoft Office XP', 'Outlook 2002', 162)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('Microsoft Office XP', 'Powerpoint 2002', 163)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('Microsoft Office XP', 'Publisher 2002', 164)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('Microsoft Office XP', 'Word 2002', 165)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('Microsoft Office 2003', 'Access 2003', 166)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('Microsoft Office 2003', 'Excel 2003', 167)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('Microsoft Office 2003', 'FrontPage 2003', 168)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('Microsoft Office 2003', 'Office 2003', 169)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('Microsoft Office 2003', 'Office Web Components 2003', 170)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('Microsoft Office 2003', 'Outlook 2003', 171)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('Microsoft Office 2003', 'Powerpoint 2003', 172)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('Microsoft Office 2003', 'Publisher 2003', 173)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('Microsoft Office 2003', 'Word 2003', 174)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('MDAC', 'Microsoft Data Access Components', 175)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('.NET Framework', '.NET Framework', 176)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('Windows 2000', 'Windows 2000', 177)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('Windows NT 4.0', 'Windows NT 4.0', 178)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('Windows XP', 'Windows XP', 179)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('Windows Server 2003', 'Windows Server 2003', 180)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('Windows Server 2003', 'Windows Server 2003 64-bit Itanium Edition', 181)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('Windows Server 2003', 'Windows Server 2003 x64 Edition', 182)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('Windows XP', 'Windows XP x64 Edition', 183)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('Windows Media Player', 'Windows Media Services', 184)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('Windows Media Player', 'Windows Media Services 4.0', 185)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('Windows Media Player', 'Windows Media Services 4.1', 186)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('Windows Media Player', 'Windows Media Encoder 4.0', 187)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('Windows Media Player', 'Windows Media Encoder 4.1', 188)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('Windows Media Player', 'Windows Media Rights Manager 1', 189)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('Windows Media Player', 'Windows Media Player', 190)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('IIS', 'FrontPage 98 Personal Web Server 1.0', 191)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('IIS', 'FrontPage 97 Personal Web Server 1.0', 192)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('IIS', 'Personal Web Server 4.0', 193)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('IIS', 'FrontPage 97 Server Extensions', 194)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('IIS', 'FrontPage 98 Server Extensions', 195)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('IIS', 'FrontPage 2000 Server Extensions', 196)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('IIS', 'Internet Information Services', 197)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('IIS', 'Internet Information Services and Windows XP', 198)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('IIS', 'FrontPage Server Extensions 2002', 199)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('Virtual Machine', 'Microsoft Virtual Machine  Microsoft VM', 200)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('Internet Explorer', 'Internet Explorer for Windows 2000', 201)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('Internet Explorer', 'Internet Explorer for Windows Server 2003', 202)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('Internet Explorer', 'Internet Explorer for Windows Server 2003 64-bit Itanium Edition', 203)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('Internet Explorer', 'Internet Explorer for Windows Server 2003 x64 Edition', 204)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('Internet Explorer', 'Internet Explorer for Windows XP', 205)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('Internet Explorer', 'Internet Explorer for Windows XP Service Pack 2', 206)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('Internet Explorer', 'Internet Explorer for Windows XP x64 Edition', 207)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('Picture It!/Digital Image', 'PictureIt 2000', 208)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('MSN Messenger', 'Windows Messenger', 209)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('Windows Vista', 'Windows Vista', 210)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('Windows Vista', 'Windows Vista for x64-based Systems', 211)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('Windows Vista', 'Windows Vista Business', 212)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('Windows Vista', 'Windows Vista Enterprise', 213)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('Windows Vista', 'Windows Vista Home Basic', 214)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('Windows Vista', 'Windows Vista Home Premium', 215)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('Windows Vista', 'Windows Vista Ultimate', 216)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('Microsoft Office 2007', 'Microsoft Office 2007', 217)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('Microsoft Office 2007', 'Access 2007', 218)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('Microsoft Office 2007', 'Excel 2007', 219)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('Microsoft Office 2007', 'FrontPage 2007', 220)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('Microsoft Office 2007', 'Office 2007', 221)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('Microsoft Office 2007', 'Office Web Components 2007', 222)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('Microsoft Office 2007', 'Outlook 2007', 223)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('Microsoft Office 2007', 'Powerpoint 2007', 224)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('Microsoft Office 2007', 'Publisher 2007', 225)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('Microsoft Office 2007', 'Word 2007', 226)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('Internet Explorer', 'Internet Explorer 7 in Windows Vista', 227)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('Internet Explorer', 'Internet Explorer 7 in Windows Vista x64-based Systems', 228)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('Visual Studio .NET', 'Microsoft Visual Studio 2005', 229)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('Windows Server 2008', 'Windows Server 2008', 238)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('Windows Server 2008', 'Windows Server 2008 Datacenter', 239)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('Windows Server 2008', 'Windows Server 2008 Enterprise', 240)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('Windows Server 2008', 'Windows Server 2008 Standard', 241)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('Windows Server 2008', 'Windows Web Server 2008', 242)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('Windows Server 2008', 'Windows HPC Server 2008', 243)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('Windows Server 2008', 'Windows Server 2008 for Itanium-Based Systems', 244)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('Windows Server 2008', 'Windows Server 2008 Datacenter without Hyper-V', 245)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('Windows Server 2008', 'Windows Server 2008 Enterprise without Hyper-V', 246)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('Windows Server 2008', 'Windows Server 2008 Standard without Hyper-V', 247)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('Windows Server 2008', 'Windows Server 2008 x64 Edition', 248)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('Internet Explorer', 'Internet Explorer for Windows Server 2008', 249)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('Internet Explorer', 'Internet Explorer for Windows Server 2008 x64 Edition', 250)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('Windows 7', 'Windows 7', 251)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('Windows 7', 'Windows 7 for x64-based Systems', 252)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('Windows Server 2008 R2', 'Windows Server 2008 R2', 253)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('Windows Server 2008 R2', 'Windows Server 2008 R2 Itanium Edition', 254)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('Internet Explorer', 'Internet Explorer for Windows Vista', 255)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('Internet Explorer', 'Internet Explorer for Windows Vista x64-based Systems', 256)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('Internet Explorer', 'Internet Explorer for Windows 7', 257)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('Internet Explorer', 'Internet Explorer for Windows 7 x64-based Systems', 258)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('Internet Explorer', 'Internet Explorer for Windows Server 2008 R2', 259)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('Internet Explorer', 'Internet Explorer x64 for Windows Server 2008 R2', 260)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('BackOffice Server', 'BackOffice Server', 261)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('DirectX', 'DirectX', 262)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('DirectX', 'DirectX 7.0', 263)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('DirectX', 'DirectX 8.0', 264)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('DirectX', 'DirectX 8.1', 265)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('DirectX', 'DirectX 8.2', 266)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('DirectX', 'DirectX 9.0', 267)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('Internet Explorer', 'Internet Explorer', 268)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('Internet Explorer', 'Internet Explorer 8 for Windows Vista', 269)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('Internet Explorer', 'Internet Explorer 8 for Windows Vista for x64-based Systems', 270)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('ISA Server', 'ISA Server', 271)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('ISA Server', 'ISA Server 2004 Enterprise Edition', 272)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('ISA Server', 'ISA Server 2004 Standard Edition', 273)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('ISA Server', 'ISA Server 2006 RTM', 274)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('ISA Server', 'ISA Server 2006 SP1', 275)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('ISA Server', 'ISA Server 2006 Supportability Pack', 276)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('Microsoft Forefront', 'Microsoft Forefront', 277)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('Microsoft Forefront', 'Microsoft Forefront Client Security', 278)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('Windows 2000', 'Microsoft Jscript version Windows 2000', 279)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('Windows XP', 'Microsoft Jscript version Windows Windows XP', 280)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('Microsoft Works', 'Microsoft Works', 281)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('Microsoft Office 2007', 'OneNote 2007', 282)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('Microsoft Office XP', 'Outlook Express for XP', 283)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('Windows 2000', 'Windows 2000 with Service Pack 2 or Service Pack 3', 284)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('Journal Viewer', 'Windows Journal Viewer', 285)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('Windows Media Player', 'Windows Media Encoder', 286)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('Windows Media Player', 'Windows Media Encoder 9', 287)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('Windows XP', 'Windows XP or Windows XP Service Pack 1', 288)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('Microsoft Works', 'Works 2005', 289)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('Visual Studio .NET', 'Microsoft Visual Studio 2008', 290)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('SQL Server', 'SQL Server', 291)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('Visual Studio .NET', 'Microsoft Visual Studio', 292)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('Microsoft Office 2010', 'Microsoft Office 2010', 293)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('Microsoft Office 2013', 'Microsoft Office 2013', 294)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('Adobe Reader', 'Adobe Reader', 300)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('Adobe Flash Player', 'Adobe Flash Player', 301)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('Java', 'Java Runtime 8', 302)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('Adobe Acrobat Reader DC', 'Adobe Acrobat Reader DC', 303)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('VLC media player', 'VLC media player', 304)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('Windows 7', 'Windows Embedded Standard 7', 305)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('Windows 7', 'Windows Embedded Standard 7 for x64-based Systems', 306)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('Windows 8', 'Windows 8', 307)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('Windows 8', 'Windows 8 for x64-based Systems', 308)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('Windows Server 2012', 'Windows Server 2012', 309)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('Windows 8.1', 'Windows 8.1', 310)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('Windows 8.1', 'Windows 8.1 for x64-based Systems', 311)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('Windows Server 2012 R2', 'Windows Server 2012 R2', 312)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('Windows 10', 'Windows 10', 313)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('Google Chrome', 'Google Chrome Enterprise', 314)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('Windows 10', 'Windows 10 for x64-based Systems', 315)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('Windows 10', 'Windows 10 Version 1511', 316)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('Windows 10', 'Windows 10 Version 1511 for x64-based Systems', 317)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('Windows 10', 'Windows 10 Version 1607', 318)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('Windows 10', 'Windows 10 Version 1607 for x64-based Systems', 319)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('Windows Server 2016', 'Windows Server 2016 x64 Edition', 320)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('Adobe Air', 'Adobe Air', 323)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('Adobe Shockwave Player', 'Adobe Shockwave Player', 324)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('Windows', 'Adobe Flash Player for Windows', 325)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('Microsoft Office 2016', 'Microsoft Office 2016', 326)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('Windows 10', 'Windows 10 Version 1703', 327)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('Windows 10', 'Windows 10 Version 1703 for x64-based Systems', 328)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('Windows 10', 'Windows 10 Version 1709', 329)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('Windows 10', 'Windows 10 Version 1709 for x64-based Systems', 330)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('Java', 'Java Runtime 10', 331)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('Windows 10', 'Windows 10 Version 1803', 332)
	INSERT INTO SecurityPatchUIProductGroup (GroupName, Product, SecurityPatchUIProductGroupID)
		VALUES ('Windows 10', 'Windows 10 Version 1803 for x64-based Systems', 333)
	SET IDENTITY_INSERT SecurityPatchUIProductGroup OFF
END
GO
IF NOT EXISTS (
	SELECT * FROM SecurityPatchUIProductGroup
	WHERE SecurityPatchUIProductGroupID > 1000
)
BEGIN
	DBCC CHECKIDENT("[dbo].[SecurityPatchUIProductGroup]", RESEED, 1000)
END
GO

IF NOT EXISTS(SELECT * FROM SecurityPatchProductGroup)
BEGIN
	SET IDENTITY_INSERT SecurityPatchProductGroup ON
	INSERT INTO SecurityPatchProductGroup (Environment, Product, SecurityPatchProductGroupID)
		VALUES ('Microsoft Windows 2000 Server', 'Windows 2000 Advanced Server', 1)
	INSERT INTO SecurityPatchProductGroup (Environment, Product, SecurityPatchProductGroupID)
		VALUES ('Microsoft Windows 2000 Server', 'Windows 2000 Datacenter Server', 2)
	INSERT INTO SecurityPatchProductGroup (Environment, Product, SecurityPatchProductGroupID)
		VALUES ('Microsoft Windows 2000 Professional', 'Windows 2000 Professional', 3)
	INSERT INTO SecurityPatchProductGroup (Environment, Product, SecurityPatchProductGroupID)
		VALUES ('Microsoft Windows 2000 Server', 'Windows 2000 Server', 4)
	INSERT INTO SecurityPatchProductGroup (Environment, Product, SecurityPatchProductGroupID)
		VALUES ('Microsoft Windows 95', 'Windows 95', 5)
	INSERT INTO SecurityPatchProductGroup (Environment, Product, SecurityPatchProductGroupID)
		VALUES ('Microsoft Windows 98', 'Windows 98', 6)
	INSERT INTO SecurityPatchProductGroup (Environment, Product, SecurityPatchProductGroupID)
		VALUES ('Microsoft Windows 98', 'Windows 98 Resource Kit', 7)
	INSERT INTO SecurityPatchProductGroup (Environment, Product, SecurityPatchProductGroupID)
		VALUES ('Microsoft Windows 98', 'Windows 98 Resource Kit Sampler', 8)
	INSERT INTO SecurityPatchProductGroup (Environment, Product, SecurityPatchProductGroupID)
		VALUES ('Microsoft Windows 98', 'Windows 98 SE', 9)
	INSERT INTO SecurityPatchProductGroup (Environment, Product, SecurityPatchProductGroupID)
		VALUES ('Microsoft Windows Millennium Edition', 'Windows Me', 10)
	INSERT INTO SecurityPatchProductGroup (Environment, Product, SecurityPatchProductGroupID)
		VALUES ('Microsoft Windows NT 4.0', 'Windows NT Server 4.0', 11)
	INSERT INTO SecurityPatchProductGroup (Environment, Product, SecurityPatchProductGroupID)
		VALUES ('Microsoft Windows NT 4.0', 'Windows NT Server 4.0, Enterprise Edition', 12)
	INSERT INTO SecurityPatchProductGroup (Environment, Product, SecurityPatchProductGroupID)
		VALUES ('Microsoft Windows NT 4.0', 'Windows NT Server 4.0, Terminal Server Edition', 13)
	INSERT INTO SecurityPatchProductGroup (Environment, Product, SecurityPatchProductGroupID)
		VALUES ('Microsoft Windows NT 4.0', 'Windows NT Workstation 4.0', 14)
	INSERT INTO SecurityPatchProductGroup (Environment, Product, SecurityPatchProductGroupID)
		VALUES ('Microsoft Windows Server 2003', 'Windows Server 2003 for Small Business Server', 15)
	INSERT INTO SecurityPatchProductGroup (Environment, Product, SecurityPatchProductGroupID)
		VALUES ('Microsoft Windows Server 2003', 'Windows Server 2003, Datacenter Edition', 16)
	INSERT INTO SecurityPatchProductGroup (Environment, Product, SecurityPatchProductGroupID)
		VALUES ('Microsoft Windows Server 2003', 'Windows Server 2003, Enterprise Edition', 17)
	INSERT INTO SecurityPatchProductGroup (Environment, Product, SecurityPatchProductGroupID)
		VALUES ('Microsoft Windows Server 2003', 'Windows Server 2003, Standard Edition', 18)
	INSERT INTO SecurityPatchProductGroup (Environment, Product, SecurityPatchProductGroupID)
		VALUES ('Microsoft Windows Server 2003', 'Windows Server 2003, Web Edition', 19)
	INSERT INTO SecurityPatchProductGroup (Environment, Product, SecurityPatchProductGroupID)
		VALUES ('Microsoft Windows XP', 'Windows XP Home Edition', 20)
	INSERT INTO SecurityPatchProductGroup (Environment, Product, SecurityPatchProductGroupID)
		VALUES ('Microsoft Windows XP', 'Windows XP Professional', 21)
	INSERT INTO SecurityPatchProductGroup (Environment, Product, SecurityPatchProductGroupID)
		VALUES ('Microsoft Windows XP Pro x64', 'Windows XP x64 Edition', 22)
	INSERT INTO SecurityPatchProductGroup (Environment, Product, SecurityPatchProductGroupID)
		VALUES ('Microsoft Windows Vista', 'Windows Vista Business', 23)
	INSERT INTO SecurityPatchProductGroup (Environment, Product, SecurityPatchProductGroupID)
		VALUES ('Microsoft Windows Vista', 'Windows Vista Enterprise', 24)
	INSERT INTO SecurityPatchProductGroup (Environment, Product, SecurityPatchProductGroupID)
		VALUES ('Microsoft Windows Vista', 'Windows Vista Home Basic', 25)
	INSERT INTO SecurityPatchProductGroup (Environment, Product, SecurityPatchProductGroupID)
		VALUES ('Microsoft Windows Vista', 'Windows Vista Home Premium', 26)
	INSERT INTO SecurityPatchProductGroup (Environment, Product, SecurityPatchProductGroupID)
		VALUES ('Microsoft Windows Vista', 'Windows Vista Ultimate', 27)
	INSERT INTO SecurityPatchProductGroup (Environment, Product, SecurityPatchProductGroupID)
		VALUES ('Microsoft Windows Server 2008', 'Windows Server 2008 Datacenter', 28)
	INSERT INTO SecurityPatchProductGroup (Environment, Product, SecurityPatchProductGroupID)
		VALUES ('Microsoft Windows Server 2008', 'Windows Server 2008 Enterprise', 29)
	INSERT INTO SecurityPatchProductGroup (Environment, Product, SecurityPatchProductGroupID)
		VALUES ('Microsoft Windows Server 2008', 'Windows Server 2008 Standard', 30)
	INSERT INTO SecurityPatchProductGroup (Environment, Product, SecurityPatchProductGroupID)
		VALUES ('Microsoft Windows Server 2008', 'Windows Web Server 2008', 31)
	INSERT INTO SecurityPatchProductGroup (Environment, Product, SecurityPatchProductGroupID)
		VALUES ('Microsoft Windows Server 2008', 'Windows HPC Server 2008', 32)
	INSERT INTO SecurityPatchProductGroup (Environment, Product, SecurityPatchProductGroupID)
		VALUES ('Microsoft Windows Server 2008', 'Windows Server 2008 for Itanium-Based Systems', 33)
	INSERT INTO SecurityPatchProductGroup (Environment, Product, SecurityPatchProductGroupID)
		VALUES ('Microsoft Windows Server 2008', 'Windows Server 2008 Datacenter without Hyper-V', 34)
	INSERT INTO SecurityPatchProductGroup (Environment, Product, SecurityPatchProductGroupID)
		VALUES ('Microsoft Windows Server 2008', 'Windows Server 2008 Enterprise without Hyper-V', 35)
	INSERT INTO SecurityPatchProductGroup (Environment, Product, SecurityPatchProductGroupID)
		VALUES ('Microsoft Windows Server 2008', 'Windows Server 2008 Standard without Hyper-V', 36)
	INSERT INTO SecurityPatchProductGroup (Environment, Product, SecurityPatchProductGroupID)
		VALUES ('Microsoft Windows 7', 'Windows 7 Starter', 37)
	INSERT INTO SecurityPatchProductGroup (Environment, Product, SecurityPatchProductGroupID)
		VALUES ('Microsoft Windows 7', 'Windows 7 Home Basic', 38)
	INSERT INTO SecurityPatchProductGroup (Environment, Product, SecurityPatchProductGroupID)
		VALUES ('Microsoft Windows 7', 'Windows 7 Home Premium', 39)
	INSERT INTO SecurityPatchProductGroup (Environment, Product, SecurityPatchProductGroupID)
		VALUES ('Microsoft Windows 7', 'Windows 7 Professional', 40)
	INSERT INTO SecurityPatchProductGroup (Environment, Product, SecurityPatchProductGroupID)
		VALUES ('Microsoft Windows 7', 'Windows 7 Enterprise', 41)
	INSERT INTO SecurityPatchProductGroup (Environment, Product, SecurityPatchProductGroupID)
		VALUES ('Microsoft Windows 7', 'Windows 7 Ultimate', 42)
	SET IDENTITY_INSERT SecurityPatchProductGroup OFF
END
GO
IF NOT EXISTS (
	SELECT * FROM SecurityPatchProductGroup
	WHERE SecurityPatchProductGroupID > 1000
)
BEGIN
	DBCC CHECKIDENT("[dbo].[SecurityPatchProductGroup]", RESEED, 1000)
END
GO

IF NOT EXISTS(SELECT * FROM SecurityPatchLanguage)
BEGIN
	INSERT INTO SecurityPatchLanguage (Description, LanguageName, Selected, Visible)
		VALUES ('Arabic', 'ara', 0, 0)
	INSERT INTO SecurityPatchLanguage (Description, LanguageName, Selected, Visible)
		VALUES ('Chinese - Hong Kong SAR', 'zho-hk', 0, 0)
	INSERT INTO SecurityPatchLanguage (Description, LanguageName, Selected, Visible)
		VALUES ('Chinese - Simplified', 'zho-cn', 0, 0)
	INSERT INTO SecurityPatchLanguage (Description, LanguageName, Selected, Visible)
		VALUES ('Chinese - Traditional', 'zho-tw', 0, 0)
	INSERT INTO SecurityPatchLanguage (Description, LanguageName, Selected, Visible)
		VALUES ('Czech', 'ces', 0, 0)
	INSERT INTO SecurityPatchLanguage (Description, LanguageName, Selected, Visible)
		VALUES ('Danish', 'dan', 0, 0)
	INSERT INTO SecurityPatchLanguage (Description, LanguageName, Selected, Visible)
		VALUES ('Dutch', 'nld', 0, 0)
	INSERT INTO SecurityPatchLanguage (Description, LanguageName, Selected, Visible)
		VALUES ('English*', 'eng', 1, 1)
	INSERT INTO SecurityPatchLanguage (Description, LanguageName, Selected, Visible)
		VALUES ('Finnish', 'fin', 0, 0)
	INSERT INTO SecurityPatchLanguage (Description, LanguageName, Selected, Visible)
		VALUES ('French*', 'fra', 0, 1)
	INSERT INTO SecurityPatchLanguage (Description, LanguageName, Selected, Visible)
		VALUES ('German*', 'deu', 0, 1)
	INSERT INTO SecurityPatchLanguage (Description, LanguageName, Selected, Visible)
		VALUES ('Greek', 'ell-gr', 0, 0)
	INSERT INTO SecurityPatchLanguage (Description, LanguageName, Selected, Visible)
		VALUES ('Hebrew', 'heb', 0, 0)
	INSERT INTO SecurityPatchLanguage (Description, LanguageName, Selected, Visible)
		VALUES ('Hungarian', 'hun', 0, 0)
	INSERT INTO SecurityPatchLanguage (Description, LanguageName, Selected, Visible)
		VALUES ('Italian', 'ita', 0, 0)
	INSERT INTO SecurityPatchLanguage (Description, LanguageName, Selected, Visible)
		VALUES ('Japanese*', 'jpn', 0, 1)
	INSERT INTO SecurityPatchLanguage (Description, LanguageName, Selected, Visible)
		VALUES ('Korean', 'kor', 0, 0)
	INSERT INTO SecurityPatchLanguage (Description, LanguageName, Selected, Visible)
		VALUES ('Norwegian - Bokmal', 'nob-no', 0, 0)
	INSERT INTO SecurityPatchLanguage (Description, LanguageName, Selected, Visible)
		VALUES ('Norwegian - Nynorsk', 'nno-no', 0, 0)
	INSERT INTO SecurityPatchLanguage (Description, LanguageName, Selected, Visible)
		VALUES ('Polish', 'pol', 0, 0)
	INSERT INTO SecurityPatchLanguage (Description, LanguageName, Selected, Visible)
		VALUES ('Portuguese (Portugal)', 'por-pt', 0, 0)
	INSERT INTO SecurityPatchLanguage (Description, LanguageName, Selected, Visible)
		VALUES ('Portuguese (Brazil)', 'por-br', 0, 0)
	INSERT INTO SecurityPatchLanguage (Description, LanguageName, Selected, Visible)
		VALUES ('Russian', 'rus', 0, 0)
	INSERT INTO SecurityPatchLanguage (Description, LanguageName, Selected, Visible)
		VALUES ('Slovenian', 'slv', 0, 0)
	INSERT INTO SecurityPatchLanguage (Description, LanguageName, Selected, Visible)
		VALUES ('Slovak', 'slk', 0, 0)
	INSERT INTO SecurityPatchLanguage (Description, LanguageName, Selected, Visible)
		VALUES ('Spanish', 'spa', 0, 0)
	INSERT INTO SecurityPatchLanguage (Description, LanguageName, Selected, Visible)
		VALUES ('Swedish', 'swe', 0, 0)
	INSERT INTO SecurityPatchLanguage (Description, LanguageName, Selected, Visible)
		VALUES ('Thai', 'tha', 0, 0)
	INSERT INTO SecurityPatchLanguage (Description, LanguageName, Selected, Visible)
		VALUES ('Turkish', 'tur', 0, 0)
END
GO

IF NOT EXISTS(SELECT * FROM SecurityPatchOperatingSystem)
BEGIN
	INSERT INTO SecurityPatchOperatingSystem (Description, OperatingSystemID, Selected, Visible)
		VALUES ('Microsoft Windows 2000', 21, 0, 0)
	INSERT INTO SecurityPatchOperatingSystem (Description, OperatingSystemID, Selected, Visible)
		VALUES ('Microsoft Windows 95', 25, 0, 0)
	INSERT INTO SecurityPatchOperatingSystem (Description, OperatingSystemID, Selected, Visible)
		VALUES ('Microsoft Windows 98', 26, 0, 0)
	INSERT INTO SecurityPatchOperatingSystem (Description, OperatingSystemID, Selected, Visible)
		VALUES ('Microsoft Windows Millennium', 30, 0, 0)
	INSERT INTO SecurityPatchOperatingSystem (Description, OperatingSystemID, Selected, Visible)
		VALUES ('Microsoft Windows NT 4', 31, 0, 0)
	INSERT INTO SecurityPatchOperatingSystem (Description, OperatingSystemID, Selected, Visible)
		VALUES ('Microsoft Windows Server 2003', 180, 0, 0)
	INSERT INTO SecurityPatchOperatingSystem (Description, OperatingSystemID, Selected, Visible)
		VALUES ('Microsoft Windows XP', 179, 0, 0)
	INSERT INTO SecurityPatchOperatingSystem (Description, OperatingSystemID, Selected, Visible)
		VALUES ('Microsoft Windows XP Pro x64', 183, 0, 0)
	INSERT INTO SecurityPatchOperatingSystem (Description, OperatingSystemID, Selected, Visible)
		VALUES ('Microsoft Windows Vista', 210, 0, 0)
	INSERT INTO SecurityPatchOperatingSystem (Description, OperatingSystemID, Selected, Visible)
		VALUES ('Microsoft Windows Vista x64', 211, 0, 0)
	INSERT INTO SecurityPatchOperatingSystem (Description, OperatingSystemID, Selected, Visible)
		VALUES ('Microsoft Windows Server 2008', 238, 1, 1)
	INSERT INTO SecurityPatchOperatingSystem (Description, OperatingSystemID, Selected, Visible)
		VALUES ('Microsoft Windows Server 2008 x64', 248, 1, 1)
	INSERT INTO SecurityPatchOperatingSystem (Description, OperatingSystemID, Selected, Visible)
		VALUES ('Microsoft Windows Server 2008 x64 IB', 244, 0, 1)
	INSERT INTO SecurityPatchOperatingSystem (Description, OperatingSystemID, Selected, Visible)
		VALUES ('Microsoft Windows Server 2008 R2', 253, 1, 1)
	INSERT INTO SecurityPatchOperatingSystem (Description, OperatingSystemID, Selected, Visible)
		VALUES ('Microsoft Windows Server 2008 R2 IB', 254, 0, 1)
	INSERT INTO SecurityPatchOperatingSystem (Description, OperatingSystemID, Selected, Visible)
		VALUES ('Microsoft Windows 7', 251, 1, 1)
	INSERT INTO SecurityPatchOperatingSystem (Description, OperatingSystemID, Selected, Visible)
		VALUES ('Microsoft Windows 7 x64', 252, 1, 1)
	INSERT INTO SecurityPatchOperatingSystem (Description, OperatingSystemID, Selected, Visible)
		VALUES ('Microsoft Windows ES 7', 305, 0, 1)
	INSERT INTO SecurityPatchOperatingSystem (Description, OperatingSystemID, Selected, Visible)
		VALUES ('Microsoft Windows ES 7 x64', 306, 0, 1)
	INSERT INTO SecurityPatchOperatingSystem (Description, OperatingSystemID, Selected, Visible)
		VALUES ('Microsoft Windows 8', 307, 1, 1)
	INSERT INTO SecurityPatchOperatingSystem (Description, OperatingSystemID, Selected, Visible)
		VALUES ('Microsoft Windows 8 x64', 308, 1, 1)
	INSERT INTO SecurityPatchOperatingSystem (Description, OperatingSystemID, Selected, Visible)
		VALUES ('Microsoft Windows Server 2012', 309, 1, 1)
	INSERT INTO SecurityPatchOperatingSystem (Description, OperatingSystemID, Selected, Visible)
		VALUES ('Microsoft Windows 8.1', 310, 1, 1)
	INSERT INTO SecurityPatchOperatingSystem (Description, OperatingSystemID, Selected, Visible)
		VALUES ('Microsoft Windows 8.1 x64', 311, 1, 1)
	INSERT INTO SecurityPatchOperatingSystem (Description, OperatingSystemID, Selected, Visible)
		VALUES ('Microsoft Windows Server 2012 R2', 312, 1, 1)
	INSERT INTO SecurityPatchOperatingSystem (Description, OperatingSystemID, Selected, Visible)
		VALUES ('Microsoft Windows 10', 313, 1, 1)
	INSERT INTO SecurityPatchOperatingSystem (Description, OperatingSystemID, Selected, Visible)
		VALUES ('Microsoft Windows 10 x64', 315, 1, 1)
	INSERT INTO SecurityPatchOperatingSystem (Description, OperatingSystemID, Selected, Visible)
		VALUES ('Microsoft Windows 10 v1511', 316, 1, 1)
	INSERT INTO SecurityPatchOperatingSystem (Description, OperatingSystemID, Selected, Visible)
		VALUES ('Microsoft Windows 10 v1511 x64', 317, 1, 1)
	INSERT INTO SecurityPatchOperatingSystem (Description, OperatingSystemID, Selected, Visible)
		VALUES ('Microsoft Windows 10 v1607', 318, 1, 1)
	INSERT INTO SecurityPatchOperatingSystem (Description, OperatingSystemID, Selected, Visible)
		VALUES ('Microsoft Windows 10 v1607 x64', 319, 1, 1)
	INSERT INTO SecurityPatchOperatingSystem (Description, OperatingSystemID, Selected, Visible)
		VALUES ('Microsoft Windows Server 2016 x64', 320, 1, 1)
	INSERT INTO SecurityPatchOperatingSystem (Description, OperatingSystemID, Selected, Visible)
		VALUES ('Microsoft Windows 10 v1703', 327, 1, 1)
	INSERT INTO SecurityPatchOperatingSystem (Description, OperatingSystemID, Selected, Visible)
		VALUES ('Microsoft Windows 10 v1703 x64', 328, 1, 1)
	INSERT INTO SecurityPatchOperatingSystem (Description, OperatingSystemID, Selected, Visible)
		VALUES ('Microsoft Windows 10 v1709', 329, 1, 1)
	INSERT INTO SecurityPatchOperatingSystem (Description, OperatingSystemID, Selected, Visible)
		VALUES ('Microsoft Windows 10 v1709 x64', 330, 1, 1)
	INSERT INTO SecurityPatchOperatingSystem (Description, OperatingSystemID, Selected, Visible)
		VALUES ('Microsoft Windows 10 v1803', 332, 1, 1)
	INSERT INTO SecurityPatchOperatingSystem (Description, OperatingSystemID, Selected, Visible)
		VALUES ('Microsoft Windows 10 v1803 x64', 333, 1, 1)
END
GO

IF NOT EXISTS(SELECT * FROM SecurityPatchArchitecture)
BEGIN
	INSERT INTO SecurityPatchArchitecture (ArchitectureID, Description, Selected, Visible)
		VALUES (1, 'Intel i386', 1, 1)
	INSERT INTO SecurityPatchArchitecture (ArchitectureID, Description, Selected, Visible)
		VALUES (2, 'Intel i586', 1, 1)
	INSERT INTO SecurityPatchArchitecture (ArchitectureID, Description, Selected, Visible)
		VALUES (3, 'Intel i686', 1, 1)
	INSERT INTO SecurityPatchArchitecture (ArchitectureID, Description, Selected, Visible)
		VALUES (4, 'AMD Athlon', 1, 1)
END
GO

IF NOT EXISTS(SELECT * FROM SecurityPatchModel)
BEGIN
	INSERT INTO SecurityPatchModel (Description, ModelID, Selected, Visible)
		VALUES ('SMQR', 1, 1, 1)
	INSERT INTO SecurityPatchModel (Description, ModelID, Selected, Visible)
		VALUES ('SoQU', 2, 1, 1)
	INSERT INTO SecurityPatchModel (Description, ModelID, Selected, Visible)
		VALUES ('SaQR', 3, 1, 1)
	INSERT INTO SecurityPatchModel (Description, ModelID, Selected, Visible)
		VALUES ('SoU', 4, 1, 1)
END
GO

IF NOT EXISTS(SELECT * FROM SecurityPatchApplication)
BEGIN
	INSERT INTO SecurityPatchApplication (ApplicationID, Description, Selected, Visible)
		VALUES (57, 'Microsoft Exchange Server', 1, 1)
	INSERT INTO SecurityPatchApplication (ApplicationID, Description, Selected, Visible)
		VALUES (13, 'Internet Information Services', 1, 1)
	INSERT INTO SecurityPatchApplication (ApplicationID, Description, Selected, Visible)
		VALUES (3, 'Microsoft Internet Explorer', 1, 1)
	INSERT INTO SecurityPatchApplication (ApplicationID, Description, Selected, Visible)
		VALUES (63, 'MDAC', 1, 1)
	INSERT INTO SecurityPatchApplication (ApplicationID, Description, Selected, Visible)
		VALUES (42, 'Microsoft Office 2000', 0, 0)
	INSERT INTO SecurityPatchApplication (ApplicationID, Description, Selected, Visible)
		VALUES (44, 'Microsoft Office 2003', 0, 0)
	INSERT INTO SecurityPatchApplication (ApplicationID, Description, Selected, Visible)
		VALUES (43, 'Microsoft Office XP', 0, 0)
	INSERT INTO SecurityPatchApplication (ApplicationID, Description, Selected, Visible)
		VALUES (69, 'MSXML', 1, 1)
	INSERT INTO SecurityPatchApplication (ApplicationID, Description, Selected, Visible)
		VALUES (18, 'SQL Server', 1, 1)
	INSERT INTO SecurityPatchApplication (ApplicationID, Description, Selected, Visible)
		VALUES (68, 'Virtual Machine', 1, 1)
	INSERT INTO SecurityPatchApplication (ApplicationID, Description, Selected, Visible)
		VALUES (45, 'Microsoft Windows Media Player', 1, 1)
	INSERT INTO SecurityPatchApplication (ApplicationID, Description, Selected, Visible)
		VALUES (83, 'Visual Studio .NET', 1, 1)
	INSERT INTO SecurityPatchApplication (ApplicationID, Description, Selected, Visible)
		VALUES (85, '.NET Framework', 1, 1)
	INSERT INTO SecurityPatchApplication (ApplicationID, Description, Selected, Visible)
		VALUES (93, 'Picture It!/Digital Image', 1, 1)
	INSERT INTO SecurityPatchApplication (ApplicationID, Description, Selected, Visible)
		VALUES (96, 'Microsoft Producer', 1, 1)
	INSERT INTO SecurityPatchApplication (ApplicationID, Description, Selected, Visible)
		VALUES (97, 'Journal Viewer', 1, 1)
	INSERT INTO SecurityPatchApplication (ApplicationID, Description, Selected, Visible)
		VALUES (98, 'Visual FoxPro', 1, 1)
	INSERT INTO SecurityPatchApplication (ApplicationID, Description, Selected, Visible)
		VALUES (99, 'Windows Sharepoint Services', 1, 1)
	INSERT INTO SecurityPatchApplication (ApplicationID, Description, Selected, Visible)
		VALUES (100, 'MSN Messenger', 1, 1)
	INSERT INTO SecurityPatchApplication (ApplicationID, Description, Selected, Visible)
		VALUES (104, 'Outlook Express', 1, 1)
	INSERT INTO SecurityPatchApplication (ApplicationID, Description, Selected, Visible)
		VALUES (116, 'BackOffice Server', 1, 1)
	INSERT INTO SecurityPatchApplication (ApplicationID, Description, Selected, Visible)
		VALUES (133, 'Microsoft Works', 1, 1)
	INSERT INTO SecurityPatchApplication (ApplicationID, Description, Selected, Visible)
		VALUES (142, 'Microsoft Office 97', 0, 0)
	INSERT INTO SecurityPatchApplication (ApplicationID, Description, Selected, Visible)
		VALUES (217, 'Microsoft Office 2007', 0, 1)
	INSERT INTO SecurityPatchApplication (ApplicationID, Description, Selected, Visible)
		VALUES (262, 'DirectX', 1, 1)
	INSERT INTO SecurityPatchApplication (ApplicationID, Description, Selected, Visible)
		VALUES (271, 'ISA Server', 1, 1)
	INSERT INTO SecurityPatchApplication (ApplicationID, Description, Selected, Visible)
		VALUES (277, 'Microsoft Forefront', 1, 1)
	INSERT INTO SecurityPatchApplication (ApplicationID, Description, Selected, Visible)
		VALUES (293, 'Microsoft Office 2010', 1, 1)
	INSERT INTO SecurityPatchApplication (ApplicationID, Description, Selected, Visible)
		VALUES (294, 'Microsoft Office 2013', 1, 1)
	INSERT INTO SecurityPatchApplication (ApplicationID, Description, Selected, Visible)
		VALUES (300, 'Adobe Reader', 0, 1)
	INSERT INTO SecurityPatchApplication (ApplicationID, Description, Selected, Visible)
		VALUES (301, 'Adobe Flash Player', 0, 1)
	INSERT INTO SecurityPatchApplication (ApplicationID, Description, Selected, Visible)
		VALUES (302, 'Java Runtime 8', 0, 1)
	INSERT INTO SecurityPatchApplication (ApplicationID, Description, Selected, Visible)
		VALUES (303, 'Adobe Acrobat Reader DC', 0, 1)
	INSERT INTO SecurityPatchApplication (ApplicationID, Description, Selected, Visible)
		VALUES (304, 'VLC media player', 0, 1)
	INSERT INTO SecurityPatchApplication (ApplicationID, Description, Selected, Visible)
		VALUES (314, 'Google Chrome', 0, 1)
	INSERT INTO SecurityPatchApplication (ApplicationID, Description, Selected, Visible)
		VALUES (323, 'Adobe Air', 0, 1)
	INSERT INTO SecurityPatchApplication (ApplicationID, Description, Selected, Visible)
		VALUES (324, 'Adobe Shockwave Player', 0, 1)
	INSERT INTO SecurityPatchApplication (ApplicationID, Description, Selected, Visible)
		VALUES (325, 'Adobe Flash Player for Windows', 1, 1)
	INSERT INTO SecurityPatchApplication (ApplicationID, Description, Selected, Visible)
		VALUES (326, 'Microsoft Office 2016', 1, 1)
	INSERT INTO SecurityPatchApplication (ApplicationID, Description, Selected, Visible)
		VALUES (331, 'Java Runtime 10', 0, 1)
END
GO

IF NOT EXISTS(SELECT * FROM SecurityBulletinChangeType)
BEGIN
	SET IDENTITY_INSERT SecurityBulletinChangeType ON
	INSERT INTO SecurityBulletinChangeType (Description, SecurityBulletinChangeTypeID)
		VALUES ('BulletinRevision', 1)
	INSERT INTO SecurityBulletinChangeType (Description, SecurityBulletinChangeTypeID)
		VALUES ('PatchCommandLines', 2)
	INSERT INTO SecurityBulletinChangeType (Description, SecurityBulletinChangeTypeID)
		VALUES ('PatchDownloadLocations', 3)
	INSERT INTO SecurityBulletinChangeType (Description, SecurityBulletinChangeTypeID)
		VALUES ('BulletinInformation', 4)
	SET IDENTITY_INSERT SecurityBulletinChangeType OFF
END
GO
IF NOT EXISTS (
	SELECT * FROM SecurityBulletinChangeType
	WHERE SecurityBulletinChangeTypeID > 1000
)
BEGIN
	DBCC CHECKIDENT("[dbo].[SecurityBulletinChangeType]", RESEED, 1000)
END
GO

IF NOT EXISTS(SELECT * FROM ThreatType)
BEGIN
	SET IDENTITY_INSERT ThreatType ON
	INSERT INTO ThreatType (Name, ThreatTypeID)
		VALUES ('malware', 1)
	INSERT INTO ThreatType (Name, ThreatTypeID)
		VALUES ('patch', 2)
	INSERT INTO ThreatType (Name, ThreatTypeID)
		VALUES ('vulnerability', 3)
	SET IDENTITY_INSERT ThreatType OFF
END
GO
IF NOT EXISTS (
	SELECT * FROM ThreatType
	WHERE ThreatTypeID > 1000
)
BEGIN
	DBCC CHECKIDENT("[dbo].[ThreatType]", RESEED, 1000)
END
GO

IF NOT EXISTS(SELECT * FROM ThreatCategory)
BEGIN
	SET IDENTITY_INSERT ThreatCategory ON
	INSERT INTO ThreatCategory (Name, ThreatCategoryID, ThreatTypeID)
		VALUES ('Windows', 1, 3)
	INSERT INTO ThreatCategory (Name, ThreatCategoryID, ThreatTypeID)
		VALUES ('Password', 2, 3)
	INSERT INTO ThreatCategory (Name, ThreatCategoryID, ThreatTypeID)
		VALUES ('IIS', 3, 3)
	INSERT INTO ThreatCategory (Name, ThreatCategoryID, ThreatTypeID)
		VALUES ('SQL Server', 4, 3)
	SET IDENTITY_INSERT ThreatCategory OFF
END
GO
IF NOT EXISTS (
	SELECT * FROM ThreatCategory
	WHERE ThreatCategoryID > 1000
)
BEGIN
	DBCC CHECKIDENT("[dbo].[ThreatCategory]", RESEED, 1000)
END
GO

IF NOT EXISTS(SELECT * FROM ThreatAction)
BEGIN
	SET IDENTITY_INSERT ThreatAction ON
	INSERT INTO ThreatAction (Name, ThreatActionID)
		VALUES ('none', 1)
	INSERT INTO ThreatAction (Name, ThreatActionID)
		VALUES ('ignore', 2)
	INSERT INTO ThreatAction (Name, ThreatActionID)
		VALUES ('quarantine', 3)
	INSERT INTO ThreatAction (Name, ThreatActionID)
		VALUES ('restore', 4)
	INSERT INTO ThreatAction (Name, ThreatActionID)
		VALUES ('clean', 5)
	SET IDENTITY_INSERT ThreatAction OFF
END
GO
IF NOT EXISTS (
	SELECT * FROM ThreatAction
	WHERE ThreatActionID > 1000
)
BEGIN
	DBCC CHECKIDENT("[dbo].[ThreatAction]", RESEED, 1000)
END
GO

IF NOT EXISTS(SELECT * FROM ThreatRisk)
BEGIN
	SET IDENTITY_INSERT ThreatRisk ON
	INSERT INTO ThreatRisk (Name, Severity, ThreatRiskID)
		VALUES ('none', 0, 1)
	INSERT INTO ThreatRisk (Name, Severity, ThreatRiskID)
		VALUES ('not applicable', 1, 2)
	INSERT INTO ThreatRisk (Name, Severity, ThreatRiskID)
		VALUES ('unknown', 2, 3)
	INSERT INTO ThreatRisk (Name, Severity, ThreatRiskID)
		VALUES ('low', 3, 4)
	INSERT INTO ThreatRisk (Name, Severity, ThreatRiskID)
		VALUES ('guarded', 4, 5)
	INSERT INTO ThreatRisk (Name, Severity, ThreatRiskID)
		VALUES ('elevated', 5, 6)
	INSERT INTO ThreatRisk (Name, Severity, ThreatRiskID)
		VALUES ('high', 6, 7)
	INSERT INTO ThreatRisk (Name, Severity, ThreatRiskID)
		VALUES ('critical', 7, 8)
	SET IDENTITY_INSERT ThreatRisk OFF
END
GO
IF NOT EXISTS (
	SELECT * FROM ThreatRisk
	WHERE ThreatRiskID > 1000
)
BEGIN
	DBCC CHECKIDENT("[dbo].[ThreatRisk]", RESEED, 1000)
END
GO

IF NOT EXISTS(SELECT * FROM ThreatScanner)
BEGIN
	SET IDENTITY_INSERT ThreatScanner ON
	INSERT INTO ThreatScanner (Name, ThreatScannerID)
		VALUES ('Unknown', 0)
	INSERT INTO ThreatScanner (Manufacturer, Name, ThreatScannerID, Title, Version)
		VALUES ('Microsoft', 'MBSA 1.2', 1, 'Microsoft Baseline Security Analyzer', '1.2')
	INSERT INTO ThreatScanner (Manufacturer, Name, ThreatScannerID, Title, Version)
		VALUES ('Microsoft', 'MBSA 2.0', 2, 'Microsoft Baseline Security Analyzer', '2.0')
	INSERT INTO ThreatScanner (Manufacturer, Name, ThreatScannerID, Title, Version)
		VALUES ('Aluria', 'Spyware Eliminator 1.0', 3, 'Aluria Spyware Eliminator', '1.0')
	INSERT INTO ThreatScanner (Manufacturer, Name, ThreatScannerID, Title)
		VALUES ('Microsoft', 'EST', 4, 'Microsoft Enterprise Update Scan Tool')
	SET IDENTITY_INSERT ThreatScanner OFF
END
GO
IF NOT EXISTS (
	SELECT * FROM ThreatScanner
	WHERE ThreatScannerID > 1000
)
BEGIN
	DBCC CHECKIDENT("[dbo].[ThreatScanner]", RESEED, 1000)
END
GO

IF NOT EXISTS(SELECT * FROM ThreatScannerToType)
BEGIN
	INSERT INTO ThreatScannerToType (ThreatScannerID, ThreatTypeID)
		VALUES (1, 1)
	INSERT INTO ThreatScannerToType (ThreatScannerID, ThreatTypeID)
		VALUES (2, 2)
	INSERT INTO ThreatScannerToType (ThreatScannerID, ThreatTypeID)
		VALUES (2, 3)
	INSERT INTO ThreatScannerToType (ThreatScannerID, ThreatTypeID)
		VALUES (3, 2)
	INSERT INTO ThreatScannerToType (ThreatScannerID, ThreatTypeID)
		VALUES (3, 3)
	INSERT INTO ThreatScannerToType (ThreatScannerID, ThreatTypeID)
		VALUES (4, 2)
END
GO

IF NOT EXISTS(SELECT * FROM BlockedApplication)
BEGIN
	INSERT INTO BlockedApplication (ApplicationName, Description, FileName, Visible)
		VALUES ('Ares', 'Ares filesharing application', 'Ares.exe', 1)
	INSERT INTO BlockedApplication (ApplicationName, Description, FileName, Visible)
		VALUES ('Ares', 'Ares filesharing application', 'Areslite.exe', 1)
	INSERT INTO BlockedApplication (ApplicationName, Description, FileName, Visible)
		VALUES ('AIMaster Peer to Peer', 'AIMaster Peer to Peer filesharing application', 'Buddyizer.exe', 1)
	INSERT INTO BlockedApplication (ApplicationName, Description, FileName, Visible)
		VALUES ('CuteMX', 'CuteMX filesharing application', 'Cutemx.exe', 1)
	INSERT INTO BlockedApplication (ApplicationName, Description, FileName, Visible)
		VALUES ('Kaza', 'Kaza filesharing application', 'Kazaalite.exe', 1)
	INSERT INTO BlockedApplication (ApplicationName, Description, FileName, Visible)
		VALUES ('Overnet Peer to Peer', 'Overnet Peer to Peer filesharing application', 'Overnet.exe', 1)
	INSERT INTO BlockedApplication (ApplicationName, Description, FileName, Visible)
		VALUES ('Joltid Peer Enabler', 'Joltid filesharing application', 'P2P Networking.exe', 1)
	INSERT INTO BlockedApplication (ApplicationName, Description, FileName, Visible)
		VALUES ('Joltid', 'Joltid adware', 'P2P Networking3.exe', 1)
	INSERT INTO BlockedApplication (ApplicationName, Description, FileName, Visible)
		VALUES ('Piolet Music Sharing', 'Piolet Music filesharing application', 'Piolet.exe', 1)
	INSERT INTO BlockedApplication (ApplicationName, Description, FileName, Visible)
		VALUES ('Warez P2P Client', 'Warez P2P filesharing application', 'Warez.exe', 1)
	INSERT INTO BlockedApplication (ApplicationName, Description, FileName, Visible)
		VALUES ('FileFreedom P2P', 'FileFreedom P2P filesharing application', 'Wtm.exe', 1)
	INSERT INTO BlockedApplication (ApplicationName, Description, FileName, Visible)
		VALUES ('Microgaming/Casino gaming software', 'Downloader for Microgaming/Casino Software', 'Dlhelper.exe', 1)
	INSERT INTO BlockedApplication (ApplicationName, Description, FileName, Visible)
		VALUES ('Valve game software', 'Valve game patching utility', 'Steam.exe', 1)
	INSERT INTO BlockedApplication (ApplicationName, Description, FileName, Visible)
		VALUES ('AOL Instant Messenger', 'AOL Instant Messenger', 'Aim.exe', 1)
	INSERT INTO BlockedApplication (ApplicationName, Description, FileName, Visible)
		VALUES ('IBM/Lotus Instant Messaging', 'IBM/Lotus Instant Messaging', 'Connect.exe', 1)
	INSERT INTO BlockedApplication (ApplicationName, Description, FileName, Visible)
		VALUES ('Digital Dreams Messenger', 'Digital Dreams Instant Messenger', 'Dgdrhe~1.exe', 1)
	INSERT INTO BlockedApplication (ApplicationName, Description, FileName, Visible)
		VALUES ('Easy Messenger', 'Easy Instant Messenger', 'Em2.exe', 1)
	INSERT INTO BlockedApplication (ApplicationName, Description, FileName, Visible)
		VALUES ('Praize Messenger', 'Praize Instant Messenger', 'itLoad.exe', 1)
	INSERT INTO BlockedApplication (ApplicationName, Description, FileName, Visible)
		VALUES ('Meca IM Client', 'Meca Instant Messenger', 'Meca.exe', 1)
	INSERT INTO BlockedApplication (ApplicationName, Description, FileName, Visible)
		VALUES ('Windows Messenger', 'Windows Instant Messenger', 'Msmsgs.exe', 1)
	INSERT INTO BlockedApplication (ApplicationName, Description, FileName, Visible)
		VALUES ('Net message', 'Net message', 'Netmsg.exe', 1)
	INSERT INTO BlockedApplication (ApplicationName, Description, FileName, Visible)
		VALUES ('Windows Pop-up Utility', 'Windows Pop-up Utility', 'Winpopup.exe', 1)
	INSERT INTO BlockedApplication (ApplicationName, Description, FileName, Visible)
		VALUES ('Yahoo Messenger', 'Yahoo Instant Messenger', 'Ypager.exe', 1)
	INSERT INTO BlockedApplication (ApplicationName, Description, FileName, Visible)
		VALUES ('Trillian', 'Trillian Instant Messenger', 'Trillian.exe', 1)
	INSERT INTO BlockedApplication (ApplicationName, Description, FileName, Visible)
		VALUES ('Ipod media playing application', 'Detects when Ipod is connected', 'Ipodwatcher.exe', 1)
	INSERT INTO BlockedApplication (ApplicationName, Description, FileName, Visible)
		VALUES ('Macvtfy', 'Macvtfy media playing application', 'Macvtfy.exe', 1)
	INSERT INTO BlockedApplication (ApplicationName, Description, FileName, Visible)
		VALUES ('MusicMatch Juikebox', 'MusicMatch Juikebox media playing application', 'mmtask.exe', 1)
	INSERT INTO BlockedApplication (ApplicationName, Description, FileName, Visible)
		VALUES ('Omgstartup', 'Omgstartup media playing application', 'Omgstartup.exe', 1)
	INSERT INTO BlockedApplication (ApplicationName, Description, FileName, Visible)
		VALUES ('Screendragon video player', 'Screendragon video player', 'Screendragon_VS_Taskbar.exe', 1)
	INSERT INTO BlockedApplication (ApplicationName, Description, FileName, Visible)
		VALUES ('SM1BG', 'SM1BG media playing application', 'SM1BG.exe', 1)
	INSERT INTO BlockedApplication (ApplicationName, Description, FileName, Visible)
		VALUES ('Windows Media Player', 'Windows Media Player', 'WMPLAYER.exe', 1)
	INSERT INTO BlockedApplication (ApplicationName, Description, FileName, Visible)
		VALUES ('Windows Media Player', 'Windows Media Player', 'Mplayer.exe', 1)
	INSERT INTO BlockedApplication (ApplicationName, Description, FileName, Visible)
		VALUES ('iTunes', 'iTunes media playing application', 'Itunes.exe', 1)
	INSERT INTO BlockedApplication (ApplicationName, Description, FileName, Visible)
		VALUES ('Active Ports', 'Active Ports network monitoring tool', 'Aports.exe', 1)
END
GO

IF EXISTS(
	SELECT *
	FROM INFORMATION_SCHEMA.TABLES
	WHERE TABLE_NAME = 'SecPatch_AffectedProduct'
	AND TABLE_SCHEMA = 'dbo'
)
AND EXISTS(
	SELECT *
	FROM INFORMATION_SCHEMA.TABLES
	WHERE TABLE_NAME = 'SecPatch_Patch'
	AND TABLE_SCHEMA = 'dbo'
)	
AND NOT EXISTS (
	SELECT *
	FROM INFORMATION_SCHEMA.CONSTRAINT_COLUMN_USAGE
	WHERE CONSTRAINT_NAME = 'FK_AffectedProduct_Patch'
)
BEGIN
	PRINT 'Create FK_AffectedProduct_Patch constraint'

	ALTER TABLE [dbo].[SecPatch_AffectedProduct]
		ADD CONSTRAINT FK_AffectedProduct_Patch
		FOREIGN KEY(PatchID)
		REFERENCES SecPatch_Patch(PatchID) ON DELETE CASCADE
END
GO
IF EXISTS(
	SELECT *
	FROM INFORMATION_SCHEMA.TABLES
	WHERE TABLE_NAME = 'SecPatch_AffectedProduct'
	AND TABLE_SCHEMA = 'dbo'
)
AND EXISTS(
	SELECT *
	FROM INFORMATION_SCHEMA.TABLES
	WHERE TABLE_NAME = 'SecPatch_Product'
	AND TABLE_SCHEMA = 'dbo'
)	
AND NOT EXISTS (
	SELECT *
	FROM INFORMATION_SCHEMA.CONSTRAINT_COLUMN_USAGE
	WHERE CONSTRAINT_NAME = 'FK_AffectedProduct_Product'
)
BEGIN
	PRINT 'Create FK_AffectedProduct_Product constraint'

	ALTER TABLE [dbo].[SecPatch_AffectedProduct]
		ADD CONSTRAINT FK_AffectedProduct_Product
		FOREIGN KEY(ProductID)
		REFERENCES SecPatch_Product(ProductID) ON DELETE CASCADE
END
GO
IF EXISTS(
	SELECT *
	FROM INFORMATION_SCHEMA.TABLES
	WHERE TABLE_NAME = 'SecPatch_AffectedServicePack'
	AND TABLE_SCHEMA = 'dbo'
)
AND EXISTS(
	SELECT *
	FROM INFORMATION_SCHEMA.TABLES
	WHERE TABLE_NAME = 'SecPatch_ServicePack'
	AND TABLE_SCHEMA = 'dbo'
)	
AND NOT EXISTS (
	SELECT *
	FROM INFORMATION_SCHEMA.CONSTRAINT_COLUMN_USAGE
	WHERE CONSTRAINT_NAME = 'FK_AffectedServicePack_ServicePack'
)
BEGIN
	PRINT 'Create FK_AffectedServicePack_ServicePack constraint'

	ALTER TABLE [dbo].[SecPatch_AffectedServicePack]
		ADD CONSTRAINT FK_AffectedServicePack_ServicePack
		FOREIGN KEY(ServicePackID)
		REFERENCES SecPatch_ServicePack(ServicePackID) ON DELETE CASCADE
END
GO
IF EXISTS(
	SELECT *
	FROM INFORMATION_SCHEMA.TABLES
	WHERE TABLE_NAME = 'SecPatch_BulletinComment'
	AND TABLE_SCHEMA = 'dbo'
)
AND EXISTS(
	SELECT *
	FROM INFORMATION_SCHEMA.TABLES
	WHERE TABLE_NAME = 'SecPatch_Bulletin'
	AND TABLE_SCHEMA = 'dbo'
)	
AND NOT EXISTS (
	SELECT *
	FROM INFORMATION_SCHEMA.CONSTRAINT_COLUMN_USAGE
	WHERE CONSTRAINT_NAME = 'FK_BulletinComment_Bulletin'
)
BEGIN
	PRINT 'Create FK_BulletinComment_Bulletin constraint'

	ALTER TABLE [dbo].[SecPatch_BulletinComment]
		ADD CONSTRAINT FK_BulletinComment_Bulletin
		FOREIGN KEY(BulletinID)
		REFERENCES SecPatch_Bulletin(BulletinID) ON DELETE CASCADE
END
GO
IF EXISTS(
	SELECT *
	FROM INFORMATION_SCHEMA.TABLES
	WHERE TABLE_NAME = 'SecPatch_QNumber'
	AND TABLE_SCHEMA = 'dbo'
)
AND EXISTS(
	SELECT *
	FROM INFORMATION_SCHEMA.TABLES
	WHERE TABLE_NAME = 'SecPatch_Bulletin'
	AND TABLE_SCHEMA = 'dbo'
)	
AND NOT EXISTS (
	SELECT *
	FROM INFORMATION_SCHEMA.CONSTRAINT_COLUMN_USAGE
	WHERE CONSTRAINT_NAME = 'FK_qNumber_Bulletin'
)
BEGIN
	PRINT 'Create FK_qNumber_Bulletin constraint'

	ALTER TABLE [dbo].[SecPatch_QNumber]
		ADD CONSTRAINT FK_qNumber_Bulletin
		FOREIGN KEY(BulletinID)
		REFERENCES SecPatch_Bulletin(BulletinID) ON DELETE CASCADE
END
GO
IF EXISTS(
	SELECT *
	FROM INFORMATION_SCHEMA.TABLES
	WHERE TABLE_NAME = 'SecPatch_Patch'
	AND TABLE_SCHEMA = 'dbo'
)
AND EXISTS(
	SELECT *
	FROM INFORMATION_SCHEMA.TABLES
	WHERE TABLE_NAME = 'SecPatch_Bulletin'
	AND TABLE_SCHEMA = 'dbo'
)	
AND NOT EXISTS (
	SELECT *
	FROM INFORMATION_SCHEMA.CONSTRAINT_COLUMN_USAGE
	WHERE CONSTRAINT_NAME = 'FK_Patch_Bulletin'
)
BEGIN
	PRINT 'Create FK_Patch_Bulletin constraint'

	ALTER TABLE [dbo].[SecPatch_Patch]
		ADD CONSTRAINT FK_Patch_Bulletin
		FOREIGN KEY(BulletinID)
		REFERENCES SecPatch_Bulletin(BulletinID) ON DELETE CASCADE
END
GO
IF EXISTS(
	SELECT *
	FROM INFORMATION_SCHEMA.TABLES
	WHERE TABLE_NAME = 'SecPatch_Patch'
	AND TABLE_SCHEMA = 'dbo'
)
AND EXISTS(
	SELECT *
	FROM INFORMATION_SCHEMA.TABLES
	WHERE TABLE_NAME = 'SecurityPatch'
	AND TABLE_SCHEMA = 'dbo'
)	
AND NOT EXISTS (
	SELECT *
	FROM INFORMATION_SCHEMA.CONSTRAINT_COLUMN_USAGE
	WHERE CONSTRAINT_NAME = 'FK_Patch_SecurityPatch'
)
BEGIN
	PRINT 'Create FK_Patch_SecurityPatch constraint'

	ALTER TABLE [dbo].[SecPatch_Patch]
		ADD CONSTRAINT FK_Patch_SecurityPatch
		FOREIGN KEY(SecurityPatchID)
		REFERENCES SecurityPatch(PatchID) ON DELETE CASCADE
END
GO
IF EXISTS(
	SELECT *
	FROM INFORMATION_SCHEMA.TABLES
	WHERE TABLE_NAME = 'SecurityBulletin'
	AND TABLE_SCHEMA = 'dbo'
)
AND EXISTS(
	SELECT *
	FROM INFORMATION_SCHEMA.TABLES
	WHERE TABLE_NAME = 'SecurityState'
	AND TABLE_SCHEMA = 'dbo'
)	
AND NOT EXISTS (
	SELECT *
	FROM INFORMATION_SCHEMA.CONSTRAINT_COLUMN_USAGE
	WHERE CONSTRAINT_NAME = 'FK_SecurityBulletin_SecurityState'
)
BEGIN
	PRINT 'Create FK_SecurityBulletin_SecurityState constraint'

	ALTER TABLE [dbo].[SecurityBulletin]
		ADD CONSTRAINT FK_SecurityBulletin_SecurityState
		FOREIGN KEY(State)
		REFERENCES SecurityState(StateID) ON DELETE NO ACTION
END
GO
IF EXISTS(
	SELECT *
	FROM INFORMATION_SCHEMA.TABLES
	WHERE TABLE_NAME = 'SecurityBulletin'
	AND TABLE_SCHEMA = 'dbo'
)
AND EXISTS(
	SELECT *
	FROM INFORMATION_SCHEMA.TABLES
	WHERE TABLE_NAME = 'PackageVersion'
	AND TABLE_SCHEMA = 'dbo'
)	
AND NOT EXISTS (
	SELECT *
	FROM INFORMATION_SCHEMA.CONSTRAINT_COLUMN_USAGE
	WHERE CONSTRAINT_NAME = 'FK_SecurityBulletin_PackageVersion'
)
BEGIN
	PRINT 'Create FK_SecurityBulletin_PackageVersion constraint'

	ALTER TABLE [dbo].[SecurityBulletin]
		ADD CONSTRAINT FK_SecurityBulletin_PackageVersion
		FOREIGN KEY(PackageVersionID)
		REFERENCES PackageVersion(PackageVersionID) ON DELETE CASCADE
END
GO
IF EXISTS(
	SELECT *
	FROM INFORMATION_SCHEMA.TABLES
	WHERE TABLE_NAME = 'SecurityBulletinToUserState'
	AND TABLE_SCHEMA = 'dbo'
)
AND EXISTS(
	SELECT *
	FROM INFORMATION_SCHEMA.TABLES
	WHERE TABLE_NAME = 'SecurityUserState'
	AND TABLE_SCHEMA = 'dbo'
)	
AND NOT EXISTS (
	SELECT *
	FROM INFORMATION_SCHEMA.CONSTRAINT_COLUMN_USAGE
	WHERE CONSTRAINT_NAME = 'FK_SecurityBulletinToUserState_SecurityUserState'
)
BEGIN
	PRINT 'Create FK_SecurityBulletinToUserState_SecurityUserState constraint'

	ALTER TABLE [dbo].[SecurityBulletinToUserState]
		ADD CONSTRAINT FK_SecurityBulletinToUserState_SecurityUserState
		FOREIGN KEY(UserStateID)
		REFERENCES SecurityUserState(UserStateID) ON DELETE NO ACTION
END
GO
IF EXISTS(
	SELECT *
	FROM INFORMATION_SCHEMA.TABLES
	WHERE TABLE_NAME = 'SecurityPatch'
	AND TABLE_SCHEMA = 'dbo'
)
AND EXISTS(
	SELECT *
	FROM INFORMATION_SCHEMA.TABLES
	WHERE TABLE_NAME = 'SecurityState'
	AND TABLE_SCHEMA = 'dbo'
)	
AND NOT EXISTS (
	SELECT *
	FROM INFORMATION_SCHEMA.CONSTRAINT_COLUMN_USAGE
	WHERE CONSTRAINT_NAME = 'FK_SecurityPatch_SecurityState'
)
BEGIN
	PRINT 'Create FK_SecurityPatch_SecurityState constraint'

	ALTER TABLE [dbo].[SecurityPatch]
		ADD CONSTRAINT FK_SecurityPatch_SecurityState
		FOREIGN KEY(State)
		REFERENCES SecurityState(StateID) ON DELETE NO ACTION
END
GO
IF EXISTS(
	SELECT *
	FROM INFORMATION_SCHEMA.TABLES
	WHERE TABLE_NAME = 'SecurityAnalysisReport'
	AND TABLE_SCHEMA = 'dbo'
)
AND EXISTS(
	SELECT *
	FROM INFORMATION_SCHEMA.TABLES
	WHERE TABLE_NAME = 'Computer'
	AND TABLE_SCHEMA = 'dbo'
)	
AND NOT EXISTS (
	SELECT *
	FROM INFORMATION_SCHEMA.CONSTRAINT_COLUMN_USAGE
	WHERE CONSTRAINT_NAME = 'FK_SecurityAnalysisReport_Computer'
)
BEGIN
	PRINT 'Create FK_SecurityAnalysisReport_Computer constraint'

	ALTER TABLE [dbo].[SecurityAnalysisReport]
		ADD CONSTRAINT FK_SecurityAnalysisReport_Computer
		FOREIGN KEY(ComputerID)
		REFERENCES Computer(ComputerID) ON DELETE CASCADE
END
GO
IF EXISTS(
	SELECT *
	FROM INFORMATION_SCHEMA.TABLES
	WHERE TABLE_NAME = 'SecurityPatchAnalysis'
	AND TABLE_SCHEMA = 'dbo'
)
AND EXISTS(
	SELECT *
	FROM INFORMATION_SCHEMA.TABLES
	WHERE TABLE_NAME = 'SecurityPatchStatus'
	AND TABLE_SCHEMA = 'dbo'
)	
AND NOT EXISTS (
	SELECT *
	FROM INFORMATION_SCHEMA.CONSTRAINT_COLUMN_USAGE
	WHERE CONSTRAINT_NAME = 'FK_SecurityPatchAnalysis_SecurityPatchStatus'
)
BEGIN
	PRINT 'Create FK_SecurityPatchAnalysis_SecurityPatchStatus constraint'

	ALTER TABLE [dbo].[SecurityPatchAnalysis]
		ADD CONSTRAINT FK_SecurityPatchAnalysis_SecurityPatchStatus
		FOREIGN KEY(Status)
		REFERENCES SecurityPatchStatus(SecurityPatchStatusID) ON DELETE NO ACTION
END
GO
IF EXISTS(
	SELECT *
	FROM INFORMATION_SCHEMA.TABLES
	WHERE TABLE_NAME = 'SecurityPatchAnalysis'
	AND TABLE_SCHEMA = 'dbo'
)
AND EXISTS(
	SELECT *
	FROM INFORMATION_SCHEMA.TABLES
	WHERE TABLE_NAME = 'Computer'
	AND TABLE_SCHEMA = 'dbo'
)	
AND NOT EXISTS (
	SELECT *
	FROM INFORMATION_SCHEMA.CONSTRAINT_COLUMN_USAGE
	WHERE CONSTRAINT_NAME = 'FK_SecurityPatchAnalysis_Computer'
)
BEGIN
	PRINT 'Create FK_SecurityPatchAnalysis_Computer constraint'

	ALTER TABLE [dbo].[SecurityPatchAnalysis]
		ADD CONSTRAINT FK_SecurityPatchAnalysis_Computer
		FOREIGN KEY(ComputerID)
		REFERENCES Computer(ComputerID) ON DELETE CASCADE
END
GO
IF EXISTS(
	SELECT *
	FROM INFORMATION_SCHEMA.TABLES
	WHERE TABLE_NAME = 'SecurityPatchResource'
	AND TABLE_SCHEMA = 'dbo'
)
AND EXISTS(
	SELECT *
	FROM INFORMATION_SCHEMA.TABLES
	WHERE TABLE_NAME = 'Architecture'
	AND TABLE_SCHEMA = 'dbo'
)	
AND NOT EXISTS (
	SELECT *
	FROM INFORMATION_SCHEMA.CONSTRAINT_COLUMN_USAGE
	WHERE CONSTRAINT_NAME = 'FK_SecurityPatchResource_Architecture'
)
BEGIN
	PRINT 'Create FK_SecurityPatchResource_Architecture constraint'

	ALTER TABLE [dbo].[SecurityPatchResource]
		ADD CONSTRAINT FK_SecurityPatchResource_Architecture
		FOREIGN KEY(ArchitectureID)
		REFERENCES Architecture(ArchitectureID) ON DELETE CASCADE
END
GO
IF EXISTS(
	SELECT *
	FROM INFORMATION_SCHEMA.TABLES
	WHERE TABLE_NAME = 'SecurityPatchResource'
	AND TABLE_SCHEMA = 'dbo'
)
AND EXISTS(
	SELECT *
	FROM INFORMATION_SCHEMA.TABLES
	WHERE TABLE_NAME = 'Locale'
	AND TABLE_SCHEMA = 'dbo'
)	
AND NOT EXISTS (
	SELECT *
	FROM INFORMATION_SCHEMA.CONSTRAINT_COLUMN_USAGE
	WHERE CONSTRAINT_NAME = 'FK_SecurityPatchResource_Locale'
)
BEGIN
	PRINT 'Create FK_SecurityPatchResource_Locale constraint'

	ALTER TABLE [dbo].[SecurityPatchResource]
		ADD CONSTRAINT FK_SecurityPatchResource_Locale
		FOREIGN KEY(LanguageID)
		REFERENCES Locale(LocaleCode) ON DELETE CASCADE
END
GO
IF EXISTS(
	SELECT *
	FROM INFORMATION_SCHEMA.TABLES
	WHERE TABLE_NAME = 'SecurityPatchLanguage'
	AND TABLE_SCHEMA = 'dbo'
)
AND EXISTS(
	SELECT *
	FROM INFORMATION_SCHEMA.TABLES
	WHERE TABLE_NAME = 'Locale'
	AND TABLE_SCHEMA = 'dbo'
)	
AND NOT EXISTS (
	SELECT *
	FROM INFORMATION_SCHEMA.CONSTRAINT_COLUMN_USAGE
	WHERE CONSTRAINT_NAME = 'FK_SecurityPatchLanguage_Locale'
)
BEGIN
	PRINT 'Create FK_SecurityPatchLanguage_Locale constraint'

	ALTER TABLE [dbo].[SecurityPatchLanguage]
		ADD CONSTRAINT FK_SecurityPatchLanguage_Locale
		FOREIGN KEY(LanguageName)
		REFERENCES Locale(LocaleCode) ON DELETE CASCADE
END
GO
IF EXISTS(
	SELECT *
	FROM INFORMATION_SCHEMA.TABLES
	WHERE TABLE_NAME = 'SecurityPatchArchitecture'
	AND TABLE_SCHEMA = 'dbo'
)
AND EXISTS(
	SELECT *
	FROM INFORMATION_SCHEMA.TABLES
	WHERE TABLE_NAME = 'Architecture'
	AND TABLE_SCHEMA = 'dbo'
)	
AND NOT EXISTS (
	SELECT *
	FROM INFORMATION_SCHEMA.CONSTRAINT_COLUMN_USAGE
	WHERE CONSTRAINT_NAME = 'FK_SecurityPatchApplication_Architecture'
)
BEGIN
	PRINT 'Create FK_SecurityPatchApplication_Architecture constraint'

	ALTER TABLE [dbo].[SecurityPatchArchitecture]
		ADD CONSTRAINT FK_SecurityPatchApplication_Architecture
		FOREIGN KEY(ArchitectureID)
		REFERENCES Architecture(ArchitectureID) ON DELETE CASCADE
END
GO
IF EXISTS(
	SELECT *
	FROM INFORMATION_SCHEMA.TABLES
	WHERE TABLE_NAME = 'SecurityBulletinChange'
	AND TABLE_SCHEMA = 'dbo'
)
AND EXISTS(
	SELECT *
	FROM INFORMATION_SCHEMA.TABLES
	WHERE TABLE_NAME = 'SecurityBulletinChangeType'
	AND TABLE_SCHEMA = 'dbo'
)	
AND NOT EXISTS (
	SELECT *
	FROM INFORMATION_SCHEMA.CONSTRAINT_COLUMN_USAGE
	WHERE CONSTRAINT_NAME = 'FK_SecurityBulletinChange_SecurityBulletinChangeType'
)
BEGIN
	PRINT 'Create FK_SecurityBulletinChange_SecurityBulletinChangeType constraint'

	ALTER TABLE [dbo].[SecurityBulletinChange]
		ADD CONSTRAINT FK_SecurityBulletinChange_SecurityBulletinChangeType
		FOREIGN KEY(ChangeTypeID)
		REFERENCES SecurityBulletinChangeType(SecurityBulletinChangeTypeID) ON DELETE CASCADE
END
GO
IF EXISTS(
	SELECT *
	FROM INFORMATION_SCHEMA.TABLES
	WHERE TABLE_NAME = 'ThreatScan'
	AND TABLE_SCHEMA = 'dbo'
)
AND EXISTS(
	SELECT *
	FROM INFORMATION_SCHEMA.TABLES
	WHERE TABLE_NAME = 'Computer'
	AND TABLE_SCHEMA = 'dbo'
)	
AND NOT EXISTS (
	SELECT *
	FROM INFORMATION_SCHEMA.CONSTRAINT_COLUMN_USAGE
	WHERE CONSTRAINT_NAME = 'FK_ThreatScan_Computer'
)
BEGIN
	PRINT 'Create FK_ThreatScan_Computer constraint'

	ALTER TABLE [dbo].[ThreatScan]
		ADD CONSTRAINT FK_ThreatScan_Computer
		FOREIGN KEY(ComputerID)
		REFERENCES Computer(ComputerID) ON DELETE CASCADE
END
GO
IF EXISTS(
	SELECT *
	FROM INFORMATION_SCHEMA.TABLES
	WHERE TABLE_NAME = 'ThreatScan'
	AND TABLE_SCHEMA = 'dbo'
)
AND EXISTS(
	SELECT *
	FROM INFORMATION_SCHEMA.TABLES
	WHERE TABLE_NAME = 'User'
	AND TABLE_SCHEMA = 'dbo'
)	
AND NOT EXISTS (
	SELECT *
	FROM INFORMATION_SCHEMA.CONSTRAINT_COLUMN_USAGE
	WHERE CONSTRAINT_NAME = 'FK_ThreatScan_User'
)
BEGIN
	PRINT 'Create FK_ThreatScan_User constraint'

	ALTER TABLE [dbo].[ThreatScan]
		ADD CONSTRAINT FK_ThreatScan_User
		FOREIGN KEY(UserID)
		REFERENCES [User](UserID) ON DELETE CASCADE
END
GO
IF EXISTS(
	SELECT *
	FROM INFORMATION_SCHEMA.TABLES
	WHERE TABLE_NAME = 'ThreatOccurrence'
	AND TABLE_SCHEMA = 'dbo'
)
AND EXISTS(
	SELECT *
	FROM INFORMATION_SCHEMA.TABLES
	WHERE TABLE_NAME = 'Computer'
	AND TABLE_SCHEMA = 'dbo'
)	
AND NOT EXISTS (
	SELECT *
	FROM INFORMATION_SCHEMA.CONSTRAINT_COLUMN_USAGE
	WHERE CONSTRAINT_NAME = 'FK_ThreatOccurrence_Computer'
)
BEGIN
	PRINT 'Create FK_ThreatOccurrence_Computer constraint'

	ALTER TABLE [dbo].[ThreatOccurrence]
		ADD CONSTRAINT FK_ThreatOccurrence_Computer
		FOREIGN KEY(ComputerID)
		REFERENCES Computer(ComputerID) ON DELETE CASCADE
END
GO
IF EXISTS(
	SELECT *
	FROM INFORMATION_SCHEMA.TABLES
	WHERE TABLE_NAME = 'ThreatOccurrence'
	AND TABLE_SCHEMA = 'dbo'
)
AND EXISTS(
	SELECT *
	FROM INFORMATION_SCHEMA.TABLES
	WHERE TABLE_NAME = 'User'
	AND TABLE_SCHEMA = 'dbo'
)	
AND NOT EXISTS (
	SELECT *
	FROM INFORMATION_SCHEMA.CONSTRAINT_COLUMN_USAGE
	WHERE CONSTRAINT_NAME = 'FK_ThreatOccurrence_User'
)
BEGIN
	PRINT 'Create FK_ThreatOccurrence_User constraint'

	ALTER TABLE [dbo].[ThreatOccurrence]
		ADD CONSTRAINT FK_ThreatOccurrence_User
		FOREIGN KEY(UserID)
		REFERENCES [User](UserID) ON DELETE CASCADE
END
GO
